=== modified file '.bzrignore'
--- .bzrignore	2014-12-16 17:54:15 +0000
+++ .bzrignore	2017-02-10 10:33:11 +0000
@@ -5,4 +5,4 @@
 ./Makefile
 __pycache__
 build/
-
+unicode_data/

=== modified file 'Onboard/Appearance.py'
--- Onboard/Appearance.py	2016-08-15 15:08:44 +0000
+++ Onboard/Appearance.py	2017-02-10 10:33:11 +0000
@@ -467,7 +467,7 @@
                 return True
         return False
 
-    def get_key_rgba(self, key, element, state = None):
+    def get_key_rgba(self, key, element, state=None):
         """
         Get the color for the given key element and optionally key state.
         If <state> is None the key state is retrieved from <key>.
@@ -475,8 +475,9 @@
 
         if state is None:
             state = key.get_state()
-            state["insensitive"] = not key.sensitive
-            del state["sensitive"]
+            if "insensitive" in state:
+                state["insensitive"] = not key.sensitive
+                del state["sensitive"]
 
         rgb = None
         opacity = None
@@ -490,17 +491,18 @@
         # Let numbered keys fall back to their base id, e.g. instead
         # of prediction0, prediction1,... have only "prediction" in
         # the color scheme.
-        if key.id == "correctionsbg":
-            ids.append("wordlist")
-        elif key.id == "predictionsbg":
-            ids.append("wordlist")
-        elif key.is_prediction_key():
-            ids.append("prediction")
-        elif key.is_correction_key():
-            ids.append("correction")
-        elif key.is_layer_button():
-            ids.append(key.get_similar_theme_id("layer"))
-            ids.append("layer")
+        if key.is_key():  # can be a DrawingItem too
+            if key.id == "correctionsbg":
+                ids.append("wordlist")
+            elif key.id == "predictionsbg":
+                ids.append("wordlist")
+            elif key.is_prediction_key():
+                ids.append("prediction")
+            elif key.is_correction_key():
+                ids.append("correction")
+            elif key.is_layer_button():
+                ids.append(key.get_similar_theme_id("layer"))
+                ids.append("layer")
 
         # look for a matching key_group and color in the color scheme
         for id in ids:
@@ -519,8 +521,9 @@
         # Special case for layer buttons:
         # don't take fill color from the root group,
         # we want the layer fill color instead (via get_key_default_rgba()).
-        if element == "fill" and key.is_layer_button() or \
-           element == "label" and key.is_correction_key():
+        if key.is_key() and \
+           (element == "fill" and key.is_layer_button() or \
+            element == "label" and key.is_correction_key()):
             # Don't pick layer fill opacity when there is
             # an rgb color defined in the color scheme.
             if not rgb is None and \
@@ -560,7 +563,8 @@
         rgba = [0.0, 0.0, 0.0, 1.0]
 
         if element == "fill":
-            if key.is_layer_button() and \
+            if key.is_key() and \
+               key.is_layer_button() and \
                not any(state.values()):
                 # Special case for base fill color of layer buttons:
                 # default color is layer fill color (as in onboard <=0.95).

=== added file 'Onboard/CharacterPalette.py'
--- Onboard/CharacterPalette.py	1970-01-01 00:00:00 +0000
+++ Onboard/CharacterPalette.py	2017-02-10 10:33:11 +0000
@@ -0,0 +1,248 @@
+# -*- coding: utf-8 -*-
+
+# Copyright © 2017 marmuta <marmvta@gmail.com>
+#
+# This file is part of Onboard.
+#
+# Onboard is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 3 of the License, or
+# (at your option) any later version.
+#
+# Onboard is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program. If not, see <http://www.gnu.org/licenses/>.
+
+from __future__ import division, print_function, unicode_literals
+
+import logging
+_logger = logging.getLogger(__name__)
+
+from Onboard.Layout            import (LayoutPanel, ScrolledLayoutPanel,
+                                       RectangleItem)
+from Onboard.KeyGtk            import FlatKey
+from Onboard                   import KeyCommon
+from Onboard.UnicodeData       import UnicodeData
+
+
+class CharacterPaletteKey(FlatKey):
+    pass
+
+
+class PaletteHeaderKey(FlatKey):
+    palette_panel = None
+
+    def get_style(self):
+        # No gradient when active to match the background rectangle.
+        if self.is_active_only():
+            return "flat"
+        return super(PaletteHeaderKey, self).get_style()
+
+    def get_fill_color(self):
+        # Show active state with the inactive fill color.
+        # In inactive state the key is transparent (FlatKey).
+        state = {} if self.is_active_only() else None
+        return self.get_color("fill", state)
+
+    def build_rect_path(self, context, rect):
+        # Only round top corners.
+        r = rect.copy()
+        r.h += 1
+        self.build_rect_path_custom(context, r, 0b0011)
+
+    def on_release(self, view, button, event_type):
+        self.palette_panel.set_active_category_index(self.code)
+
+
+class CharacterPaletteBackground(RectangleItem):
+    pass
+
+
+class CharacterGridPanel(ScrolledLayoutPanel):
+    characters = ""
+    key_rect = None
+    key_group = None
+    color_scheme = None
+    background_rgba = None
+
+    def update_log_rect(self):
+        self.set_clip_rect(self.get_canvas_border_rect())
+        super(CharacterGridPanel, self).update_log_rect()
+
+    def update_content(self):
+        # create content
+        keys = []
+        if self.characters:
+            keys, r = self._create_content(self.get_border_rect(),
+                                           self.key_rect,
+                                           self.key_group,
+                                           self.color_scheme,
+                                           self.characters)
+
+        item = CharacterPaletteBackground()
+        item.set_border_rect(self.get_border_rect())
+        item.color_scheme = self.color_scheme
+
+        self.set_items([item] + keys)
+
+    @staticmethod
+    def _create_content(grid_rect, key_rect, key_group,
+                        color_scheme, sequence):
+        spacing = (0, 0)
+
+        key_rects, bounds = grid_rect.flow_layout(key_rect, len(sequence),
+                                                  *spacing, True)
+        keys = []
+
+        for i, label in enumerate(sequence):
+            id = "_palette_character" + str(i)
+            key = CharacterPaletteKey()
+            key.type = KeyCommon.CHAR_TYPE
+            key.labels = {0: label}
+            key.code  = label
+            # generate key on key_up and allow drag selection
+            key.action = KeyCommon.DELAYED_STROKE_ACTION
+            key.set_border_rect(key_rects[i])
+            if len(label) <= 2:
+                key.group = key_group
+            else:
+                key.group = id
+            key.color_scheme = color_scheme
+            keys.append(key)
+
+        return keys, bounds
+
+
+class CharacterPalettePanel(LayoutPanel):
+    """ Emoji/symbol palette """
+
+    def __init__(self):
+        LayoutPanel.__init__(self)
+        self.keyboard = None
+        self._unicode_data = UnicodeData()
+        self._character_grid = None
+        self._header_keys = None
+        self._active_category_index = 0
+        self._character_grid = None
+
+    def update_log_rect(self):
+        self.update_content()
+        super(CharacterPalettePanel, self).update_log_rect()
+
+    def update_content(self):
+        if self._header_keys is None:
+            self.create_keys()
+
+    def create_keys(self):
+        background = self.find_id("character-palette")
+        header_template = self.find_id("palette-header-template")
+        key_template = self.find_id("palette-key-template")
+        keys = []
+
+        if not background or \
+           not header_template or \
+           not key_template:
+            return []
+
+        color_scheme = key_template.color_scheme
+        remaining_rect = background.get_border_rect().copy()
+
+        # create header keys
+        headers = self.get_header_labels()
+        ks, r = self._create_header_keys(
+            remaining_rect,
+            header_template.get_border_rect(),
+            color_scheme, headers)
+        keys += ks
+        remaining_rect.y += r.h
+        remaining_rect.h -= r.h
+
+        # create scrolled panel with grid of keys
+        item = CharacterGridPanel()
+        item.set_id("_character_grid")
+        item.set_border_rect(remaining_rect)
+        item.characters = ""
+        item.key_rect = key_template.get_border_rect()
+        item.key_group = "_" + self.content_type
+        item.color_scheme = color_scheme
+
+        self.set_items([background, header_template,
+                        key_template, item] + keys)
+
+        self._character_grid = item
+        self._header_keys = keys
+
+        self.set_active_category_index(0)
+
+    def set_active_category_index(self, index):
+        self._active_category_index = index
+
+        # update header keys
+        keys_to_redraw = []
+        for key in self._header_keys:
+            active = (key.code == self._active_category_index)
+            if key.active != active:
+                key.active = active
+                keys_to_redraw.append(key)
+
+        self._character_grid.characters = \
+            self.get_grid_labels(self._active_category_index)
+
+        self._character_grid.update_content()
+
+        self.keyboard.layout.invalidate_font_sizes()
+        self.keyboard.layout.invalidate_caches()
+
+        self.keyboard.redraw(keys_to_redraw + [self._character_grid])
+
+    def _create_header_keys(self, palette_rect, header_key_rect,
+                            color_scheme, sequence):
+        spacing = (0, 0)
+
+        header_rect = palette_rect.copy()
+        header_rect.h = header_key_rect.h
+        key_rects = header_rect.subdivide(len(sequence), 1, *spacing)
+
+        keys = []
+        for i, label in enumerate(sequence):
+            key = PaletteHeaderKey("_palette_header" + str(i))
+            key.type = KeyCommon.BUTTON_TYPE
+            key.labels = {0: label}
+            key.code  = i
+            key.set_border_rect(key_rects[i])
+            key.group = "_palette_header"
+            key.color_scheme = color_scheme
+            key.unlatch_layer = False
+            key.palette_panel = self
+            keys.append(key)
+
+        return keys, header_rect
+
+
+class EmojiPalettePanel(CharacterPalettePanel):
+    """ Emoji palette """
+
+    content_type = "emoji"
+
+    def get_header_labels(self):
+        return self._unicode_data.get_emoji_categories()  # + ["⭐"]
+
+    def get_grid_labels(self, category):
+        return self._unicode_data.get_emoji(category)
+
+
+class SymbolPalettePanel(CharacterPalettePanel):
+    """ Emoji palette """
+
+    content_type = "symbols"
+
+    def get_header_labels(self):
+        return self._unicode_data.get_symbol_categories()
+
+    def get_grid_labels(self, category):
+        return self._unicode_data.get_symbols(category)
+

=== modified file 'Onboard/KeyCommon.py'
--- Onboard/KeyCommon.py	2017-02-08 11:38:12 +0000
+++ Onboard/KeyCommon.py	2017-02-10 10:33:11 +0000
@@ -33,7 +33,7 @@
 from Onboard.utils import Rect, LABEL_MODIFIERS, Modifiers, \
                           polygon_to_rounded_path
 
-from Onboard.Layout import LayoutItem
+from Onboard.Layout import DrawingItem
 
 ### Logging ###
 import logging
@@ -168,20 +168,12 @@
     NORMAL = 0
     ACTIVE = 1
 
-class KeyCommon(LayoutItem):
+class KeyCommon(DrawingItem):
     """
     library-independent key class. Specific rendering options
     are stored elsewhere.
     """
 
-    # extended id for key specific theme tweaks
-    # e.g. theme_id=DELE.numpad (with id=DELE)
-    theme_id = None
-
-    # extended id for layout specific tweaks
-    # e.g. "hide.wordlist", for hide button in wordlist mode
-    svg_id = None
-
     # optional id of a sublayout used as long-press popup
     popup_id = None
 
@@ -260,7 +252,7 @@
 ###################
 
     def __init__(self):
-        LayoutItem.__init__(self)
+        super(KeyCommon, self).__init__()
 
     def configure_label(self, mod_mask):
         SHIFT = Modifiers.SHIFT
@@ -342,56 +334,6 @@
     def is_active(self):
         return not self.type is None
 
-    def get_id(self):
-        return ""
-
-    def get_svg_id(self):
-        return ""
-
-    def set_id(self, id, theme_id = None, svg_id = None):
-        self.theme_id, self.id = self.parse_id(id)
-        if theme_id:
-            self.theme_id = theme_id
-        self.svg_id = self.id if not svg_id else svg_id
-
-    @staticmethod
-    def parse_id(value):
-        """
-        The theme id has the form <id>.<arbitrary identifier>, where
-        the identifier should be a description of the location of
-        the key relative to its surroundings, e.g. 'DELE.next-to-backspace'.
-        Don't use layout names or layer ids for the theme id, they lose
-        their meaning when layouts are copied or renamed by users.
-        """
-        theme_id = value
-        id = value.split(".")[0]
-        return theme_id, id
-
-    @staticmethod
-    def split_theme_id(theme_id):
-        """
-        Simple split in prefix (id) before the dot and suffix after the dot.
-        """
-        components = theme_id.split(".")
-        if len(components) == 1:
-            return components[0], ""
-        return components[0], components[1]
-
-    @staticmethod
-    def build_theme_id(prefix, postfix):
-        if postfix:
-            return prefix + "." + postfix
-        return prefix
-
-    def get_similar_theme_id(self, prefix = None):
-        if prefix is None:
-            prefix = self.id
-        theme_id = prefix
-        comps = self.theme_id.split(".")[1:]
-        if comps:
-            theme_id += "." + comps[0]
-        return theme_id
-
     def is_layer_button(self):
         return bool(self.target_layer_id) or self.id.startswith("layer")
 
@@ -419,14 +361,20 @@
                            "middleclick",
                            "doubleclick",
                            "dragclick"]
+
     def is_button(self):
         return self.type == BUTTON_TYPE
 
     def is_pressed_only(self):
-        return self.pressed and not (self.active or \
-                                     self.locked or \
+        return self.pressed and not (self.active or
+                                     self.locked or
                                      self.scanned)
 
+    def is_active_only(self):
+        return self.active and not (self.pressed or
+                                    self.locked or
+                                    self.scanned)
+
     def is_text_changing(self):
         if not self.is_modifier() and \
                self.type in [KEYCODE_TYPE,
@@ -478,6 +426,18 @@
     def get_target_layer_id(self):
         return self.target_layer_id
 
+    def get_target_layer_parent_id(self):
+        """
+        Split off the parent path of the (dot-separated)
+        target_layer_id.
+        """
+        layer_id = self.target_layer_id
+        pos = layer_id.rfind(".")
+        if pos >= 0:
+            return layer_id[:pos]
+        else:
+            return None
+
     def get_popup_layout(self):
         if self.popup_id:
             return self.find_sublayout(self.popup_id)
@@ -517,13 +477,9 @@
     def __init__(self, id, border_rect):
         KeyCommon.__init__(self)
         self.id = id
-        self.colors = {}
         self.context.log_rect = border_rect \
                                 if not border_rect is None else Rect()
 
-    def get_id(self):
-        return self.id
-
     def get_svg_id(self):
         return self.svg_id
 
@@ -584,43 +540,23 @@
     def get_light_direction(self):
         return config.theme_settings.key_gradient_direction * pi / 180.0
 
-    def get_fill_color(self):
-        return self._get_color("fill")
-
-    def get_stroke_color(self):
-        return self._get_color("stroke")
-
-    def get_label_color(self):
-        return self._get_color("label")
-
-    def get_secondary_label_color(self):
-        return self._get_color("secondary-label")
-
-    def get_dwell_progress_color(self):
-        return self._get_color("dwell-progress")
-
     def get_dwell_progress_canvas_rect(self):
         rect = self.get_label_rect().inflate(0.5)
         return self.context.log_to_canvas_rect(rect)
 
-    def _get_color(self, element):
-        color_key = (element, self.prelight, self.pressed,
-                              self.active, self.locked,
-                              self.sensitive, self.scanned)
-        rgba = self.colors.get(color_key)
-        if not rgba:
-            if self.color_scheme:
-                rgba = self.color_scheme.get_key_rgba(self, element)
-            elif element == "label":
-                rgba = [0.0, 0.0, 0.0, 1.0]
-            else:
-                rgba = [1.0, 1.0, 1.0, 1.0]
-            self.colors[color_key] = rgba
-        return rgba
+    def get_color(self, element, state = None):
+        color_key = (element,
+                     self.prelight, self.pressed,
+                     self.active, self.locked,
+                     self.sensitive, self.scanned)
+        try:
+            return self.colors[color_key]
+        except KeyError:
+            return self.cache_color(element, color_key, state)
 
     def get_fullsize_rect(self):
         """ Get bounding box of the key at 100% size in logical coordinates """
-        return LayoutItem.get_rect(self)
+        return DrawingItem.get_rect(self)
 
     def get_canvas_fullsize_rect(self):
         """ Get bounding box of the key at 100% size in canvas coordinates """

=== modified file 'Onboard/KeyGtk.py'
--- Onboard/KeyGtk.py	2016-08-15 15:08:44 +0000
+++ Onboard/KeyGtk.py	2017-02-10 10:33:11 +0000
@@ -31,12 +31,15 @@
 require_gi_versions()
 from gi.repository import GLib, Gdk, Pango, PangoCairo, GdkPixbuf
 
+from Onboard.KeyCommon   import (KeyCommon, RectKeyCommon,
+                                 LOD, InputlineKeyCommon)
 from Onboard.KeyCommon   import *
 from Onboard.WindowUtils import DwellProgress
-from Onboard.utils       import brighten, unicode_str, \
-                                gradient_line, drop_shadow, \
-                                roundrect_curve, rounded_path, \
-                                rounded_polygon_path_to_cairo_path
+from Onboard.utils       import (brighten, unicode_str,
+                                 gradient_line, drop_shadow,
+                                 roundrect_curve, roundrect_curve_custom,
+                                 rounded_path,
+                                 rounded_polygon_path_to_cairo_path)
 
 ### Logging ###
 import logging
@@ -50,6 +53,7 @@
 
 PangoUnscale = 1.0 / Pango.SCALE
 
+
 class Key(KeyCommon):
     _pango_layouts = None
     _label_extents = None  # resolution independent size {mod_mask: (w, h)}
@@ -182,12 +186,17 @@
         self.draw(cr)
         cr.restore()
 
-        Gdk.flush()   # else artefacts in labels and images
-                      # on Nexus 7, Raring
+        Gdk.flush()  # else artefacts in labels and images on Nexus 7, Raring
 
         return surface, clip_rect
 
-    def draw(self, cr, lod = LOD.FULL):
+    def draw_item(self, context):
+        if context.draw_cached:
+            self.draw_cached(context.cr)
+        else:
+            self.draw(context.cr, context.lod)
+
+    def draw(self, cr, lod=LOD.FULL):
         self.draw_geometry(cr, lod)
         self.draw_image(cr, lod)
         self.draw_label(cr, lod)
@@ -732,7 +741,7 @@
         else:
             if not rect:
                 rect = self.get_canvas_rect()
-            self._build_rect_path(cr, rect)
+            self.build_rect_path(cr, rect)
 
     def _build_complex_path(self, cr, path):
         roundness = config.theme_settings.roundrect_radius
@@ -740,13 +749,20 @@
         chamfer_size = self.context.scale_log_to_canvas_y(chamfer_size)
         rounded_path(cr, path, roundness, chamfer_size)
 
-    def _build_rect_path(self, context, rect):
+    def build_rect_path(self, context, rect):
         roundness = config.theme_settings.roundrect_radius
         if roundness:
             roundrect_curve(context, rect, roundness)
         else:
             context.rectangle(*rect)
 
+    def build_rect_path_custom(self, context, rect, corner_mask):
+        roundness = config.theme_settings.roundrect_radius
+        if roundness:
+            roundrect_curve_custom(context, rect, roundness, corner_mask)
+        else:
+            context.rectangle(*rect)
+
     def get_gradient_angle(self):
         return -pi/2.0 + self.get_light_direction()
 
@@ -948,7 +964,6 @@
         pass
 
 
-
 class FullSizeKey(WordlistKey):
     def __init__(self, id = "", border_rect = None):
         super(FullSizeKey, self).__init__(id, border_rect)
@@ -959,28 +974,33 @@
         return self.get_fullsize_rect()
 
 
-class BarKey(FullSizeKey):
-    def __init__(self, id = "", border_rect = None):
-        super(BarKey, self).__init__(id, border_rect)
+class FlatKey(FullSizeKey):
+    def __init__(self, id="", border_rect=None):
+        super(FlatKey, self).__init__(id, border_rect)
 
-    def draw(self, context, lod = LOD.FULL):
+    def draw(self, context, lod=LOD.FULL):
         # draw only when pressed, to blend in with the word list bar
         if self.pressed or self.active or self.scanned:
             self.draw_geometry(context, lod)
         self.draw_image(context, lod)
         self.draw_label(context, lod)
 
-    def can_show_label_popup(self):
-        return False
-
     def get_stroke_width(self):
         # Turn down stroke width -> no annoying banding at
         # what should be flat key edges.
         return 0.0
 
 
+class BarKey(FlatKey):
+    def __init__(self, id="", border_rect=None):
+        super(BarKey, self).__init__(id, border_rect)
+
+    def can_show_label_popup(self):
+        return False
+
+
 class WordKey(FixedFontMixin, BarKey):
-    def __init__(self, id="", border_rect = None):
+    def __init__(self, id="", border_rect=None):
         super(WordKey, self).__init__(id, border_rect)
 
 

=== modified file 'Onboard/Keyboard.py'
--- Onboard/Keyboard.py	2017-02-08 11:38:12 +0000
+++ Onboard/Keyboard.py	2017-02-10 10:33:11 +0000
@@ -56,6 +56,7 @@
 from Onboard.AutoHide              import AutoHide
 from Onboard.WordSuggestions       import WordSuggestions
 from Onboard.canonical_equivalents import canonical_equivalents
+from Onboard.CharacterPalette      import CharacterPalettePanel
 
 import Onboard.osk as osk
 
@@ -563,7 +564,7 @@
 
     color_scheme = None
 
-    _active_layer_id = None
+    _active_layer_ids = {}
     _layer_locked = False
     _last_alt_key = None
     _alt_locked   = False
@@ -669,6 +670,8 @@
         self._visibility_locked = False
         self._visibility_requested = None
 
+        self._character_palettes = []
+
         self.reset()
 
     def reset(self):
@@ -706,6 +709,7 @@
         self._disabled_keys = None
 
         self.layout = None
+        self._active_layer_ids.clear()
 
         self._pending_modifier_redraws_timer.stop()
         self._pending_modifier_redraws = {}
@@ -889,11 +893,16 @@
     def on_layout_loaded(self):
         """ called when the layout has been loaded """
 
+        palette_panels = \
+            self.find_items_from_classes((CharacterPalettePanel,))
+        for item in palette_panels:
+            item.keyboard = self
+
         # hide all still visible feedback popups; keys have changed.
         self._touch_feedback.hide()
 
-        self._connect_button_controllers()
-        self.assure_valid_active_layer()
+        self._init_button_controllers()
+        self._init_active_layers()
 
         WordSuggestions.on_layout_loaded(self)
 
@@ -909,6 +918,16 @@
         self.invalidate_ui()
         self.commit_ui_updates()
 
+    def _get_character_palettes(self):
+        return self._character_palettes
+
+    def update_character_palettes(self):
+        items = self._get_character_palettes()
+        for item in items:
+            item.keyboard = self
+            item.create_keys()
+        self.layout.invalidate_font_sizes()
+
     def set_virtkey(self, vk):
         self._init_text_changers(vk)
 
@@ -925,9 +944,13 @@
         else:
             return self.text_changer_key_stroke
 
-    def _connect_button_controllers(self):
+    def _init_button_controllers(self):
+        self.button_controllers = \
+            self._create_button_controllers(self.layout.iter_global_keys())
+
+    def _create_button_controllers(self, keys):
         """ connect button controllers to button keys """
-        self.button_controllers = {}
+        button_controllers = {}
 
         # connect button controllers to button keys
         types = {type.id : type for type in
@@ -938,14 +961,30 @@
                   BCNextPredictions, BCPauseLearning, BCLanguage,
                   BCStealthMode, BCAutoLearn, BCAutoPunctuation, BCInputline,
                   ]}
-        for key in self.layout.iter_global_keys():
+        for key in keys:
             if key.is_layer_button():
                 bc = BCLayer(self, key)
-                self.button_controllers[key] = bc
+                button_controllers[key] = bc
             else:
                 type = types.get(key.id)
                 if type:
-                    self.button_controllers[key] = type(self, key)
+                    button_controllers[key] = type(self, key)
+
+        return button_controllers
+
+    def add_button_controllers(self, keys):
+        controllers = self._create_button_controllers(keys)
+        self.button_controllers.update(controllers)
+
+    def remove_button_controllers(self, keys):
+        for key in keys:
+            try:
+                del self.button_controllers[key]
+            except KeyError:
+                pass
+
+    def get_button_controller(self, key):
+        return self.button_controllers.get(key)
 
     def update_scanner_enabled(self):
         """ Enable keyboard scanning if it is enabled in gsettings. """
@@ -982,53 +1021,76 @@
         self.key_down(key)
         self.key_up(key)
 
-    def get_layers(self):
+    def get_layer_ids(self, parent_layer_id=None):
         if self.layout:
-            return self.layout.get_layer_ids()
+            return self.layout.get_layer_ids(parent_layer_id)
         return []
 
-    def is_first_layer_id(self, layer_id):
-        layers = self.get_layers()
+    def is_first_layer_id(self, layer_id, parent_layer_id=None):
+        layers = self.get_layer_ids(parent_layer_id)
         if not layers:
             return False
         return layers[0] == layer_id
 
-    def get_active_layer_id(self):
-        """ Return currently active layer id """
-        layers = self.get_layers()
+    def get_active_layer_id(self, parent_layer_id=None):
+        """
+        Return currently active layer id.
+        """
+        active_id = self._active_layer_ids.get(parent_layer_id)
+
+        layers = self.get_layer_ids(parent_layer_id)
         if not layers:
             return None
 
-        if self._active_layer_id is None or \
-           self._active_layer_id not in layers:
+        if active_id is None or \
+           active_id not in layers:
             return layers[0]
 
-        return self._active_layer_id
-
-    def set_active_layer_id(self, layer_id=None):
+        return active_id
+
+    def get_active_layer_ids(self):
+        return list(self._active_layer_ids.values())
+
+    def set_active_layer_id(self, layer_id=None, parent_layer_id=None):
         """ Set currently active layer id. """
         """ None = make first layer active """
-        if not layer_id:
-            layers = self.get_layers()
-            if layers:
-                self._active_layer_id = layers[0]
-        else:
-            self._active_layer_id = layer_id
-
-    def is_first_layer_active(self):
-        return self.is_first_layer_id(self._active_layer_id)
-
-    def assure_valid_active_layer(self):
+        layers = self.get_layer_ids(parent_layer_id)
+        if layers:
+            if not layer_id or \
+               layer_id not in layers:
+                self._active_layer_ids[parent_layer_id] = layers[0]
+            else:
+                self._active_layer_ids[parent_layer_id] = layer_id
+
+    def is_first_layer_active(self, parent_layer_id=None):
+        return self.is_first_layer_id(self._active_layer_ids, parent_layer_id)
+
+    def _init_active_layers(self):
         """
         Reset layer index if it is out of range. e.g. due to
         loading a layout with fewer panes.
         """
-        layers = self.get_layers()
-        if layers:
-            if self._active_layer_id not in layers:
-                self._active_layer_id = layers[0]
-        else:
-            self._active_layer_id = None
+        # Remove entries that don't exist in the current layout.
+        # Helps testing new layouts by not resetting all layers
+        # when a layout is loaded.
+        layer_ids = self.get_layer_ids()
+        for parent_layer_id in list(self._active_layer_ids.keys()):
+            if parent_layer_id and parent_layer_id not in layer_ids:
+                del self._active_layer_ids[parent_layer_id]
+
+        # Set valid layer_ids for all parent_layers. Create entries
+        # for all existing layer_parent_ids for correct initial visibility of
+        # sublayers.
+        for lid in self.get_layer_ids():
+            parent_layer_id = self.layout.layer_to_parent_id(lid)
+            layer_ids = self.get_layer_ids(parent_layer_id)
+
+            if layer_ids:
+                active_layer_id = self._active_layer_ids.get(parent_layer_id)
+                if active_layer_id not in layer_ids:
+                    self._active_layer_ids[parent_layer_id] = layer_ids[0]
+            else:
+                self._active_layer_ids[parent_layer_id] = None
 
     def iter_keys(self, group_name=None):
         """ iterate through all keys or all keys of a group """
@@ -1228,7 +1290,7 @@
 
             elif key_type == KeyCommon.BUTTON_TYPE:
                 # Buttons decide for themselves what is to happen.
-                controller = self.button_controllers.get(key)
+                controller = self.get_button_controller(key)
                 if controller:
                     controller.long_press(view, button)
 
@@ -1275,7 +1337,7 @@
             self.redraw_labels(False)
 
         if key.type == KeyCommon.BUTTON_TYPE:
-            controller = self.button_controllers.get(key)
+            controller = self.get_button_controller(key)
             if controller:
                 key.activated = controller.is_activated_on_press()
 
@@ -1583,7 +1645,8 @@
 
         elif key_type == KeyCommon.BUTTON_TYPE:
             activated = False
-            controller = self.button_controllers.get(key)
+            key.on_press(view, button, event_type)
+            controller = self.get_button_controller(key)
             if controller:
                 activated = controller.is_activated_on_press()
                 controller.press(view, button, event_type)
@@ -1623,7 +1686,8 @@
             self.get_text_changer().release_keycode(key.code)
 
         elif key_type == KeyCommon.BUTTON_TYPE:
-            controller = self.button_controllers.get(key)
+            key.on_release(view, button, event_type)
+            controller = self.get_button_controller(key)
             if controller:
                 controller.release(view, button, event_type)
 
@@ -2049,7 +2113,7 @@
         if action is None:
             if key.type == KeyCommon.BUTTON_TYPE:
                 action = KeyCommon.DELAYED_STROKE_ACTION
-                controller = self.button_controllers.get(key)
+                controller = self.get_button_controller(key)
                 if controller and \
                    controller.is_activated_on_press():
                     action = KeyCommon.SINGLE_STROKE_ACTION
@@ -2235,12 +2299,10 @@
 
     def update_visible_layers(self):
         """ show/hide layers """
-        layout = self.layout
-        if layout:
-            layers = layout.get_layer_ids()
-            if layers:
-                layout.set_visible_layers([layers[0],
-                                           self.get_active_layer_id()])
+        layers = self.get_layer_ids()
+        if layers:
+            self.layout.set_visible_layers(
+                [None, layers[0]] + list(self._active_layer_ids.values()))
 
     def update_scanner(self):
         """ tell scanner to update on layout changes """
@@ -2786,21 +2848,21 @@
 
 
 class BCLayer(ButtonController):
-    """ layer switch button, switches to layer <layer_id> when released """
+    """
+    Layer switch button, switches to layer <self._layer_id> when released.
+    """
 
     def __init__(self, keyboard, key):
         super(BCLayer, self).__init__(keyboard, key)
         self._layer_id = key.get_target_layer_id()
-
-    def _get_id(self):
-        return self._layer_id
-
-    id = property(_get_id)
+        self._parent_layer_id = key.get_target_layer_parent_id()
 
     def release(self, view, button, event_type):
         keyboard = self.keyboard
+        parent = self._parent_layer_id
 
-        active_before = keyboard.get_active_layer_id() == self._layer_id
+        active_before = \
+            keyboard.get_active_layer_id(parent) == self._layer_id
         locked_before = active_before and keyboard._layer_locked
 
         active, locked = \
@@ -2812,29 +2874,33 @@
         if not keyboard.can_activate_key(self.key):
             active = True
 
-        keyboard.set_active_layer_id(self._layer_id
-                                     if active else None)
+        keyboard.set_active_layer_id(self._layer_id if active else None,
+                                     parent)
 
-        keyboard._layer_locked = \
-            (locked
-             if not keyboard.is_first_layer_id(self._layer_id)
-             else False)
+        # sublayers don't lock at all for now
+        if not parent:
+            keyboard._layer_locked = \
+                (locked
+                 if not keyboard.is_first_layer_id(self._layer_id)
+                 else False)
 
         if active_before != active:
             keyboard.invalidate_visible_layers()
             keyboard.invalidate_canvas()
 
     def update(self):
+        keyboard = self.keyboard
         key = self.key
+        parent = self._parent_layer_id
 
         # don't show active state for layer 0, it'd be visible all the time
-        active = key.show_active and \
-            key.get_target_layer_id() == self.keyboard.get_active_layer_id()
+        active_layer_id = keyboard.get_active_layer_id(parent)
+        active = key.show_active and self._layer_id == active_layer_id
         if active:
-            active = self.keyboard.can_activate_key(key)
+            active = keyboard.can_activate_key(key)
 
         self.set_active(active)
-        self.set_locked(active and self.keyboard._layer_locked)
+        self.set_locked(active and not parent and keyboard._layer_locked)
 
 
 class BCPreferences(ButtonController):
@@ -3033,3 +3099,5 @@
     def update(self):
         self.set_active(config.wp.stealth_mode)
 
+
+

=== modified file 'Onboard/KeyboardPopups.py'
--- Onboard/KeyboardPopups.py	2016-10-22 01:26:08 +0000
+++ Onboard/KeyboardPopups.py	2017-02-10 10:33:11 +0000
@@ -500,7 +500,7 @@
         frame_width = LayoutBuilder._calc_frame_width(context)
 
         layout = LayoutRoot(layout)
-        layout.update_log_rect()
+        layout.update_log_rects()
         log_rect = layout.get_border_rect()
         canvas_rect = Rect(frame_width, frame_width,
                            log_rect.w * context.scale_log_to_canvas_x(1.0),

=== modified file 'Onboard/KeyboardWidget.py'
--- Onboard/KeyboardWidget.py	2017-02-08 11:38:12 +0000
+++ Onboard/KeyboardWidget.py	2017-02-10 10:33:11 +0000
@@ -480,7 +480,7 @@
 
         rect = self.get_canvas_content_rect()
 
-        layout.update_log_rect()  # update logical tree to base aspect ratio
+        layout.update_log_rects()  # update logical tree to base aspect ratio
         rect = self._get_aspect_corrected_layout_rect(
             rect, self.get_base_aspect_rect())
         layout.do_fit_inside_canvas(rect)  # update contexts to final aspect
@@ -1148,7 +1148,7 @@
             # start dwelling if we have entered a dwell-enabled key
             if hit_key and \
                hit_key.sensitive:
-                controller = self.keyboard.button_controllers.get(hit_key)
+                controller = self.keyboard.get_button_controller(hit_key)
                 if controller and controller.can_dwell() and \
                    not self.is_dwelling() and \
                    not self.already_dwelled(hit_key) and \
@@ -1985,3 +1985,89 @@
         return 0
 
 
+    class AuxcharSelection:
+        """ Select emoji or other special characters """
+
+        def __init__(self, keyboard, parent):
+            self._keyboard = keyboard
+            self._parent = parent
+
+        def show(self):
+
+            # turn off AT-SPI listeners to prevent D-BUS deadlocks (Quantal).
+            self._keyboard.on_focusable_gui_opening()
+
+            dialog = Gtk.Dialog(title=title,
+                            transient_for=self._parent.get_toplevel(),
+                            flags=0)
+            dialog.add_button(_("_Close"), Gtk.ResponseType.OK)
+
+            # Don't hide dialog behind the keyboard in force-to-top mode.
+            if config.is_force_to_top():
+                dialog.set_position(Gtk.WindowPosition.CENTER)
+
+            dialog.set_default_response(Gtk.ResponseType.OK)
+
+            # outer vertical box
+            box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL,
+                          spacing=12, border_width=5)
+
+            # search entry
+            search_entry = Gtk.SearchEntry()
+            box.add(search_entry)
+
+            # glyph tree
+            self._glyph_tree_view = Gtl.TreeView()
+            init_glyph_tree_view()
+            box.add(self._glyph_tree_view)
+
+            dialog.get_content_area().add(box)
+
+            dialog.connect("response", self._on_dialog_response)
+            dialog.show_all()
+
+        def _on_dialog_response(self, dialog, response, snippet_id, \
+                                        label_entry, text_entry):
+            dialog.destroy()
+
+            # Reenable AT-SPI keystroke listeners.
+            # Delay this until the dialog is really gone.
+            GLib.idle_add(self._keyboard.on_focusable_gui_closed)
+
+        def init_glyph_tree_view(self):
+            (
+                self.GLYPH_COL_GLYPH,
+                self.GLYPH_COL_DESCRIPTION
+            ) = range(2)
+
+            self._glyph_model = Gtk.TreeStore(str, int)
+            self._glyph_tree_view.set_model(self._glyph_model)
+            #model.clear()
+
+            self._add_layout_section(model, system, _("Core layouts"))
+            self._add_layout_section(model, contributed, _("Contributions"))
+            self._add_layout_section(model, user, _("My layouts"))
+
+            self.layout_view.expand_all()
+            self.update_layout_view_selection()
+
+        def update_layout_view_selection(self):
+            self.select_tree_view_row(self.layout_view, self.LAYOUT_COL_FILENAME,
+                                    config.layout_filename)
+
+        def _add_layout_section(self, model, lis, section_name):
+            if lis:
+                parent_iter = model.append(None)
+                model.set(parent_iter,
+                        self.LAYOUT_COL_NAME, "<b>{}</b>" \
+                        .format(escape_markup(section_name)))
+                for li in lis:
+                    child_iter = model.append(parent_iter)
+                    model.set(child_iter,
+                            self.LAYOUT_COL_NAME, li.id_string,
+                            self.LAYOUT_COL_SUMMARY, li.summary,
+                            self.LAYOUT_COL_FILENAME, li.filename,
+                            self.LAYOUT_COL_HAS_ABOUT_INFO, li.has_about_info,
+                            self.LAYOUT_COL_IS_ROW_SENSITIVE, bool(li.filename))
+
+

=== modified file 'Onboard/Layout.py'
--- Onboard/Layout.py	2017-02-08 11:38:12 +0000
+++ Onboard/Layout.py	2017-02-10 10:33:11 +0000
@@ -35,14 +35,15 @@
     Transforms logical coordinates to canvas coordinates and vice versa.
     """
     def __init__(self):
-        # logical rectangle as defined by the keyboard layout,
-        # never changed after loading.
+        # Logical rectangle as defined by the keyboard layout.
+        # Never changed after loading.
         self.initial_log_rect = Rect(0.0, 0.0, 1.0, 1.0)  # includes border
 
-        # logical rectangle as defined by the keyboard layout
+        # Logical rectangle as defined by the keyboard layout.
+        # May be changed after loading e.g. for word suggestion keys.
         self.log_rect = Rect(0.0, 0.0, 1.0, 1.0)  # includes border
 
-        # canvas rectangle in drawing units
+        # Canvas rectangle in drawing units.
         self.canvas_rect = Rect(0.0, 0.0, 1.0, 1.0)
 
     def __repr__(self):
@@ -163,6 +164,8 @@
         self.__dict__['_item'] = item    # item to decorate
         self.invalidate_caches()
         self.init_chamfer_sizes()
+        self._font_sizes_valid = False
+        self._item.root_decorator = self  # point back here from the tree root
 
     def __getattr__(self, name):
         return getattr(self._item, name)
@@ -192,6 +195,18 @@
         self._last_hit_args = None
         self._last_hit_key = None
 
+    def invalidate_font_sizes(self):
+        """
+        Update font_sizes at the next possible chance.
+        """
+        self._font_sizes_valid = False
+
+    def get_font_sizes_valid(self):
+        return self._font_sizes_valid
+
+    def set_font_sizes_valid(self, valid):
+        self._font_sizes_valid = valid
+
     def fit_inside_canvas(self, canvas_border_rect):
         self._item.fit_inside_canvas(canvas_border_rect)
 
@@ -250,7 +265,7 @@
             self._cached_layer_keys[layer_id] = items
         return items
 
-    def iter_layer_items(self, layer_id = None, only_visible = True):
+    def iter_layer_items(self, layer_id=None, only_visible=True):
         args = (layer_id, only_visible)
         items = self._cached_layer_items.get(args)
         if not items:
@@ -258,11 +273,16 @@
             self._cached_layer_items[args] = items
         return items
 
-    def get_layer_ids(self):
+    def get_layer_ids(self, parent_layer_id=None):
         layer_ids = self._cached_layer_ids
         if not layer_ids:
             layer_ids = self._item.get_layer_ids()
             self._cached_layer_ids = layer_ids
+
+        if parent_layer_id:
+            prefix = parent_layer_id + "."
+            return [id for id in layer_ids if id.startswith(prefix)]
+
         return layer_ids
 
     def get_key_groups(self):
@@ -275,19 +295,21 @@
             self._cached_key_groups = key_groups
         return key_groups
 
-    def get_key_at(self, point, active_layer):
+    def get_key_at(self, point, active_layer_ids):
         """
         Find the topmost key at point.
         """
+        active_layer_ids = tuple(active_layer_ids)
+
         # After motion-notify-event the query-tooltip event calls this
         # a second time with the same point. Don't search again in that case.
-        args = (point, active_layer)
+        args = (point, active_layer_ids)
         if self._last_hit_args == args:
             return self._last_hit_key
 
         key = None
         x, y = point
-        hit_rects = self._get_hit_rects(active_layer)
+        hit_rects = self._get_hit_rects(active_layer_ids)
         for x0, y0, x1, y1, k in hit_rects:
             # Inlined test, not using Rect.is_point_within for speed.
             if x >= x0 and x < x1 and \
@@ -302,20 +324,26 @@
 
         return key
 
-    def _get_hit_rects(self, active_layer):
+    def _get_hit_rects(self, active_layer_ids):
         try:
-            hit_rects = self._cached_hit_rects[active_layer]
+            hit_rects = self._cached_hit_rects[active_layer_ids]
         except KeyError:
-            # All visible and sensitive key items sorted in z-order.
+            # All visible and sensitive key items sorted by z-order.
             # Keys of the active layer have priority over non-layer keys
             # (layer switcher, hide, etc.).
             iter_layer_keys = self.iter_layer_keys
-            items = list(reversed(list(iter_layer_keys(active_layer)))) + \
-                    list(reversed(list(iter_layer_keys(None))))
-
-            hit_rects = [item.get_hit_rect().to_extents() + (item,) \
-                     for item in items]
-            self._cached_hit_rects[active_layer] = hit_rects
+            items = []
+            for layer_id in reversed(active_layer_ids):
+                items.extend(list(reversed(list(iter_layer_keys(layer_id)))))
+            items.extend(list(reversed(list(iter_layer_keys(None)))))
+
+            hit_rects = []
+            for item in items:
+                r = item.get_hit_rect()
+                if r is not None:  # not clipped away?
+                    hit_rects.append(r.to_extents() + (item,))
+
+            self._cached_hit_rects[active_layer_ids] = hit_rects
 
         return hit_rects
 
@@ -325,7 +353,7 @@
             if key.chamfer_size is None:
                 layer_id = key.get_layer()
                 chamfer_size = chamfer_sizes.get(layer_id)
-                if not chamfer_size is None:
+                if chamfer_size is not None:
                     key.chamfer_size = chamfer_size
 
     def _calc_chamfer_sizes(self):
@@ -405,6 +433,12 @@
     templates = None
     keysym_rules = None
 
+    # root decorator
+    root_decorator = None
+
+    # Clip children?
+    clip_rect = None
+
     def __init__(self):
         self.context = KeyContext()
 
@@ -499,7 +533,7 @@
         Scale item and its children to fit inside the given canvas_rect.
         """
         # recursively update item's bounding boxes
-        self.update_log_rect()
+        self.update_log_rects()
 
         # recursively fit inside canvas
         self.do_fit_inside_canvas(canvas_border_rect)
@@ -510,12 +544,16 @@
         """
         self.context.canvas_rect = canvas_border_rect
 
-    def update_log_rect(self):
+    def update_log_rects(self):
+        """
+        Recursively update the log_rects of this sub-tree.
+        """
         for item in self.iter_depth_first():
-            item._update_log_rect()
+            item.update_log_rect()
 
-    def _update_log_rect(self):
+    def update_log_rect(self):
         """
+        Update the log_rect of this item.
         Override this for layout items that have to calculate their
         logical rectangle.
         """
@@ -523,11 +561,30 @@
 
     def get_hit_rect(self):
         """ Returns true if the point lies within the items borders. """
-        return self.get_canvas_border_rect().inflate(1)
+        rect = self.get_canvas_border_rect().inflate(1)
+
+        # attempt to clip at the parents clip_rect
+        parent = self.get_parent()
+        if parent and parent.clip_rect is not None:
+            rect = parent.clip(rect)
+            if rect.is_empty():
+                return None
+
+        return rect
+
+    def set_clip_rect(self, canvas_rect):
+        """ Set clipping rectangle in canvas coordinates. """
+        self.clip_rect = canvas_rect
+
+    def clip(self, canvas_rect):
+        """ Clip rect at the current clipping rect """
+        return self.clip_rect.intersection(canvas_rect)
 
     def is_point_within(self, canvas_point):
         """ Returns true if the point lies within the items borders. """
         rect = self.get_hit_rect()
+        if rect is None:
+            return False
         return rect.is_point_within(canvas_point)
 
     def set_visible(self, visible):
@@ -582,13 +639,9 @@
                 return item
             item = item.parent
 
-    def get_global_layout_root(self):
-        """ Return the root layout item """
-        item = self
-        while item:
-            if item.parent is None:
-                return item
-            item = item.parent
+    def get_root_decorator(self):
+        """ Return the root decorator if available """
+        return self.get_layout_root().root_decorator
 
     def get_layer(self):
         """ Return the first layer_id on the path from the tree root to self """
@@ -600,11 +653,33 @@
             item = item.parent
         return layer_id
 
+    @staticmethod
+    def layer_to_parent_id(layer_id):
+        """
+        Doctests:
+        >>> repr(LayoutItem.layer_to_parent_id(None))
+        'None'
+        >>> repr(LayoutItem.layer_to_parent_id("abc"))
+        'None'
+        >>> LayoutItem.layer_to_parent_id("abc.cde")
+        'abc'
+        >>> LayoutItem.layer_to_parent_id("abc.cde.fgh")
+        'abc.cde'
+        """
+        if layer_id is None:
+            return None
+
+        pos = layer_id.rfind(".")
+        if pos >= 0:
+            return layer_id[:pos]
+        return None
+
     def set_visible_layers(self, layer_ids):
         """
-        Show all items of layers <layer_ids>, hide all items of the other layers.
+        Show all items of layers <layer_ids>, hide all items of
+        the other layers.
         """
-        if not self.layer_id is None:
+        if self.layer_id is not None:
             if not self.is_key():
                 self.visible = self.layer_id in layer_ids
 
@@ -618,8 +693,8 @@
         if _layer_ids is None:
             _layer_ids = []
 
-        if not self.layer_id is None and \
-           not self.layer_id in _layer_ids:
+        if self.layer_id is not None and \
+           self.layer_id not in _layer_ids:
             _layer_ids.append(self.layer_id)
 
         for item in self.items:
@@ -694,12 +769,38 @@
                 for visible_item in item.iter_visible_items():
                     yield visible_item
 
-    def iter_keys(self, group_name = None):
+    def draw_tree(self, context):
+        """
+        Traverses top to bottom all visible layout items of the
+        layout tree. Invisible paths are cut short.
+        """
+        if self.visible:
+            if context.draw_rect.intersects(self.get_canvas_border_rect()):
+                if self.clip_rect is not None:
+                    cr = context.cr
+                    cr.save()
+                    cr.rectangle(*self.clip_rect)
+                    cr.clip()
+
+                self.draw_item(context)
+
+                for item in self.items:
+                    item.draw_tree(context)
+
+                if self.clip_rect is not None:
+                    context.cr.restore()
+
+    def draw_item(self, context):
+        if self.layer_id:
+            context.draw_layer_background(self)
+
+    def iter_keys(self, group_name=None):
+
         """
         Iterates through all keys of the layout tree.
         """
         if self.is_key():
-            if group_name is None or key.group == group_name:
+            if group_name is None or self.group == group_name:
                 yield self
 
         for item in self.items:
@@ -721,12 +822,12 @@
                 for child in item.iter_global_items():
                     yield child
 
-    def iter_global_keys(self, group_name = None):
+    def iter_global_keys(self, group_name=None):
         """
         Iterates through all keys of the layout tree including sublayouts.
         """
         if self.is_key():
-            if group_name is None or key.group == group_name:
+            if group_name is None or self.group == group_name:
                 yield self
 
         for item in self.items:
@@ -746,8 +847,8 @@
             if item.is_key():
                 yield item
 
-    def iter_layer_items(self, layer_id = None, only_visible = True,
-                              _found_layer_id = None):
+    def iter_layer_items(self, layer_id=None, only_visible=True,
+                         _found_layer_id=None):
         """
         Iterate through all items of the given layer.
         The first layer definition found in the path to each key wins.
@@ -760,7 +861,11 @@
         if self.layer_id == layer_id:
             _found_layer_id = layer_id
 
-        if self.layer_id and self.layer_id != _found_layer_id:
+        if self.layer_id and \
+           self.layer_id != _found_layer_id and \
+           (not layer_id or
+            not (self.layer_id.startswith(layer_id + ".") or
+                 layer_id.startswith(self.layer_id + "."))):
             return
 
         if _found_layer_id == layer_id:
@@ -772,7 +877,9 @@
                 yield item
 
     def find_instance_in_path(self, classinfo):
-        """ Find an item of a certain type in the path from self to the root. """
+        """
+        Find an item of a certain type in the path from self to the root.
+        """
         item = self
         while item:
             if isinstance(item, classinfo):
@@ -823,6 +930,12 @@
             yield item
             item = item.parent or item.sublayout_parent
 
+    def on_press(self, view, button, event_type):
+        pass
+
+    def on_release(self, view, button, event_type):
+        pass
+
 
 class LayoutBox(LayoutItem):
     """
@@ -844,7 +957,7 @@
         if self.horizontal != horizontal:
             self.horizontal = horizontal
 
-    def _update_log_rect(self):
+    def update_log_rect(self):
         self.context.log_rect = self._calc_bounds()
 
     def _calc_bounds(self):
@@ -989,13 +1102,13 @@
         else:
             context = KeyContext()
             context.log_rect = self.get_border_rect()
-            context.canvas_rect = self.get_canvas_rect() # exclude border
+            context.canvas_rect = self.get_canvas_rect()  # exclude border
 
             for item in self.items:
                 rect = context.log_to_canvas_rect(item.context.log_rect)
                 item.do_fit_inside_canvas(rect)
 
-    def _update_log_rect(self):
+    def update_log_rect(self):
         self.context.log_rect = self._calc_bounds()
 
     def _calc_bounds(self):
@@ -1019,3 +1132,122 @@
             return Rect()
         return bounds
 
+
+class ScrolledLayoutPanel(LayoutPanel):
+
+    def update_log_rect(self):
+        pass
+
+
+class DrawingItem(LayoutItem):
+
+    # extended id for key specific theme tweaks
+    # e.g. theme_id=DELE.numpad (with id=DELE)
+    theme_id = None
+
+    # extended id for layout specific tweaks
+    # e.g. "hide.wordlist", for hide button in wordlist mode
+    svg_id = None
+
+    # color scheme
+    color_scheme = None
+
+    def __init__(self):
+        LayoutItem.__init__(self)
+        self.colors = {}
+
+    def get_svg_id(self):
+        return ""
+
+    def set_id(self, id, theme_id=None, svg_id=None):
+        self.theme_id, self.id = self.parse_id(id)
+        if theme_id:
+            self.theme_id = theme_id
+        self.svg_id = self.id if not svg_id else svg_id
+
+    @staticmethod
+    def parse_id(value):
+        """
+        The theme id has the form <id>.<arbitrary identifier>, where
+        the identifier should be a description of the location of
+        the key relative to its surroundings, e.g. 'DELE.next-to-backspace'.
+        Don't use layout names or layer ids for the theme id, they lose
+        their meaning when layouts are copied or renamed by users.
+        """
+        theme_id = value
+        id = value.split(".")[0]
+        return theme_id, id
+
+    @staticmethod
+    def split_theme_id(theme_id):
+        """
+        Simple split in prefix (id) before the dot and suffix after the dot.
+        """
+        components = theme_id.split(".")
+        if len(components) == 1:
+            return components[0], ""
+        return components[0], components[1]
+
+    @staticmethod
+    def build_theme_id(prefix, postfix):
+        if postfix:
+            return prefix + "." + postfix
+        return prefix
+
+    def get_similar_theme_id(self, prefix=None):
+        if prefix is None:
+            prefix = self.id
+        theme_id = prefix
+        comps = self.theme_id.split(".")[1:]
+        if comps:
+            theme_id += "." + comps[0]
+        return theme_id
+
+    def get_fill_color(self):
+        return self.get_color("fill")
+
+    def get_stroke_color(self):
+        return self.get_color("stroke")
+
+    def get_label_color(self):
+        return self.get_color("label")
+
+    def get_secondary_label_color(self):
+        return self.get_color("secondary-label")
+
+    def get_dwell_progress_color(self):
+        return self.get_color("dwell-progress")
+
+    def get_color(self, element, state=None):
+        color_key = (element)
+        try:
+            return self.colors[color_key]
+        except KeyError:
+            return self.cache_color(element, color_key, state)
+
+    def cache_color(self, element, color_key, state=None):
+        if self.color_scheme:
+            rgba = self.color_scheme.get_key_rgba(self, element, state)
+        elif element == "label":
+            rgba = [0.0, 0.0, 0.0, 1.0]
+        else:
+            rgba = [1.0, 1.0, 1.0, 1.0]
+        self.colors[color_key] = rgba
+        return rgba
+
+    def get_state(self):
+        state = {}
+        return state
+
+
+class RectangleItem(DrawingItem):
+
+    def draw_item(self, context):
+        cr = context.cr
+        cr.save()
+        cr.set_source_rgba(*self.get_fill_color())
+        cr.rectangle(*self.get_canvas_rect())
+        cr.fill()
+        cr.restore()
+
+

=== modified file 'Onboard/LayoutLoaderSVG.py'
--- Onboard/LayoutLoaderSVG.py	2017-02-08 11:38:12 +0000
+++ Onboard/LayoutLoaderSVG.py	2017-02-10 10:33:11 +0000
@@ -44,7 +44,9 @@
 from Onboard.WordSuggestions import WordListPanel  # noqa: flake8
 from Onboard.KeyGtk          import (RectKey, WordlistKey, BarKey, # noqa: flake8
                                      WordKey, InputlineKey)
-from Onboard.CharacterPalette import CharacterPalettePanel, CharacterPaletteKey  # noqa: flake8
+from Onboard.CharacterPalette import (EmojiPalettePanel,    # noqa: flake8
+                                      SymbolPalettePanel,
+                                      CharacterPaletteKey)
 
 from Onboard.Config import Config
 config = Config()
@@ -500,7 +502,7 @@
         elif "target_layer" in attributes:
             key.code = key.id[:]
             key.type = KeyCommon.BUTTON_TYPE
-            key.target_layer = attributes["target_layer"]
+            key.target_layer_id = attributes["target_layer"]
         elif "button" in attributes:
             key.code = key.id[:]
             key.type = KeyCommon.BUTTON_TYPE

=== modified file 'Onboard/LayoutView.py'
--- Onboard/LayoutView.py	2017-02-08 11:38:12 +0000
+++ Onboard/LayoutView.py	2017-02-10 10:33:11 +0000
@@ -59,7 +59,6 @@
         self.supports_alpha = False
 
         self._lod = LOD.FULL
-        self._font_sizes_valid = False
         self._shadow_quality_valid = False
         self._last_canvas_shadow_rect = Rect()
 
@@ -90,18 +89,14 @@
         return self.keyboard.color_scheme
 
     def invalidate_for_resize(self, lod=LOD.FULL):
-        self.invalidate_keys()
-        if self._lod == LOD.FULL:
-            self.invalidate_shadows()
-        self.invalidate_font_sizes()
-        # self.invalidate_label_extents()
-        self.keyboard.invalidate_for_resize()
-
-    def invalidate_font_sizes(self):
-        """
-        Update font_sizes at the next possible chance.
-        """
-        self._font_sizes_valid = False
+        layout = self.get_layout()
+        if layout:
+            self.invalidate_keys()
+            if self._lod == LOD.FULL:
+                self.invalidate_shadows()
+            layout.invalidate_font_sizes()
+            # self.invalidate_label_extents()
+            self.keyboard.invalidate_for_resize()
 
     def invalidate_keys(self):
         """
@@ -239,14 +234,14 @@
     def render(self, context):
         """ Pre-render key surfaces for instant initial drawing. """
 
+        layout = self.get_layout()
+        if not layout:
+            return
+
         # lazily update font sizes and labels
-        if not self._font_sizes_valid:
+        if not layout.get_font_sizes_valid():
             self.update_labels()
 
-        layout = self.get_layout()
-        if not layout:
-            return
-
         self._auto_select_shadow_quality(context)
 
         # run through all visible layout items
@@ -270,25 +265,23 @@
         return (lod == LOD.FULL) and \
                (not self._starting_up or self._keys_pre_rendered)
 
-    def draw(self, widget, context):
-        if not Gtk.cairo_should_draw_window(context, widget.get_window()):
+    def draw(self, widget, cr):
+        if not Gtk.cairo_should_draw_window(cr, widget.get_window()):
+            return
+
+        layout = self.get_layout()
+        if not layout:
             return
 
         lod = self._lod
         draw_cached = self._can_draw_cached(lod)
 
         # lazily update font sizes and labels
-        if not self._font_sizes_valid:
+        if not layout.get_font_sizes_valid():
             self.update_labels(lod)
 
-        draw_rect = self.get_damage_rect(context)
-
         # draw background
-        decorated = self._draw_background(context, lod)
-
-        layout = self.get_layout()
-        if not layout:
-            return
+        decorated = self._draw_background(cr, lod)
 
         # draw layer 0 and None-layer background
         layer_ids = layout.get_layer_ids()
@@ -298,24 +291,27 @@
             alpha = self.get_background_rgba()[3]
         else:
             alpha = 1.0
-        self._draw_layer_key_background(context, alpha,
+        self._draw_layer_key_background(cr, alpha,
                                         None, None, lod)
         if layer_ids:
-            self._draw_layer_key_background(context, alpha,
+            self._draw_layer_key_background(cr, alpha,
                                             None, layer_ids[0], lod)
 
-        # run through all visible layout items
-        for item in layout.iter_visible_items():
-            if item.layer_id:
-                self._draw_layer_background(context, item, layer_ids, decorated)
+        # Yet another context; this one helps pass all the
+        # accumulated values for easier in-tree drawing.
+        class DrawingContext:
+            def draw_layer_background(self, item):
+                self.view._draw_layer_background(self.cr, item,
+                                                 layer_ids, decorated)
+        context = DrawingContext()
+        context.cr = cr
+        context.draw_rect = self.get_damage_rect(cr)
+        context.lod = lod
+        context.draw_cached = draw_cached
+        context.view = self
 
-            # draw key
-            if item.is_key() and \
-               draw_rect.intersects(item.get_canvas_border_rect()):
-                if draw_cached:
-                    item.draw_cached(context)
-                else:
-                    item.draw(context, lod)
+        # draw all visible layout items
+        layout.draw_tree(context)
 
         self._starting_up = False
 
@@ -742,12 +738,14 @@
         if layout:
             if lod == LOD.FULL:  # no label changes necessary while dragging
 
+                # update label text
                 for key in layout.iter_keys():
                     old_label = key.get_label()
                     key.configure_label(mod_mask)
                     if key.get_label() != old_label:
                         changed_keys.add(key)
 
+            # update font sizes
             for keys in layout.get_key_groups().values():
                 max_size = 0
                 for key in keys:
@@ -767,14 +765,15 @@
                         key.font_size = max_size
                         changed_keys.add(key)
 
-        self._font_sizes_valid = True
+        layout.set_font_sizes_valid(True)
+
         return tuple(changed_keys)
 
     def get_key_at_location(self, point):
         layout = self.get_layout()
         keyboard = self.keyboard
         if layout and keyboard:  # may be gone on exit
-            return layout.get_key_at(point, keyboard.get_active_layer_id())
+            return layout.get_key_at(point, keyboard.get_active_layer_ids())
         return None
 
     def get_xid(self):

=== added file 'Onboard/UnicodeData.py'
--- Onboard/UnicodeData.py	1970-01-01 00:00:00 +0000
+++ Onboard/UnicodeData.py	2017-02-10 10:33:11 +0000
@@ -0,0 +1,95 @@
+#!/usr/bin/python3
+# -*- coding: utf-8 -*-
+
+# Copyright © 2017 marmuta <marmvta@gmail.com>
+#
+# This file is part of Onboard.
+#
+# Onboard is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 3 of the License, or
+# (at your option) any later version.
+#
+# Onboard is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program. If not, see <http://www.gnu.org/licenses/>.
+
+from Onboard.emoji_data import emoji_data
+
+
+class UnicodeData:
+    """
+    Singleton class providing emoji data and general Unicode information.
+    """
+
+    _symbol_data = [
+        ["α", "αβγδεζηθικλμνξοπρςστυφχψω"        # Greek
+              "ΑΒΓΔΕΖΗΘΙΚΛΜΝΞΟΠΡΣΤΥΦΧΨΩ"],
+
+        ["ℝ", "ℝℂℕℙℚℤ"                           # math & physics
+              "∅∃∄∈∉∀∑∥∦∡⊾∞"
+              "∩∪⊂⊃⊄⊅⊈⊉⊆⊇…"
+              "≤≥≦≧≨≩"
+              "≁≂≃≄≅≆≇≈≉≊≋≌≍"
+              "√∛∜"
+              "∫∬∭"
+              "℃℉№"
+         ],
+        ["€", "$₠₡₢₣₤₥₦₧₨₩₪₫€₭₮₯₰₱₲₳₴₵₶₷₸₹₺₻₼₽₾"   # currency
+              ""
+         ],
+        ["²₂", "⁰¹²³⁴⁵⁶⁷⁸⁹⁺⁻⁼⁽⁾"                 # super- and subscript
+               "ⁱ"
+               "₀₁₂₃₄₅₆₇₈₉₊₋₌₍₎"
+               "ₐₑₒₓₔₕₖₗₘₙₚₛₜ"
+         ],
+    ]
+
+    def __new__(cls, *args, **kwargs):
+        """
+        Singleton magic.
+        """
+        if not hasattr(cls, "self"):
+            cls.self = object.__new__(cls, *args, **kwargs)
+            cls.self.construct()
+        return cls.self
+
+    def __init__(self):
+        """
+        Called multiple times, don't use this.
+        """
+        pass
+
+    def construct(self):
+        """
+        Singleton constructor, runs only once.
+        """
+        pass
+
+    def cleanup(self):
+        pass
+
+    def get_emoji_categories(self):
+        return [label for label, data in self._get_emoji_data()]
+
+    def get_emoji(self, category):
+        return tuple(sequence
+                     for sequence, data in self._get_emoji_data()[category][1])
+
+    def _get_emoji_data(self):
+        return emoji_data
+
+    def get_symbol_categories(self):
+        return [label for label, data in self._get_symbol_data()]
+
+    def get_symbols(self, category):
+        return tuple(self._get_symbol_data()[category][1])
+
+    def _get_symbol_data(self):
+        return self._symbol_data
+
+

=== modified file 'Onboard/WordSuggestions.py'
--- Onboard/WordSuggestions.py	2017-02-02 06:08:40 +0000
+++ Onboard/WordSuggestions.py	2017-02-10 10:33:11 +0000
@@ -2414,10 +2414,7 @@
         return bg_keys + keys + exp_keys, used_rect
 
     def _get_child_button(self, id):
-        items = list(self.find_ids([id]))
-        if items:
-            return items[0]
-        return None
+        return self.find_id(id)
 
     def _create_correction_choices(self, choices, rect,
                                    key_context, font_size,

=== modified file 'Onboard/canonical_equivalents.py'
--- Onboard/canonical_equivalents.py	2016-08-15 15:08:44 +0000
+++ Onboard/canonical_equivalents.py	2017-02-10 10:33:11 +0000
@@ -72,7 +72,7 @@
                 '7' : ['⁷', '₇', '⅐'],
                 '8' : ['⁸', '₈', '⅜', '⅝', '⅞'],
                 '9' : ['⁹', '₉', '⅑'],
-                '0' : ['₀', '⅒'],
+                '0' : ['⁰', '₀', '⅒'],
 
                 '?' : ['¿', '؟'],
                 '!' : ['¡'],

=== added file 'Onboard/emoji_data.py'
--- Onboard/emoji_data.py	1970-01-01 00:00:00 +0000
+++ Onboard/emoji_data.py	2017-02-10 10:33:11 +0000
@@ -0,0 +1,2420 @@
+#
+# Generated for Onboard by gen_unicode_data
+#
+
+emoji_data = [
+    ['😀',                                         # category: Smileys
+        [
+            ('😀', None),                          # 😀 grinning face
+            ('😁', None),                          # 😁 grinning face with smiling eyes
+            ('😂', None),                          # 😂 face with tears of joy
+            ('\U0001f923', None),                 # 🤣 rolling on the floor laughing
+            ('😃', None),                          # 😃 smiling face with open mouth
+            ('😄', None),                          # 😄 smiling face with open mouth & smiling eyes
+            ('😅', None),                          # 😅 smiling face with open mouth & cold sweat
+            ('😆', None),                          # 😆 smiling face with open mouth & closed eyes
+            ('😉', None),                          # 😉 winking face
+            ('😊', None),                          # 😊 smiling face with smiling eyes
+            ('😋', None),                          # 😋 face savouring delicious food
+            ('😎', None),                          # 😎 smiling face with sunglasses
+            ('😍', None),                          # 😍 smiling face with heart-eyes
+            ('😘', None),                          # 😘 face blowing a kiss
+            ('😗', None),                          # 😗 kissing face
+            ('😙', None),                          # 😙 kissing face with smiling eyes
+            ('😚', None),                          # 😚 kissing face with closed eyes
+            ('☺', None),                          # ☺ smiling face
+            ('🙂', None),                          # 🙂 slightly smiling face
+            ('🤗', None),                          # 🤗 hugging face
+            ('🤔', None),                          # 🤔 thinking face
+            ('😐', None),                          # 😐 neutral face
+            ('😑', None),                          # 😑 expressionless face
+            ('😶', None),                          # 😶 face without mouth
+            ('🙄', None),                          # 🙄 face with rolling eyes
+            ('😏', None),                          # 😏 smirking face
+            ('😣', None),                          # 😣 persevering face
+            ('😥', None),                          # 😥 disappointed but relieved face
+            ('😮', None),                          # 😮 face with open mouth
+            ('🤐', None),                          # 🤐 zipper-mouth face
+            ('😯', None),                          # 😯 hushed face
+            ('😪', None),                          # 😪 sleepy face
+            ('😫', None),                          # 😫 tired face
+            ('😴', None),                          # 😴 sleeping face
+            ('😌', None),                          # 😌 relieved face
+            ('🤓', None),                          # 🤓 nerd face
+            ('😛', None),                          # 😛 face with stuck-out tongue
+            ('😜', None),                          # 😜 face with stuck-out tongue & winking eye
+            ('😝', None),                          # 😝 face with stuck-out tongue & closed eyes
+            ('\U0001f924', None),                 # 🤤 drooling face
+            ('😒', None),                          # 😒 unamused face
+            ('😓', None),                          # 😓 face with cold sweat
+            ('😔', None),                          # 😔 pensive face
+            ('😕', None),                          # 😕 confused face
+            ('🙃', None),                          # 🙃 upside-down face
+            ('🤑', None),                          # 🤑 money-mouth face
+            ('😲', None),                          # 😲 astonished face
+            ('☹', None),                          # ☹ frowning face
+            ('🙁', None),                          # 🙁 slightly frowning face
+            ('😖', None),                          # 😖 confounded face
+            ('😞', None),                          # 😞 disappointed face
+            ('😟', None),                          # 😟 worried face
+            ('😤', None),                          # 😤 face with steam from nose
+            ('😢', None),                          # 😢 crying face
+            ('😭', None),                          # 😭 loudly crying face
+            ('😦', None),                          # 😦 frowning face with open mouth
+            ('😧', None),                          # 😧 anguished face
+            ('😨', None),                          # 😨 fearful face
+            ('😩', None),                          # 😩 weary face
+            ('😬', None),                          # 😬 grimacing face
+            ('😰', None),                          # 😰 face with open mouth & cold sweat
+            ('😱', None),                          # 😱 face screaming in fear
+            ('😳', None),                          # 😳 flushed face
+            ('😵', None),                          # 😵 dizzy face
+            ('😡', None),                          # 😡 pouting face
+            ('😠', None),                          # 😠 angry face
+            ('😇', None),                          # 😇 smiling face with halo
+            ('\U0001f920', None),                 # 🤠 cowboy hat face
+            ('\U0001f921', None),                 # 🤡 clown face
+            ('\U0001f925', None),                 # 🤥 lying face
+            ('😷', None),                          # 😷 face with medical mask
+            ('🤒', None),                          # 🤒 face with thermometer
+            ('🤕', None),                          # 🤕 face with head-bandage
+            ('\U0001f922', None),                 # 🤢 nauseated face
+            ('\U0001f927', None),                 # 🤧 sneezing face
+            ('😈', None),                          # 😈 smiling face with horns
+            ('👿', None),                          # 👿 angry face with horns
+            ('👹', None),                          # 👹 ogre
+            ('👺', None),                          # 👺 goblin
+            ('💀', None),                          # 💀 skull
+            ('☠', None),                          # ☠ skull and crossbones
+            ('👻', None),                          # 👻 ghost
+            ('👽', None),                          # 👽 alien
+            ('👾', None),                          # 👾 alien monster
+            ('🤖', None),                          # 🤖 robot face
+            ('💩', None),                          # 💩 pile of poo
+            ('😺', None),                          # 😺 smiling cat face with open mouth
+            ('😸', None),                          # 😸 grinning cat face with smiling eyes
+            ('😹', None),                          # 😹 cat face with tears of joy
+            ('😻', None),                          # 😻 smiling cat face with heart-eyes
+            ('😼', None),                          # 😼 cat face with wry smile
+            ('😽', None),                          # 😽 kissing cat face with closed eyes
+            ('🙀', None),                          # 🙀 weary cat face
+            ('😿', None),                          # 😿 crying cat face
+            ('😾', None),                          # 😾 pouting cat face
+            ('🙈', None),                          # 🙈 see-no-evil monkey
+            ('🙉', None),                          # 🙉 hear-no-evil monkey
+            ('🙊', None),                          # 🙊 speak-no-evil monkey
+        ]],
+    ['👦',                                         # category: People
+        [
+            ('👦',                                 # 👦 boy
+                ('👦🏻',                            # 👦🏻 boy: light skin tone
+                 '👦🏼',                            # 👦🏼 boy: medium-light skin tone
+                 '👦🏽',                            # 👦🏽 boy: medium skin tone
+                 '👦🏾',                            # 👦🏾 boy: medium-dark skin tone
+                 '👦🏿')),                          # 👦🏿 boy: dark skin tone
+            ('👧',                                 # 👧 girl
+                ('👧🏻',                            # 👧🏻 girl: light skin tone
+                 '👧🏼',                            # 👧🏼 girl: medium-light skin tone
+                 '👧🏽',                            # 👧🏽 girl: medium skin tone
+                 '👧🏾',                            # 👧🏾 girl: medium-dark skin tone
+                 '👧🏿')),                          # 👧🏿 girl: dark skin tone
+            ('👨',                                 # 👨 man
+                ('👨🏻',                            # 👨🏻 man: light skin tone
+                 '👨🏼',                            # 👨🏼 man: medium-light skin tone
+                 '👨🏽',                            # 👨🏽 man: medium skin tone
+                 '👨🏾',                            # 👨🏾 man: medium-dark skin tone
+                 '👨🏿')),                          # 👨🏿 man: dark skin tone
+            ('👩',                                 # 👩 woman
+                ('👩🏻',                            # 👩🏻 woman: light skin tone
+                 '👩🏼',                            # 👩🏼 woman: medium-light skin tone
+                 '👩🏽',                            # 👩🏽 woman: medium skin tone
+                 '👩🏾',                            # 👩🏾 woman: medium-dark skin tone
+                 '👩🏿')),                          # 👩🏿 woman: dark skin tone
+            ('👴',                                 # 👴 old man
+                ('👴🏻',                            # 👴🏻 old man: light skin tone
+                 '👴🏼',                            # 👴🏼 old man: medium-light skin tone
+                 '👴🏽',                            # 👴🏽 old man: medium skin tone
+                 '👴🏾',                            # 👴🏾 old man: medium-dark skin tone
+                 '👴🏿')),                          # 👴🏿 old man: dark skin tone
+            ('👵',                                 # 👵 old woman
+                ('👵🏻',                            # 👵🏻 old woman: light skin tone
+                 '👵🏼',                            # 👵🏼 old woman: medium-light skin tone
+                 '👵🏽',                            # 👵🏽 old woman: medium skin tone
+                 '👵🏾',                            # 👵🏾 old woman: medium-dark skin tone
+                 '👵🏿')),                          # 👵🏿 old woman: dark skin tone
+            ('👶',                                 # 👶 baby
+                ('👶🏻',                            # 👶🏻 baby: light skin tone
+                 '👶🏼',                            # 👶🏼 baby: medium-light skin tone
+                 '👶🏽',                            # 👶🏽 baby: medium skin tone
+                 '👶🏾',                            # 👶🏾 baby: medium-dark skin tone
+                 '👶🏿')),                          # 👶🏿 baby: dark skin tone
+            ('👼',                                 # 👼 baby angel
+                ('👼🏻',                            # 👼🏻 baby angel: light skin tone
+                 '👼🏼',                            # 👼🏼 baby angel: medium-light skin tone
+                 '👼🏽',                            # 👼🏽 baby angel: medium skin tone
+                 '👼🏾',                            # 👼🏾 baby angel: medium-dark skin tone
+                 '👼🏿')),                          # 👼🏿 baby angel: dark skin tone
+            ('👨\u200d⚕️',                         # 👨‍⚕️ man health worker
+                ('👨🏻\u200d⚕️',                    # 👨🏻‍⚕️ man health worker: light skin tone
+                 '👨🏼\u200d⚕️',                    # 👨🏼‍⚕️ man health worker: medium-light skin tone
+                 '👨🏽\u200d⚕️',                    # 👨🏽‍⚕️ man health worker: medium skin tone
+                 '👨🏾\u200d⚕️',                    # 👨🏾‍⚕️ man health worker: medium-dark skin tone
+                 '👨🏿\u200d⚕️')),                  # 👨🏿‍⚕️ man health worker: dark skin tone
+            ('👩\u200d⚕️',                         # 👩‍⚕️ woman health worker
+                ('👩🏻\u200d⚕️',                    # 👩🏻‍⚕️ woman health worker: light skin tone
+                 '👩🏼\u200d⚕️',                    # 👩🏼‍⚕️ woman health worker: medium-light skin tone
+                 '👩🏽\u200d⚕️',                    # 👩🏽‍⚕️ woman health worker: medium skin tone
+                 '👩🏾\u200d⚕️',                    # 👩🏾‍⚕️ woman health worker: medium-dark skin tone
+                 '👩🏿\u200d⚕️')),                  # 👩🏿‍⚕️ woman health worker: dark skin tone
+            ('👨\u200d🎓',                          # 👨‍🎓 man student
+                ('👨🏻\u200d🎓',                     # 👨🏻‍🎓 man student: light skin tone
+                 '👨🏼\u200d🎓',                     # 👨🏼‍🎓 man student: medium-light skin tone
+                 '👨🏽\u200d🎓',                     # 👨🏽‍🎓 man student: medium skin tone
+                 '👨🏾\u200d🎓',                     # 👨🏾‍🎓 man student: medium-dark skin tone
+                 '👨🏿\u200d🎓')),                   # 👨🏿‍🎓 man student: dark skin tone
+            ('👩\u200d🎓',                          # 👩‍🎓 woman student
+                ('👩🏻\u200d🎓',                     # 👩🏻‍🎓 woman student: light skin tone
+                 '👩🏼\u200d🎓',                     # 👩🏼‍🎓 woman student: medium-light skin tone
+                 '👩🏽\u200d🎓',                     # 👩🏽‍🎓 woman student: medium skin tone
+                 '👩🏾\u200d🎓',                     # 👩🏾‍🎓 woman student: medium-dark skin tone
+                 '👩🏿\u200d🎓')),                   # 👩🏿‍🎓 woman student: dark skin tone
+            ('👨\u200d🏫',                          # 👨‍🏫 man teacher
+                ('👨🏻\u200d🏫',                     # 👨🏻‍🏫 man teacher: light skin tone
+                 '👨🏼\u200d🏫',                     # 👨🏼‍🏫 man teacher: medium-light skin tone
+                 '👨🏽\u200d🏫',                     # 👨🏽‍🏫 man teacher: medium skin tone
+                 '👨🏾\u200d🏫',                     # 👨🏾‍🏫 man teacher: medium-dark skin tone
+                 '👨🏿\u200d🏫')),                   # 👨🏿‍🏫 man teacher: dark skin tone
+            ('👩\u200d🏫',                          # 👩‍🏫 woman teacher
+                ('👩🏻\u200d🏫',                     # 👩🏻‍🏫 woman teacher: light skin tone
+                 '👩🏼\u200d🏫',                     # 👩🏼‍🏫 woman teacher: medium-light skin tone
+                 '👩🏽\u200d🏫',                     # 👩🏽‍🏫 woman teacher: medium skin tone
+                 '👩🏾\u200d🏫',                     # 👩🏾‍🏫 woman teacher: medium-dark skin tone
+                 '👩🏿\u200d🏫')),                   # 👩🏿‍🏫 woman teacher: dark skin tone
+            ('👨\u200d⚖️',                         # 👨‍⚖️ man judge
+                ('👨🏻\u200d⚖️',                    # 👨🏻‍⚖️ man judge: light skin tone
+                 '👨🏼\u200d⚖️',                    # 👨🏼‍⚖️ man judge: medium-light skin tone
+                 '👨🏽\u200d⚖️',                    # 👨🏽‍⚖️ man judge: medium skin tone
+                 '👨🏾\u200d⚖️',                    # 👨🏾‍⚖️ man judge: medium-dark skin tone
+                 '👨🏿\u200d⚖️')),                  # 👨🏿‍⚖️ man judge: dark skin tone
+            ('👩\u200d⚖️',                         # 👩‍⚖️ woman judge
+                ('👩🏻\u200d⚖️',                    # 👩🏻‍⚖️ woman judge: light skin tone
+                 '👩🏼\u200d⚖️',                    # 👩🏼‍⚖️ woman judge: medium-light skin tone
+                 '👩🏽\u200d⚖️',                    # 👩🏽‍⚖️ woman judge: medium skin tone
+                 '👩🏾\u200d⚖️',                    # 👩🏾‍⚖️ woman judge: medium-dark skin tone
+                 '👩🏿\u200d⚖️')),                  # 👩🏿‍⚖️ woman judge: dark skin tone
+            ('👨\u200d🌾',                          # 👨‍🌾 man farmer
+                ('👨🏻\u200d🌾',                     # 👨🏻‍🌾 man farmer: light skin tone
+                 '👨🏼\u200d🌾',                     # 👨🏼‍🌾 man farmer: medium-light skin tone
+                 '👨🏽\u200d🌾',                     # 👨🏽‍🌾 man farmer: medium skin tone
+                 '👨🏾\u200d🌾',                     # 👨🏾‍🌾 man farmer: medium-dark skin tone
+                 '👨🏿\u200d🌾')),                   # 👨🏿‍🌾 man farmer: dark skin tone
+            ('👩\u200d🌾',                          # 👩‍🌾 woman farmer
+                ('👩🏻\u200d🌾',                     # 👩🏻‍🌾 woman farmer: light skin tone
+                 '👩🏼\u200d🌾',                     # 👩🏼‍🌾 woman farmer: medium-light skin tone
+                 '👩🏽\u200d🌾',                     # 👩🏽‍🌾 woman farmer: medium skin tone
+                 '👩🏾\u200d🌾',                     # 👩🏾‍🌾 woman farmer: medium-dark skin tone
+                 '👩🏿\u200d🌾')),                   # 👩🏿‍🌾 woman farmer: dark skin tone
+            ('👨\u200d🍳',                          # 👨‍🍳 man cook
+                ('👨🏻\u200d🍳',                     # 👨🏻‍🍳 man cook: light skin tone
+                 '👨🏼\u200d🍳',                     # 👨🏼‍🍳 man cook: medium-light skin tone
+                 '👨🏽\u200d🍳',                     # 👨🏽‍🍳 man cook: medium skin tone
+                 '👨🏾\u200d🍳',                     # 👨🏾‍🍳 man cook: medium-dark skin tone
+                 '👨🏿\u200d🍳')),                   # 👨🏿‍🍳 man cook: dark skin tone
+            ('👩\u200d🍳',                          # 👩‍🍳 woman cook
+                ('👩🏻\u200d🍳',                     # 👩🏻‍🍳 woman cook: light skin tone
+                 '👩🏼\u200d🍳',                     # 👩🏼‍🍳 woman cook: medium-light skin tone
+                 '👩🏽\u200d🍳',                     # 👩🏽‍🍳 woman cook: medium skin tone
+                 '👩🏾\u200d🍳',                     # 👩🏾‍🍳 woman cook: medium-dark skin tone
+                 '👩🏿\u200d🍳')),                   # 👩🏿‍🍳 woman cook: dark skin tone
+            ('👨\u200d🔧',                          # 👨‍🔧 man mechanic
+                ('👨🏻\u200d🔧',                     # 👨🏻‍🔧 man mechanic: light skin tone
+                 '👨🏼\u200d🔧',                     # 👨🏼‍🔧 man mechanic: medium-light skin tone
+                 '👨🏽\u200d🔧',                     # 👨🏽‍🔧 man mechanic: medium skin tone
+                 '👨🏾\u200d🔧',                     # 👨🏾‍🔧 man mechanic: medium-dark skin tone
+                 '👨🏿\u200d🔧')),                   # 👨🏿‍🔧 man mechanic: dark skin tone
+            ('👩\u200d🔧',                          # 👩‍🔧 woman mechanic
+                ('👩🏻\u200d🔧',                     # 👩🏻‍🔧 woman mechanic: light skin tone
+                 '👩🏼\u200d🔧',                     # 👩🏼‍🔧 woman mechanic: medium-light skin tone
+                 '👩🏽\u200d🔧',                     # 👩🏽‍🔧 woman mechanic: medium skin tone
+                 '👩🏾\u200d🔧',                     # 👩🏾‍🔧 woman mechanic: medium-dark skin tone
+                 '👩🏿\u200d🔧')),                   # 👩🏿‍🔧 woman mechanic: dark skin tone
+            ('👨\u200d🏭',                          # 👨‍🏭 man factory worker
+                ('👨🏻\u200d🏭',                     # 👨🏻‍🏭 man factory worker: light skin tone
+                 '👨🏼\u200d🏭',                     # 👨🏼‍🏭 man factory worker: medium-light skin tone
+                 '👨🏽\u200d🏭',                     # 👨🏽‍🏭 man factory worker: medium skin tone
+                 '👨🏾\u200d🏭',                     # 👨🏾‍🏭 man factory worker: medium-dark skin tone
+                 '👨🏿\u200d🏭')),                   # 👨🏿‍🏭 man factory worker: dark skin tone
+            ('👩\u200d🏭',                          # 👩‍🏭 woman factory worker
+                ('👩🏻\u200d🏭',                     # 👩🏻‍🏭 woman factory worker: light skin tone
+                 '👩🏼\u200d🏭',                     # 👩🏼‍🏭 woman factory worker: medium-light skin tone
+                 '👩🏽\u200d🏭',                     # 👩🏽‍🏭 woman factory worker: medium skin tone
+                 '👩🏾\u200d🏭',                     # 👩🏾‍🏭 woman factory worker: medium-dark skin tone
+                 '👩🏿\u200d🏭')),                   # 👩🏿‍🏭 woman factory worker: dark skin tone
+            ('👨\u200d💼',                          # 👨‍💼 man office worker
+                ('👨🏻\u200d💼',                     # 👨🏻‍💼 man office worker: light skin tone
+                 '👨🏼\u200d💼',                     # 👨🏼‍💼 man office worker: medium-light skin tone
+                 '👨🏽\u200d💼',                     # 👨🏽‍💼 man office worker: medium skin tone
+                 '👨🏾\u200d💼',                     # 👨🏾‍💼 man office worker: medium-dark skin tone
+                 '👨🏿\u200d💼')),                   # 👨🏿‍💼 man office worker: dark skin tone
+            ('👩\u200d💼',                          # 👩‍💼 woman office worker
+                ('👩🏻\u200d💼',                     # 👩🏻‍💼 woman office worker: light skin tone
+                 '👩🏼\u200d💼',                     # 👩🏼‍💼 woman office worker: medium-light skin tone
+                 '👩🏽\u200d💼',                     # 👩🏽‍💼 woman office worker: medium skin tone
+                 '👩🏾\u200d💼',                     # 👩🏾‍💼 woman office worker: medium-dark skin tone
+                 '👩🏿\u200d💼')),                   # 👩🏿‍💼 woman office worker: dark skin tone
+            ('👨\u200d🔬',                          # 👨‍🔬 man scientist
+                ('👨🏻\u200d🔬',                     # 👨🏻‍🔬 man scientist: light skin tone
+                 '👨🏼\u200d🔬',                     # 👨🏼‍🔬 man scientist: medium-light skin tone
+                 '👨🏽\u200d🔬',                     # 👨🏽‍🔬 man scientist: medium skin tone
+                 '👨🏾\u200d🔬',                     # 👨🏾‍🔬 man scientist: medium-dark skin tone
+                 '👨🏿\u200d🔬')),                   # 👨🏿‍🔬 man scientist: dark skin tone
+            ('👩\u200d🔬',                          # 👩‍🔬 woman scientist
+                ('👩🏻\u200d🔬',                     # 👩🏻‍🔬 woman scientist: light skin tone
+                 '👩🏼\u200d🔬',                     # 👩🏼‍🔬 woman scientist: medium-light skin tone
+                 '👩🏽\u200d🔬',                     # 👩🏽‍🔬 woman scientist: medium skin tone
+                 '👩🏾\u200d🔬',                     # 👩🏾‍🔬 woman scientist: medium-dark skin tone
+                 '👩🏿\u200d🔬')),                   # 👩🏿‍🔬 woman scientist: dark skin tone
+            ('👨\u200d💻',                          # 👨‍💻 man technologist
+                ('👨🏻\u200d💻',                     # 👨🏻‍💻 man technologist: light skin tone
+                 '👨🏼\u200d💻',                     # 👨🏼‍💻 man technologist: medium-light skin tone
+                 '👨🏽\u200d💻',                     # 👨🏽‍💻 man technologist: medium skin tone
+                 '👨🏾\u200d💻',                     # 👨🏾‍💻 man technologist: medium-dark skin tone
+                 '👨🏿\u200d💻')),                   # 👨🏿‍💻 man technologist: dark skin tone
+            ('👩\u200d💻',                          # 👩‍💻 woman technologist
+                ('👩🏻\u200d💻',                     # 👩🏻‍💻 woman technologist: light skin tone
+                 '👩🏼\u200d💻',                     # 👩🏼‍💻 woman technologist: medium-light skin tone
+                 '👩🏽\u200d💻',                     # 👩🏽‍💻 woman technologist: medium skin tone
+                 '👩🏾\u200d💻',                     # 👩🏾‍💻 woman technologist: medium-dark skin tone
+                 '👩🏿\u200d💻')),                   # 👩🏿‍💻 woman technologist: dark skin tone
+            ('👨\u200d🎤',                          # 👨‍🎤 man singer
+                ('👨🏻\u200d🎤',                     # 👨🏻‍🎤 man singer: light skin tone
+                 '👨🏼\u200d🎤',                     # 👨🏼‍🎤 man singer: medium-light skin tone
+                 '👨🏽\u200d🎤',                     # 👨🏽‍🎤 man singer: medium skin tone
+                 '👨🏾\u200d🎤',                     # 👨🏾‍🎤 man singer: medium-dark skin tone
+                 '👨🏿\u200d🎤')),                   # 👨🏿‍🎤 man singer: dark skin tone
+            ('👩\u200d🎤',                          # 👩‍🎤 woman singer
+                ('👩🏻\u200d🎤',                     # 👩🏻‍🎤 woman singer: light skin tone
+                 '👩🏼\u200d🎤',                     # 👩🏼‍🎤 woman singer: medium-light skin tone
+                 '👩🏽\u200d🎤',                     # 👩🏽‍🎤 woman singer: medium skin tone
+                 '👩🏾\u200d🎤',                     # 👩🏾‍🎤 woman singer: medium-dark skin tone
+                 '👩🏿\u200d🎤')),                   # 👩🏿‍🎤 woman singer: dark skin tone
+            ('👨\u200d🎨',                          # 👨‍🎨 man artist
+                ('👨🏻\u200d🎨',                     # 👨🏻‍🎨 man artist: light skin tone
+                 '👨🏼\u200d🎨',                     # 👨🏼‍🎨 man artist: medium-light skin tone
+                 '👨🏽\u200d🎨',                     # 👨🏽‍🎨 man artist: medium skin tone
+                 '👨🏾\u200d🎨',                     # 👨🏾‍🎨 man artist: medium-dark skin tone
+                 '👨🏿\u200d🎨')),                   # 👨🏿‍🎨 man artist: dark skin tone
+            ('👩\u200d🎨',                          # 👩‍🎨 woman artist
+                ('👩🏻\u200d🎨',                     # 👩🏻‍🎨 woman artist: light skin tone
+                 '👩🏼\u200d🎨',                     # 👩🏼‍🎨 woman artist: medium-light skin tone
+                 '👩🏽\u200d🎨',                     # 👩🏽‍🎨 woman artist: medium skin tone
+                 '👩🏾\u200d🎨',                     # 👩🏾‍🎨 woman artist: medium-dark skin tone
+                 '👩🏿\u200d🎨')),                   # 👩🏿‍🎨 woman artist: dark skin tone
+            ('👨\u200d✈️',                         # 👨‍✈️ man pilot
+                ('👨🏻\u200d✈️',                    # 👨🏻‍✈️ man pilot: light skin tone
+                 '👨🏼\u200d✈️',                    # 👨🏼‍✈️ man pilot: medium-light skin tone
+                 '👨🏽\u200d✈️',                    # 👨🏽‍✈️ man pilot: medium skin tone
+                 '👨🏾\u200d✈️',                    # 👨🏾‍✈️ man pilot: medium-dark skin tone
+                 '👨🏿\u200d✈️')),                  # 👨🏿‍✈️ man pilot: dark skin tone
+            ('👩\u200d✈️',                         # 👩‍✈️ woman pilot
+                ('👩🏻\u200d✈️',                    # 👩🏻‍✈️ woman pilot: light skin tone
+                 '👩🏼\u200d✈️',                    # 👩🏼‍✈️ woman pilot: medium-light skin tone
+                 '👩🏽\u200d✈️',                    # 👩🏽‍✈️ woman pilot: medium skin tone
+                 '👩🏾\u200d✈️',                    # 👩🏾‍✈️ woman pilot: medium-dark skin tone
+                 '👩🏿\u200d✈️')),                  # 👩🏿‍✈️ woman pilot: dark skin tone
+            ('👨\u200d🚀',                          # 👨‍🚀 man astronaut
+                ('👨🏻\u200d🚀',                     # 👨🏻‍🚀 man astronaut: light skin tone
+                 '👨🏼\u200d🚀',                     # 👨🏼‍🚀 man astronaut: medium-light skin tone
+                 '👨🏽\u200d🚀',                     # 👨🏽‍🚀 man astronaut: medium skin tone
+                 '👨🏾\u200d🚀',                     # 👨🏾‍🚀 man astronaut: medium-dark skin tone
+                 '👨🏿\u200d🚀')),                   # 👨🏿‍🚀 man astronaut: dark skin tone
+            ('👩\u200d🚀',                          # 👩‍🚀 woman astronaut
+                ('👩🏻\u200d🚀',                     # 👩🏻‍🚀 woman astronaut: light skin tone
+                 '👩🏼\u200d🚀',                     # 👩🏼‍🚀 woman astronaut: medium-light skin tone
+                 '👩🏽\u200d🚀',                     # 👩🏽‍🚀 woman astronaut: medium skin tone
+                 '👩🏾\u200d🚀',                     # 👩🏾‍🚀 woman astronaut: medium-dark skin tone
+                 '👩🏿\u200d🚀')),                   # 👩🏿‍🚀 woman astronaut: dark skin tone
+            ('👨\u200d🚒',                          # 👨‍🚒 man firefighter
+                ('👨🏻\u200d🚒',                     # 👨🏻‍🚒 man firefighter: light skin tone
+                 '👨🏼\u200d🚒',                     # 👨🏼‍🚒 man firefighter: medium-light skin tone
+                 '👨🏽\u200d🚒',                     # 👨🏽‍🚒 man firefighter: medium skin tone
+                 '👨🏾\u200d🚒',                     # 👨🏾‍🚒 man firefighter: medium-dark skin tone
+                 '👨🏿\u200d🚒')),                   # 👨🏿‍🚒 man firefighter: dark skin tone
+            ('👩\u200d🚒',                          # 👩‍🚒 woman firefighter
+                ('👩🏻\u200d🚒',                     # 👩🏻‍🚒 woman firefighter: light skin tone
+                 '👩🏼\u200d🚒',                     # 👩🏼‍🚒 woman firefighter: medium-light skin tone
+                 '👩🏽\u200d🚒',                     # 👩🏽‍🚒 woman firefighter: medium skin tone
+                 '👩🏾\u200d🚒',                     # 👩🏾‍🚒 woman firefighter: medium-dark skin tone
+                 '👩🏿\u200d🚒')),                   # 👩🏿‍🚒 woman firefighter: dark skin tone
+            ('👮',                                 # 👮 police officer
+                ('👮🏻',                            # 👮🏻 police officer: light skin tone
+                 '👮🏼',                            # 👮🏼 police officer: medium-light skin tone
+                 '👮🏽',                            # 👮🏽 police officer: medium skin tone
+                 '👮🏾',                            # 👮🏾 police officer: medium-dark skin tone
+                 '👮🏿')),                          # 👮🏿 police officer: dark skin tone
+            ('👮\u200d♂️',                         # 👮‍♂️ man police officer
+                ('👮🏻\u200d♂️',                    # 👮🏻‍♂️ man police officer: light skin tone
+                 '👮🏼\u200d♂️',                    # 👮🏼‍♂️ man police officer: medium-light skin tone
+                 '👮🏽\u200d♂️',                    # 👮🏽‍♂️ man police officer: medium skin tone
+                 '👮🏾\u200d♂️',                    # 👮🏾‍♂️ man police officer: medium-dark skin tone
+                 '👮🏿\u200d♂️')),                  # 👮🏿‍♂️ man police officer: dark skin tone
+            ('👮\u200d♀️',                         # 👮‍♀️ woman police officer
+                ('👮🏻\u200d♀️',                    # 👮🏻‍♀️ woman police officer: light skin tone
+                 '👮🏼\u200d♀️',                    # 👮🏼‍♀️ woman police officer: medium-light skin tone
+                 '👮🏽\u200d♀️',                    # 👮🏽‍♀️ woman police officer: medium skin tone
+                 '👮🏾\u200d♀️',                    # 👮🏾‍♀️ woman police officer: medium-dark skin tone
+                 '👮🏿\u200d♀️')),                  # 👮🏿‍♀️ woman police officer: dark skin tone
+            ('🕵',                                 # 🕵 detective
+                ('🕵🏻',                            # 🕵🏻 detective: light skin tone
+                 '🕵🏼',                            # 🕵🏼 detective: medium-light skin tone
+                 '🕵🏽',                            # 🕵🏽 detective: medium skin tone
+                 '🕵🏾',                            # 🕵🏾 detective: medium-dark skin tone
+                 '🕵🏿')),                          # 🕵🏿 detective: dark skin tone
+            ('🕵️\u200d♂️',                        # 🕵️‍♂️ man detective
+                ('🕵🏻\u200d♂️',                    # 🕵🏻‍♂️ man detective: light skin tone
+                 '🕵🏼\u200d♂️',                    # 🕵🏼‍♂️ man detective: medium-light skin tone
+                 '🕵🏽\u200d♂️',                    # 🕵🏽‍♂️ man detective: medium skin tone
+                 '🕵🏾\u200d♂️',                    # 🕵🏾‍♂️ man detective: medium-dark skin tone
+                 '🕵🏿\u200d♂️')),                  # 🕵🏿‍♂️ man detective: dark skin tone
+            ('🕵️\u200d♀️',                        # 🕵️‍♀️ woman detective
+                ('🕵🏻\u200d♀️',                    # 🕵🏻‍♀️ woman detective: light skin tone
+                 '🕵🏼\u200d♀️',                    # 🕵🏼‍♀️ woman detective: medium-light skin tone
+                 '🕵🏽\u200d♀️',                    # 🕵🏽‍♀️ woman detective: medium skin tone
+                 '🕵🏾\u200d♀️',                    # 🕵🏾‍♀️ woman detective: medium-dark skin tone
+                 '🕵🏿\u200d♀️')),                  # 🕵🏿‍♀️ woman detective: dark skin tone
+            ('💂',                                 # 💂 guard
+                ('💂🏻',                            # 💂🏻 guard: light skin tone
+                 '💂🏼',                            # 💂🏼 guard: medium-light skin tone
+                 '💂🏽',                            # 💂🏽 guard: medium skin tone
+                 '💂🏾',                            # 💂🏾 guard: medium-dark skin tone
+                 '💂🏿')),                          # 💂🏿 guard: dark skin tone
+            ('💂\u200d♂️',                         # 💂‍♂️ man guard
+                ('💂🏻\u200d♂️',                    # 💂🏻‍♂️ man guard: light skin tone
+                 '💂🏼\u200d♂️',                    # 💂🏼‍♂️ man guard: medium-light skin tone
+                 '💂🏽\u200d♂️',                    # 💂🏽‍♂️ man guard: medium skin tone
+                 '💂🏾\u200d♂️',                    # 💂🏾‍♂️ man guard: medium-dark skin tone
+                 '💂🏿\u200d♂️')),                  # 💂🏿‍♂️ man guard: dark skin tone
+            ('💂\u200d♀️',                         # 💂‍♀️ woman guard
+                ('💂🏻\u200d♀️',                    # 💂🏻‍♀️ woman guard: light skin tone
+                 '💂🏼\u200d♀️',                    # 💂🏼‍♀️ woman guard: medium-light skin tone
+                 '💂🏽\u200d♀️',                    # 💂🏽‍♀️ woman guard: medium skin tone
+                 '💂🏾\u200d♀️',                    # 💂🏾‍♀️ woman guard: medium-dark skin tone
+                 '💂🏿\u200d♀️')),                  # 💂🏿‍♀️ woman guard: dark skin tone
+            ('👷',                                 # 👷 construction worker
+                ('👷🏻',                            # 👷🏻 construction worker: light skin tone
+                 '👷🏼',                            # 👷🏼 construction worker: medium-light skin tone
+                 '👷🏽',                            # 👷🏽 construction worker: medium skin tone
+                 '👷🏾',                            # 👷🏾 construction worker: medium-dark skin tone
+                 '👷🏿')),                          # 👷🏿 construction worker: dark skin tone
+            ('👷\u200d♂️',                         # 👷‍♂️ man construction worker
+                ('👷🏻\u200d♂️',                    # 👷🏻‍♂️ man construction worker: light skin tone
+                 '👷🏼\u200d♂️',                    # 👷🏼‍♂️ man construction worker: medium-light skin tone
+                 '👷🏽\u200d♂️',                    # 👷🏽‍♂️ man construction worker: medium skin tone
+                 '👷🏾\u200d♂️',                    # 👷🏾‍♂️ man construction worker: medium-dark skin tone
+                 '👷🏿\u200d♂️')),                  # 👷🏿‍♂️ man construction worker: dark skin tone
+            ('👷\u200d♀️',                         # 👷‍♀️ woman construction worker
+                ('👷🏻\u200d♀️',                    # 👷🏻‍♀️ woman construction worker: light skin tone
+                 '👷🏼\u200d♀️',                    # 👷🏼‍♀️ woman construction worker: medium-light skin tone
+                 '👷🏽\u200d♀️',                    # 👷🏽‍♀️ woman construction worker: medium skin tone
+                 '👷🏾\u200d♀️',                    # 👷🏾‍♀️ woman construction worker: medium-dark skin tone
+                 '👷🏿\u200d♀️')),                  # 👷🏿‍♀️ woman construction worker: dark skin tone
+            ('👳',                                 # 👳 person wearing turban
+                ('👳🏻',                            # 👳🏻 person wearing turban: light skin tone
+                 '👳🏼',                            # 👳🏼 person wearing turban: medium-light skin tone
+                 '👳🏽',                            # 👳🏽 person wearing turban: medium skin tone
+                 '👳🏾',                            # 👳🏾 person wearing turban: medium-dark skin tone
+                 '👳🏿')),                          # 👳🏿 person wearing turban: dark skin tone
+            ('👳\u200d♂️',                         # 👳‍♂️ man wearing turban
+                ('👳🏻\u200d♂️',                    # 👳🏻‍♂️ man wearing turban: light skin tone
+                 '👳🏼\u200d♂️',                    # 👳🏼‍♂️ man wearing turban: medium-light skin tone
+                 '👳🏽\u200d♂️',                    # 👳🏽‍♂️ man wearing turban: medium skin tone
+                 '👳🏾\u200d♂️',                    # 👳🏾‍♂️ man wearing turban: medium-dark skin tone
+                 '👳🏿\u200d♂️')),                  # 👳🏿‍♂️ man wearing turban: dark skin tone
+            ('👳\u200d♀️',                         # 👳‍♀️ woman wearing turban
+                ('👳🏻\u200d♀️',                    # 👳🏻‍♀️ woman wearing turban: light skin tone
+                 '👳🏼\u200d♀️',                    # 👳🏼‍♀️ woman wearing turban: medium-light skin tone
+                 '👳🏽\u200d♀️',                    # 👳🏽‍♀️ woman wearing turban: medium skin tone
+                 '👳🏾\u200d♀️',                    # 👳🏾‍♀️ woman wearing turban: medium-dark skin tone
+                 '👳🏿\u200d♀️')),                  # 👳🏿‍♀️ woman wearing turban: dark skin tone
+            ('👱',                                 # 👱 blond-haired person
+                ('👱🏻',                            # 👱🏻 blond-haired person: light skin tone
+                 '👱🏼',                            # 👱🏼 blond-haired person: medium-light skin tone
+                 '👱🏽',                            # 👱🏽 blond-haired person: medium skin tone
+                 '👱🏾',                            # 👱🏾 blond-haired person: medium-dark skin tone
+                 '👱🏿')),                          # 👱🏿 blond-haired person: dark skin tone
+            ('👱\u200d♂️',                         # 👱‍♂️ blond-haired man
+                ('👱🏻\u200d♂️',                    # 👱🏻‍♂️ blond-haired man: light skin tone
+                 '👱🏼\u200d♂️',                    # 👱🏼‍♂️ blond-haired man: medium-light skin tone
+                 '👱🏽\u200d♂️',                    # 👱🏽‍♂️ blond-haired man: medium skin tone
+                 '👱🏾\u200d♂️',                    # 👱🏾‍♂️ blond-haired man: medium-dark skin tone
+                 '👱🏿\u200d♂️')),                  # 👱🏿‍♂️ blond-haired man: dark skin tone
+            ('👱\u200d♀️',                         # 👱‍♀️ blond-haired woman
+                ('👱🏻\u200d♀️',                    # 👱🏻‍♀️ blond-haired woman: light skin tone
+                 '👱🏼\u200d♀️',                    # 👱🏼‍♀️ blond-haired woman: medium-light skin tone
+                 '👱🏽\u200d♀️',                    # 👱🏽‍♀️ blond-haired woman: medium skin tone
+                 '👱🏾\u200d♀️',                    # 👱🏾‍♀️ blond-haired woman: medium-dark skin tone
+                 '👱🏿\u200d♀️')),                  # 👱🏿‍♀️ blond-haired woman: dark skin tone
+            ('🎅',                                 # 🎅 Santa Claus
+                ('🎅🏻',                            # 🎅🏻 Santa Claus: light skin tone
+                 '🎅🏼',                            # 🎅🏼 Santa Claus: medium-light skin tone
+                 '🎅🏽',                            # 🎅🏽 Santa Claus: medium skin tone
+                 '🎅🏾',                            # 🎅🏾 Santa Claus: medium-dark skin tone
+                 '🎅🏿')),                          # 🎅🏿 Santa Claus: dark skin tone
+            ('\U0001f936',                        # 🤶 Mrs. Claus
+                ('\U0001f936🏻',                   # 🤶🏻 Mrs. Claus: light skin tone
+                 '\U0001f936🏼',                   # 🤶🏼 Mrs. Claus: medium-light skin tone
+                 '\U0001f936🏽',                   # 🤶🏽 Mrs. Claus: medium skin tone
+                 '\U0001f936🏾',                   # 🤶🏾 Mrs. Claus: medium-dark skin tone
+                 '\U0001f936🏿')),                 # 🤶🏿 Mrs. Claus: dark skin tone
+            ('👸',                                 # 👸 princess
+                ('👸🏻',                            # 👸🏻 princess: light skin tone
+                 '👸🏼',                            # 👸🏼 princess: medium-light skin tone
+                 '👸🏽',                            # 👸🏽 princess: medium skin tone
+                 '👸🏾',                            # 👸🏾 princess: medium-dark skin tone
+                 '👸🏿')),                          # 👸🏿 princess: dark skin tone
+            ('\U0001f934',                        # 🤴 prince
+                ('\U0001f934🏻',                   # 🤴🏻 prince: light skin tone
+                 '\U0001f934🏼',                   # 🤴🏼 prince: medium-light skin tone
+                 '\U0001f934🏽',                   # 🤴🏽 prince: medium skin tone
+                 '\U0001f934🏾',                   # 🤴🏾 prince: medium-dark skin tone
+                 '\U0001f934🏿')),                 # 🤴🏿 prince: dark skin tone
+            ('👰',                                 # 👰 bride with veil
+                ('👰🏻',                            # 👰🏻 bride with veil: light skin tone
+                 '👰🏼',                            # 👰🏼 bride with veil: medium-light skin tone
+                 '👰🏽',                            # 👰🏽 bride with veil: medium skin tone
+                 '👰🏾',                            # 👰🏾 bride with veil: medium-dark skin tone
+                 '👰🏿')),                          # 👰🏿 bride with veil: dark skin tone
+            ('\U0001f935',                        # 🤵 man in tuxedo
+                ('\U0001f935🏻',                   # 🤵🏻 man in tuxedo: light skin tone
+                 '\U0001f935🏼',                   # 🤵🏼 man in tuxedo: medium-light skin tone
+                 '\U0001f935🏽',                   # 🤵🏽 man in tuxedo: medium skin tone
+                 '\U0001f935🏾',                   # 🤵🏾 man in tuxedo: medium-dark skin tone
+                 '\U0001f935🏿')),                 # 🤵🏿 man in tuxedo: dark skin tone
+            ('\U0001f930',                        # 🤰 pregnant woman
+                ('\U0001f930🏻',                   # 🤰🏻 pregnant woman: light skin tone
+                 '\U0001f930🏼',                   # 🤰🏼 pregnant woman: medium-light skin tone
+                 '\U0001f930🏽',                   # 🤰🏽 pregnant woman: medium skin tone
+                 '\U0001f930🏾',                   # 🤰🏾 pregnant woman: medium-dark skin tone
+                 '\U0001f930🏿')),                 # 🤰🏿 pregnant woman: dark skin tone
+            ('👲',                                 # 👲 man with Chinese cap
+                ('👲🏻',                            # 👲🏻 man with Chinese cap: light skin tone
+                 '👲🏼',                            # 👲🏼 man with Chinese cap: medium-light skin tone
+                 '👲🏽',                            # 👲🏽 man with Chinese cap: medium skin tone
+                 '👲🏾',                            # 👲🏾 man with Chinese cap: medium-dark skin tone
+                 '👲🏿')),                          # 👲🏿 man with Chinese cap: dark skin tone
+            ('🙍',                                 # 🙍 person frowning
+                ('🙍🏻',                            # 🙍🏻 person frowning: light skin tone
+                 '🙍🏼',                            # 🙍🏼 person frowning: medium-light skin tone
+                 '🙍🏽',                            # 🙍🏽 person frowning: medium skin tone
+                 '🙍🏾',                            # 🙍🏾 person frowning: medium-dark skin tone
+                 '🙍🏿')),                          # 🙍🏿 person frowning: dark skin tone
+            ('🙍\u200d♂️',                         # 🙍‍♂️ man frowning
+                ('🙍🏻\u200d♂️',                    # 🙍🏻‍♂️ man frowning: light skin tone
+                 '🙍🏼\u200d♂️',                    # 🙍🏼‍♂️ man frowning: medium-light skin tone
+                 '🙍🏽\u200d♂️',                    # 🙍🏽‍♂️ man frowning: medium skin tone
+                 '🙍🏾\u200d♂️',                    # 🙍🏾‍♂️ man frowning: medium-dark skin tone
+                 '🙍🏿\u200d♂️')),                  # 🙍🏿‍♂️ man frowning: dark skin tone
+            ('🙍\u200d♀️',                         # 🙍‍♀️ woman frowning
+                ('🙍🏻\u200d♀️',                    # 🙍🏻‍♀️ woman frowning: light skin tone
+                 '🙍🏼\u200d♀️',                    # 🙍🏼‍♀️ woman frowning: medium-light skin tone
+                 '🙍🏽\u200d♀️',                    # 🙍🏽‍♀️ woman frowning: medium skin tone
+                 '🙍🏾\u200d♀️',                    # 🙍🏾‍♀️ woman frowning: medium-dark skin tone
+                 '🙍🏿\u200d♀️')),                  # 🙍🏿‍♀️ woman frowning: dark skin tone
+            ('🙎',                                 # 🙎 person pouting
+                ('🙎🏻',                            # 🙎🏻 person pouting: light skin tone
+                 '🙎🏼',                            # 🙎🏼 person pouting: medium-light skin tone
+                 '🙎🏽',                            # 🙎🏽 person pouting: medium skin tone
+                 '🙎🏾',                            # 🙎🏾 person pouting: medium-dark skin tone
+                 '🙎🏿')),                          # 🙎🏿 person pouting: dark skin tone
+            ('🙎\u200d♂️',                         # 🙎‍♂️ man pouting
+                ('🙎🏻\u200d♂️',                    # 🙎🏻‍♂️ man pouting: light skin tone
+                 '🙎🏼\u200d♂️',                    # 🙎🏼‍♂️ man pouting: medium-light skin tone
+                 '🙎🏽\u200d♂️',                    # 🙎🏽‍♂️ man pouting: medium skin tone
+                 '🙎🏾\u200d♂️',                    # 🙎🏾‍♂️ man pouting: medium-dark skin tone
+                 '🙎🏿\u200d♂️')),                  # 🙎🏿‍♂️ man pouting: dark skin tone
+            ('🙎\u200d♀️',                         # 🙎‍♀️ woman pouting
+                ('🙎🏻\u200d♀️',                    # 🙎🏻‍♀️ woman pouting: light skin tone
+                 '🙎🏼\u200d♀️',                    # 🙎🏼‍♀️ woman pouting: medium-light skin tone
+                 '🙎🏽\u200d♀️',                    # 🙎🏽‍♀️ woman pouting: medium skin tone
+                 '🙎🏾\u200d♀️',                    # 🙎🏾‍♀️ woman pouting: medium-dark skin tone
+                 '🙎🏿\u200d♀️')),                  # 🙎🏿‍♀️ woman pouting: dark skin tone
+            ('🙅',                                 # 🙅 person gesturing NO
+                ('🙅🏻',                            # 🙅🏻 person gesturing NO: light skin tone
+                 '🙅🏼',                            # 🙅🏼 person gesturing NO: medium-light skin tone
+                 '🙅🏽',                            # 🙅🏽 person gesturing NO: medium skin tone
+                 '🙅🏾',                            # 🙅🏾 person gesturing NO: medium-dark skin tone
+                 '🙅🏿')),                          # 🙅🏿 person gesturing NO: dark skin tone
+            ('🙅\u200d♂️',                         # 🙅‍♂️ man gesturing NO
+                ('🙅🏻\u200d♂️',                    # 🙅🏻‍♂️ man gesturing NO: light skin tone
+                 '🙅🏼\u200d♂️',                    # 🙅🏼‍♂️ man gesturing NO: medium-light skin tone
+                 '🙅🏽\u200d♂️',                    # 🙅🏽‍♂️ man gesturing NO: medium skin tone
+                 '🙅🏾\u200d♂️',                    # 🙅🏾‍♂️ man gesturing NO: medium-dark skin tone
+                 '🙅🏿\u200d♂️')),                  # 🙅🏿‍♂️ man gesturing NO: dark skin tone
+            ('🙅\u200d♀️',                         # 🙅‍♀️ woman gesturing NO
+                ('🙅🏻\u200d♀️',                    # 🙅🏻‍♀️ woman gesturing NO: light skin tone
+                 '🙅🏼\u200d♀️',                    # 🙅🏼‍♀️ woman gesturing NO: medium-light skin tone
+                 '🙅🏽\u200d♀️',                    # 🙅🏽‍♀️ woman gesturing NO: medium skin tone
+                 '🙅🏾\u200d♀️',                    # 🙅🏾‍♀️ woman gesturing NO: medium-dark skin tone
+                 '🙅🏿\u200d♀️')),                  # 🙅🏿‍♀️ woman gesturing NO: dark skin tone
+            ('🙆',                                 # 🙆 person gesturing OK
+                ('🙆🏻',                            # 🙆🏻 person gesturing OK: light skin tone
+                 '🙆🏼',                            # 🙆🏼 person gesturing OK: medium-light skin tone
+                 '🙆🏽',                            # 🙆🏽 person gesturing OK: medium skin tone
+                 '🙆🏾',                            # 🙆🏾 person gesturing OK: medium-dark skin tone
+                 '🙆🏿')),                          # 🙆🏿 person gesturing OK: dark skin tone
+            ('🙆\u200d♂️',                         # 🙆‍♂️ man gesturing OK
+                ('🙆🏻\u200d♂️',                    # 🙆🏻‍♂️ man gesturing OK: light skin tone
+                 '🙆🏼\u200d♂️',                    # 🙆🏼‍♂️ man gesturing OK: medium-light skin tone
+                 '🙆🏽\u200d♂️',                    # 🙆🏽‍♂️ man gesturing OK: medium skin tone
+                 '🙆🏾\u200d♂️',                    # 🙆🏾‍♂️ man gesturing OK: medium-dark skin tone
+                 '🙆🏿\u200d♂️')),                  # 🙆🏿‍♂️ man gesturing OK: dark skin tone
+            ('🙆\u200d♀️',                         # 🙆‍♀️ woman gesturing OK
+                ('🙆🏻\u200d♀️',                    # 🙆🏻‍♀️ woman gesturing OK: light skin tone
+                 '🙆🏼\u200d♀️',                    # 🙆🏼‍♀️ woman gesturing OK: medium-light skin tone
+                 '🙆🏽\u200d♀️',                    # 🙆🏽‍♀️ woman gesturing OK: medium skin tone
+                 '🙆🏾\u200d♀️',                    # 🙆🏾‍♀️ woman gesturing OK: medium-dark skin tone
+                 '🙆🏿\u200d♀️')),                  # 🙆🏿‍♀️ woman gesturing OK: dark skin tone
+            ('💁',                                 # 💁 person tipping hand
+                ('💁🏻',                            # 💁🏻 person tipping hand: light skin tone
+                 '💁🏼',                            # 💁🏼 person tipping hand: medium-light skin tone
+                 '💁🏽',                            # 💁🏽 person tipping hand: medium skin tone
+                 '💁🏾',                            # 💁🏾 person tipping hand: medium-dark skin tone
+                 '💁🏿')),                          # 💁🏿 person tipping hand: dark skin tone
+            ('💁\u200d♂️',                         # 💁‍♂️ man tipping hand
+                ('💁🏻\u200d♂️',                    # 💁🏻‍♂️ man tipping hand: light skin tone
+                 '💁🏼\u200d♂️',                    # 💁🏼‍♂️ man tipping hand: medium-light skin tone
+                 '💁🏽\u200d♂️',                    # 💁🏽‍♂️ man tipping hand: medium skin tone
+                 '💁🏾\u200d♂️',                    # 💁🏾‍♂️ man tipping hand: medium-dark skin tone
+                 '💁🏿\u200d♂️')),                  # 💁🏿‍♂️ man tipping hand: dark skin tone
+            ('💁\u200d♀️',                         # 💁‍♀️ woman tipping hand
+                ('💁🏻\u200d♀️',                    # 💁🏻‍♀️ woman tipping hand: light skin tone
+                 '💁🏼\u200d♀️',                    # 💁🏼‍♀️ woman tipping hand: medium-light skin tone
+                 '💁🏽\u200d♀️',                    # 💁🏽‍♀️ woman tipping hand: medium skin tone
+                 '💁🏾\u200d♀️',                    # 💁🏾‍♀️ woman tipping hand: medium-dark skin tone
+                 '💁🏿\u200d♀️')),                  # 💁🏿‍♀️ woman tipping hand: dark skin tone
+            ('🙋',                                 # 🙋 person raising hand
+                ('🙋🏻',                            # 🙋🏻 person raising hand: light skin tone
+                 '🙋🏼',                            # 🙋🏼 person raising hand: medium-light skin tone
+                 '🙋🏽',                            # 🙋🏽 person raising hand: medium skin tone
+                 '🙋🏾',                            # 🙋🏾 person raising hand: medium-dark skin tone
+                 '🙋🏿')),                          # 🙋🏿 person raising hand: dark skin tone
+            ('🙋\u200d♂️',                         # 🙋‍♂️ man raising hand
+                ('🙋🏻\u200d♂️',                    # 🙋🏻‍♂️ man raising hand: light skin tone
+                 '🙋🏼\u200d♂️',                    # 🙋🏼‍♂️ man raising hand: medium-light skin tone
+                 '🙋🏽\u200d♂️',                    # 🙋🏽‍♂️ man raising hand: medium skin tone
+                 '🙋🏾\u200d♂️',                    # 🙋🏾‍♂️ man raising hand: medium-dark skin tone
+                 '🙋🏿\u200d♂️')),                  # 🙋🏿‍♂️ man raising hand: dark skin tone
+            ('🙋\u200d♀️',                         # 🙋‍♀️ woman raising hand
+                ('🙋🏻\u200d♀️',                    # 🙋🏻‍♀️ woman raising hand: light skin tone
+                 '🙋🏼\u200d♀️',                    # 🙋🏼‍♀️ woman raising hand: medium-light skin tone
+                 '🙋🏽\u200d♀️',                    # 🙋🏽‍♀️ woman raising hand: medium skin tone
+                 '🙋🏾\u200d♀️',                    # 🙋🏾‍♀️ woman raising hand: medium-dark skin tone
+                 '🙋🏿\u200d♀️')),                  # 🙋🏿‍♀️ woman raising hand: dark skin tone
+            ('🙇',                                 # 🙇 person bowing
+                ('🙇🏻',                            # 🙇🏻 person bowing: light skin tone
+                 '🙇🏼',                            # 🙇🏼 person bowing: medium-light skin tone
+                 '🙇🏽',                            # 🙇🏽 person bowing: medium skin tone
+                 '🙇🏾',                            # 🙇🏾 person bowing: medium-dark skin tone
+                 '🙇🏿')),                          # 🙇🏿 person bowing: dark skin tone
+            ('🙇\u200d♂️',                         # 🙇‍♂️ man bowing
+                ('🙇🏻\u200d♂️',                    # 🙇🏻‍♂️ man bowing: light skin tone
+                 '🙇🏼\u200d♂️',                    # 🙇🏼‍♂️ man bowing: medium-light skin tone
+                 '🙇🏽\u200d♂️',                    # 🙇🏽‍♂️ man bowing: medium skin tone
+                 '🙇🏾\u200d♂️',                    # 🙇🏾‍♂️ man bowing: medium-dark skin tone
+                 '🙇🏿\u200d♂️')),                  # 🙇🏿‍♂️ man bowing: dark skin tone
+            ('🙇\u200d♀️',                         # 🙇‍♀️ woman bowing
+                ('🙇🏻\u200d♀️',                    # 🙇🏻‍♀️ woman bowing: light skin tone
+                 '🙇🏼\u200d♀️',                    # 🙇🏼‍♀️ woman bowing: medium-light skin tone
+                 '🙇🏽\u200d♀️',                    # 🙇🏽‍♀️ woman bowing: medium skin tone
+                 '🙇🏾\u200d♀️',                    # 🙇🏾‍♀️ woman bowing: medium-dark skin tone
+                 '🙇🏿\u200d♀️')),                  # 🙇🏿‍♀️ woman bowing: dark skin tone
+            ('\U0001f926',                        # 🤦 person facepalming
+                ('\U0001f926🏻',                   # 🤦🏻 person facepalming: light skin tone
+                 '\U0001f926🏼',                   # 🤦🏼 person facepalming: medium-light skin tone
+                 '\U0001f926🏽',                   # 🤦🏽 person facepalming: medium skin tone
+                 '\U0001f926🏾',                   # 🤦🏾 person facepalming: medium-dark skin tone
+                 '\U0001f926🏿')),                 # 🤦🏿 person facepalming: dark skin tone
+            ('\U0001f926\u200d♂️',                # 🤦‍♂️ man facepalming
+                ('\U0001f926🏻\u200d♂️',           # 🤦🏻‍♂️ man facepalming: light skin tone
+                 '\U0001f926🏼\u200d♂️',           # 🤦🏼‍♂️ man facepalming: medium-light skin tone
+                 '\U0001f926🏽\u200d♂️',           # 🤦🏽‍♂️ man facepalming: medium skin tone
+                 '\U0001f926🏾\u200d♂️',           # 🤦🏾‍♂️ man facepalming: medium-dark skin tone
+                 '\U0001f926🏿\u200d♂️')),         # 🤦🏿‍♂️ man facepalming: dark skin tone
+            ('\U0001f926\u200d♀️',                # 🤦‍♀️ woman facepalming
+                ('\U0001f926🏻\u200d♀️',           # 🤦🏻‍♀️ woman facepalming: light skin tone
+                 '\U0001f926🏼\u200d♀️',           # 🤦🏼‍♀️ woman facepalming: medium-light skin tone
+                 '\U0001f926🏽\u200d♀️',           # 🤦🏽‍♀️ woman facepalming: medium skin tone
+                 '\U0001f926🏾\u200d♀️',           # 🤦🏾‍♀️ woman facepalming: medium-dark skin tone
+                 '\U0001f926🏿\u200d♀️')),         # 🤦🏿‍♀️ woman facepalming: dark skin tone
+            ('\U0001f937',                        # 🤷 person shrugging
+                ('\U0001f937🏻',                   # 🤷🏻 person shrugging: light skin tone
+                 '\U0001f937🏼',                   # 🤷🏼 person shrugging: medium-light skin tone
+                 '\U0001f937🏽',                   # 🤷🏽 person shrugging: medium skin tone
+                 '\U0001f937🏾',                   # 🤷🏾 person shrugging: medium-dark skin tone
+                 '\U0001f937🏿')),                 # 🤷🏿 person shrugging: dark skin tone
+            ('\U0001f937\u200d♂️',                # 🤷‍♂️ man shrugging
+                ('\U0001f937🏻\u200d♂️',           # 🤷🏻‍♂️ man shrugging: light skin tone
+                 '\U0001f937🏼\u200d♂️',           # 🤷🏼‍♂️ man shrugging: medium-light skin tone
+                 '\U0001f937🏽\u200d♂️',           # 🤷🏽‍♂️ man shrugging: medium skin tone
+                 '\U0001f937🏾\u200d♂️',           # 🤷🏾‍♂️ man shrugging: medium-dark skin tone
+                 '\U0001f937🏿\u200d♂️')),         # 🤷🏿‍♂️ man shrugging: dark skin tone
+            ('\U0001f937\u200d♀️',                # 🤷‍♀️ woman shrugging
+                ('\U0001f937🏻\u200d♀️',           # 🤷🏻‍♀️ woman shrugging: light skin tone
+                 '\U0001f937🏼\u200d♀️',           # 🤷🏼‍♀️ woman shrugging: medium-light skin tone
+                 '\U0001f937🏽\u200d♀️',           # 🤷🏽‍♀️ woman shrugging: medium skin tone
+                 '\U0001f937🏾\u200d♀️',           # 🤷🏾‍♀️ woman shrugging: medium-dark skin tone
+                 '\U0001f937🏿\u200d♀️')),         # 🤷🏿‍♀️ woman shrugging: dark skin tone
+            ('💆',                                 # 💆 person getting massage
+                ('💆🏻',                            # 💆🏻 person getting massage: light skin tone
+                 '💆🏼',                            # 💆🏼 person getting massage: medium-light skin tone
+                 '💆🏽',                            # 💆🏽 person getting massage: medium skin tone
+                 '💆🏾',                            # 💆🏾 person getting massage: medium-dark skin tone
+                 '💆🏿')),                          # 💆🏿 person getting massage: dark skin tone
+            ('💆\u200d♂️',                         # 💆‍♂️ man getting massage
+                ('💆🏻\u200d♂️',                    # 💆🏻‍♂️ man getting massage: light skin tone
+                 '💆🏼\u200d♂️',                    # 💆🏼‍♂️ man getting massage: medium-light skin tone
+                 '💆🏽\u200d♂️',                    # 💆🏽‍♂️ man getting massage: medium skin tone
+                 '💆🏾\u200d♂️',                    # 💆🏾‍♂️ man getting massage: medium-dark skin tone
+                 '💆🏿\u200d♂️')),                  # 💆🏿‍♂️ man getting massage: dark skin tone
+            ('💆\u200d♀️',                         # 💆‍♀️ woman getting massage
+                ('💆🏻\u200d♀️',                    # 💆🏻‍♀️ woman getting massage: light skin tone
+                 '💆🏼\u200d♀️',                    # 💆🏼‍♀️ woman getting massage: medium-light skin tone
+                 '💆🏽\u200d♀️',                    # 💆🏽‍♀️ woman getting massage: medium skin tone
+                 '💆🏾\u200d♀️',                    # 💆🏾‍♀️ woman getting massage: medium-dark skin tone
+                 '💆🏿\u200d♀️')),                  # 💆🏿‍♀️ woman getting massage: dark skin tone
+            ('💇',                                 # 💇 person getting haircut
+                ('💇🏻',                            # 💇🏻 person getting haircut: light skin tone
+                 '💇🏼',                            # 💇🏼 person getting haircut: medium-light skin tone
+                 '💇🏽',                            # 💇🏽 person getting haircut: medium skin tone
+                 '💇🏾',                            # 💇🏾 person getting haircut: medium-dark skin tone
+                 '💇🏿')),                          # 💇🏿 person getting haircut: dark skin tone
+            ('💇\u200d♂️',                         # 💇‍♂️ man getting haircut
+                ('💇🏻\u200d♂️',                    # 💇🏻‍♂️ man getting haircut: light skin tone
+                 '💇🏼\u200d♂️',                    # 💇🏼‍♂️ man getting haircut: medium-light skin tone
+                 '💇🏽\u200d♂️',                    # 💇🏽‍♂️ man getting haircut: medium skin tone
+                 '💇🏾\u200d♂️',                    # 💇🏾‍♂️ man getting haircut: medium-dark skin tone
+                 '💇🏿\u200d♂️')),                  # 💇🏿‍♂️ man getting haircut: dark skin tone
+            ('💇\u200d♀️',                         # 💇‍♀️ woman getting haircut
+                ('💇🏻\u200d♀️',                    # 💇🏻‍♀️ woman getting haircut: light skin tone
+                 '💇🏼\u200d♀️',                    # 💇🏼‍♀️ woman getting haircut: medium-light skin tone
+                 '💇🏽\u200d♀️',                    # 💇🏽‍♀️ woman getting haircut: medium skin tone
+                 '💇🏾\u200d♀️',                    # 💇🏾‍♀️ woman getting haircut: medium-dark skin tone
+                 '💇🏿\u200d♀️')),                  # 💇🏿‍♀️ woman getting haircut: dark skin tone
+            ('🚶',                                 # 🚶 person walking
+                ('🚶🏻',                            # 🚶🏻 person walking: light skin tone
+                 '🚶🏼',                            # 🚶🏼 person walking: medium-light skin tone
+                 '🚶🏽',                            # 🚶🏽 person walking: medium skin tone
+                 '🚶🏾',                            # 🚶🏾 person walking: medium-dark skin tone
+                 '🚶🏿')),                          # 🚶🏿 person walking: dark skin tone
+            ('🚶\u200d♂️',                         # 🚶‍♂️ man walking
+                ('🚶🏻\u200d♂️',                    # 🚶🏻‍♂️ man walking: light skin tone
+                 '🚶🏼\u200d♂️',                    # 🚶🏼‍♂️ man walking: medium-light skin tone
+                 '🚶🏽\u200d♂️',                    # 🚶🏽‍♂️ man walking: medium skin tone
+                 '🚶🏾\u200d♂️',                    # 🚶🏾‍♂️ man walking: medium-dark skin tone
+                 '🚶🏿\u200d♂️')),                  # 🚶🏿‍♂️ man walking: dark skin tone
+            ('🚶\u200d♀️',                         # 🚶‍♀️ woman walking
+                ('🚶🏻\u200d♀️',                    # 🚶🏻‍♀️ woman walking: light skin tone
+                 '🚶🏼\u200d♀️',                    # 🚶🏼‍♀️ woman walking: medium-light skin tone
+                 '🚶🏽\u200d♀️',                    # 🚶🏽‍♀️ woman walking: medium skin tone
+                 '🚶🏾\u200d♀️',                    # 🚶🏾‍♀️ woman walking: medium-dark skin tone
+                 '🚶🏿\u200d♀️')),                  # 🚶🏿‍♀️ woman walking: dark skin tone
+            ('🏃',                                 # 🏃 person running
+                ('🏃🏻',                            # 🏃🏻 person running: light skin tone
+                 '🏃🏼',                            # 🏃🏼 person running: medium-light skin tone
+                 '🏃🏽',                            # 🏃🏽 person running: medium skin tone
+                 '🏃🏾',                            # 🏃🏾 person running: medium-dark skin tone
+                 '🏃🏿')),                          # 🏃🏿 person running: dark skin tone
+            ('🏃\u200d♂️',                         # 🏃‍♂️ man running
+                ('🏃🏻\u200d♂️',                    # 🏃🏻‍♂️ man running: light skin tone
+                 '🏃🏼\u200d♂️',                    # 🏃🏼‍♂️ man running: medium-light skin tone
+                 '🏃🏽\u200d♂️',                    # 🏃🏽‍♂️ man running: medium skin tone
+                 '🏃🏾\u200d♂️',                    # 🏃🏾‍♂️ man running: medium-dark skin tone
+                 '🏃🏿\u200d♂️')),                  # 🏃🏿‍♂️ man running: dark skin tone
+            ('🏃\u200d♀️',                         # 🏃‍♀️ woman running
+                ('🏃🏻\u200d♀️',                    # 🏃🏻‍♀️ woman running: light skin tone
+                 '🏃🏼\u200d♀️',                    # 🏃🏼‍♀️ woman running: medium-light skin tone
+                 '🏃🏽\u200d♀️',                    # 🏃🏽‍♀️ woman running: medium skin tone
+                 '🏃🏾\u200d♀️',                    # 🏃🏾‍♀️ woman running: medium-dark skin tone
+                 '🏃🏿\u200d♀️')),                  # 🏃🏿‍♀️ woman running: dark skin tone
+            ('💃',                                 # 💃 woman dancing
+                ('💃🏻',                            # 💃🏻 woman dancing: light skin tone
+                 '💃🏼',                            # 💃🏼 woman dancing: medium-light skin tone
+                 '💃🏽',                            # 💃🏽 woman dancing: medium skin tone
+                 '💃🏾',                            # 💃🏾 woman dancing: medium-dark skin tone
+                 '💃🏿')),                          # 💃🏿 woman dancing: dark skin tone
+            ('\U0001f57a',                        # 🕺 man dancing
+                ('\U0001f57a🏻',                   # 🕺🏻 man dancing: light skin tone
+                 '\U0001f57a🏼',                   # 🕺🏼 man dancing: medium-light skin tone
+                 '\U0001f57a🏽',                   # 🕺🏽 man dancing: medium skin tone
+                 '\U0001f57a🏾',                   # 🕺🏾 man dancing: medium-dark skin tone
+                 '\U0001f57a🏿')),                 # 🕺🏿 man dancing: dark skin tone
+            ('👯', None),                          # 👯 people with bunny ears partying
+            ('👯\u200d♂️', None),                  # 👯‍♂️ men with bunny ears partying
+            ('👯\u200d♀️', None),                  # 👯‍♀️ women with bunny ears partying
+            ('🕴',                                 # 🕴 man in business suit levitating
+                ('🕴🏻',                            # 🕴🏻 man in business suit levitating: light skin tone
+                 '🕴🏼',                            # 🕴🏼 man in business suit levitating: medium-light skin tone
+                 '🕴🏽',                            # 🕴🏽 man in business suit levitating: medium skin tone
+                 '🕴🏾',                            # 🕴🏾 man in business suit levitating: medium-dark skin tone
+                 '🕴🏿')),                          # 🕴🏿 man in business suit levitating: dark skin tone
+            ('🗣', None),                          # 🗣 speaking head
+            ('👤', None),                          # 👤 bust in silhouette
+            ('👥', None),                          # 👥 busts in silhouette
+            ('\U0001f93a', None),                 # 🤺 person fencing
+            ('🏇',                                 # 🏇 horse racing
+                ('🏇🏻',                            # 🏇🏻 horse racing: light skin tone
+                 '🏇🏼',                            # 🏇🏼 horse racing: medium-light skin tone
+                 '🏇🏽',                            # 🏇🏽 horse racing: medium skin tone
+                 '🏇🏾',                            # 🏇🏾 horse racing: medium-dark skin tone
+                 '🏇🏿')),                          # 🏇🏿 horse racing: dark skin tone
+            ('⛷', None),                          # ⛷ skier
+            ('🏂',                                 # 🏂 snowboarder
+                ('🏂🏻',                            # 🏂🏻 snowboarder: light skin tone
+                 '🏂🏼',                            # 🏂🏼 snowboarder: medium-light skin tone
+                 '🏂🏽',                            # 🏂🏽 snowboarder: medium skin tone
+                 '🏂🏾',                            # 🏂🏾 snowboarder: medium-dark skin tone
+                 '🏂🏿')),                          # 🏂🏿 snowboarder: dark skin tone
+            ('🏌',                                 # 🏌 person golfing
+                ('🏌🏻',                            # 🏌🏻 person golfing: light skin tone
+                 '🏌🏼',                            # 🏌🏼 person golfing: medium-light skin tone
+                 '🏌🏽',                            # 🏌🏽 person golfing: medium skin tone
+                 '🏌🏾',                            # 🏌🏾 person golfing: medium-dark skin tone
+                 '🏌🏿')),                          # 🏌🏿 person golfing: dark skin tone
+            ('🏌️\u200d♂️',                        # 🏌️‍♂️ man golfing
+                ('🏌🏻\u200d♂️',                    # 🏌🏻‍♂️ man golfing: light skin tone
+                 '🏌🏼\u200d♂️',                    # 🏌🏼‍♂️ man golfing: medium-light skin tone
+                 '🏌🏽\u200d♂️',                    # 🏌🏽‍♂️ man golfing: medium skin tone
+                 '🏌🏾\u200d♂️',                    # 🏌🏾‍♂️ man golfing: medium-dark skin tone
+                 '🏌🏿\u200d♂️')),                  # 🏌🏿‍♂️ man golfing: dark skin tone
+            ('🏌️\u200d♀️',                        # 🏌️‍♀️ woman golfing
+                ('🏌🏻\u200d♀️',                    # 🏌🏻‍♀️ woman golfing: light skin tone
+                 '🏌🏼\u200d♀️',                    # 🏌🏼‍♀️ woman golfing: medium-light skin tone
+                 '🏌🏽\u200d♀️',                    # 🏌🏽‍♀️ woman golfing: medium skin tone
+                 '🏌🏾\u200d♀️',                    # 🏌🏾‍♀️ woman golfing: medium-dark skin tone
+                 '🏌🏿\u200d♀️')),                  # 🏌🏿‍♀️ woman golfing: dark skin tone
+            ('🏄',                                 # 🏄 person surfing
+                ('🏄🏻',                            # 🏄🏻 person surfing: light skin tone
+                 '🏄🏼',                            # 🏄🏼 person surfing: medium-light skin tone
+                 '🏄🏽',                            # 🏄🏽 person surfing: medium skin tone
+                 '🏄🏾',                            # 🏄🏾 person surfing: medium-dark skin tone
+                 '🏄🏿')),                          # 🏄🏿 person surfing: dark skin tone
+            ('🏄\u200d♂️',                         # 🏄‍♂️ man surfing
+                ('🏄🏻\u200d♂️',                    # 🏄🏻‍♂️ man surfing: light skin tone
+                 '🏄🏼\u200d♂️',                    # 🏄🏼‍♂️ man surfing: medium-light skin tone
+                 '🏄🏽\u200d♂️',                    # 🏄🏽‍♂️ man surfing: medium skin tone
+                 '🏄🏾\u200d♂️',                    # 🏄🏾‍♂️ man surfing: medium-dark skin tone
+                 '🏄🏿\u200d♂️')),                  # 🏄🏿‍♂️ man surfing: dark skin tone
+            ('🏄\u200d♀️',                         # 🏄‍♀️ woman surfing
+                ('🏄🏻\u200d♀️',                    # 🏄🏻‍♀️ woman surfing: light skin tone
+                 '🏄🏼\u200d♀️',                    # 🏄🏼‍♀️ woman surfing: medium-light skin tone
+                 '🏄🏽\u200d♀️',                    # 🏄🏽‍♀️ woman surfing: medium skin tone
+                 '🏄🏾\u200d♀️',                    # 🏄🏾‍♀️ woman surfing: medium-dark skin tone
+                 '🏄🏿\u200d♀️')),                  # 🏄🏿‍♀️ woman surfing: dark skin tone
+            ('🚣',                                 # 🚣 person rowing boat
+                ('🚣🏻',                            # 🚣🏻 person rowing boat: light skin tone
+                 '🚣🏼',                            # 🚣🏼 person rowing boat: medium-light skin tone
+                 '🚣🏽',                            # 🚣🏽 person rowing boat: medium skin tone
+                 '🚣🏾',                            # 🚣🏾 person rowing boat: medium-dark skin tone
+                 '🚣🏿')),                          # 🚣🏿 person rowing boat: dark skin tone
+            ('🚣\u200d♂️',                         # 🚣‍♂️ man rowing boat
+                ('🚣🏻\u200d♂️',                    # 🚣🏻‍♂️ man rowing boat: light skin tone
+                 '🚣🏼\u200d♂️',                    # 🚣🏼‍♂️ man rowing boat: medium-light skin tone
+                 '🚣🏽\u200d♂️',                    # 🚣🏽‍♂️ man rowing boat: medium skin tone
+                 '🚣🏾\u200d♂️',                    # 🚣🏾‍♂️ man rowing boat: medium-dark skin tone
+                 '🚣🏿\u200d♂️')),                  # 🚣🏿‍♂️ man rowing boat: dark skin tone
+            ('🚣\u200d♀️',                         # 🚣‍♀️ woman rowing boat
+                ('🚣🏻\u200d♀️',                    # 🚣🏻‍♀️ woman rowing boat: light skin tone
+                 '🚣🏼\u200d♀️',                    # 🚣🏼‍♀️ woman rowing boat: medium-light skin tone
+                 '🚣🏽\u200d♀️',                    # 🚣🏽‍♀️ woman rowing boat: medium skin tone
+                 '🚣🏾\u200d♀️',                    # 🚣🏾‍♀️ woman rowing boat: medium-dark skin tone
+                 '🚣🏿\u200d♀️')),                  # 🚣🏿‍♀️ woman rowing boat: dark skin tone
+            ('🏊',                                 # 🏊 person swimming
+                ('🏊🏻',                            # 🏊🏻 person swimming: light skin tone
+                 '🏊🏼',                            # 🏊🏼 person swimming: medium-light skin tone
+                 '🏊🏽',                            # 🏊🏽 person swimming: medium skin tone
+                 '🏊🏾',                            # 🏊🏾 person swimming: medium-dark skin tone
+                 '🏊🏿')),                          # 🏊🏿 person swimming: dark skin tone
+            ('🏊\u200d♂️',                         # 🏊‍♂️ man swimming
+                ('🏊🏻\u200d♂️',                    # 🏊🏻‍♂️ man swimming: light skin tone
+                 '🏊🏼\u200d♂️',                    # 🏊🏼‍♂️ man swimming: medium-light skin tone
+                 '🏊🏽\u200d♂️',                    # 🏊🏽‍♂️ man swimming: medium skin tone
+                 '🏊🏾\u200d♂️',                    # 🏊🏾‍♂️ man swimming: medium-dark skin tone
+                 '🏊🏿\u200d♂️')),                  # 🏊🏿‍♂️ man swimming: dark skin tone
+            ('🏊\u200d♀️',                         # 🏊‍♀️ woman swimming
+                ('🏊🏻\u200d♀️',                    # 🏊🏻‍♀️ woman swimming: light skin tone
+                 '🏊🏼\u200d♀️',                    # 🏊🏼‍♀️ woman swimming: medium-light skin tone
+                 '🏊🏽\u200d♀️',                    # 🏊🏽‍♀️ woman swimming: medium skin tone
+                 '🏊🏾\u200d♀️',                    # 🏊🏾‍♀️ woman swimming: medium-dark skin tone
+                 '🏊🏿\u200d♀️')),                  # 🏊🏿‍♀️ woman swimming: dark skin tone
+            ('⛹',                                 # ⛹ person bouncing ball
+                ('⛹🏻',                            # ⛹🏻 person bouncing ball: light skin tone
+                 '⛹🏼',                            # ⛹🏼 person bouncing ball: medium-light skin tone
+                 '⛹🏽',                            # ⛹🏽 person bouncing ball: medium skin tone
+                 '⛹🏾',                            # ⛹🏾 person bouncing ball: medium-dark skin tone
+                 '⛹🏿')),                          # ⛹🏿 person bouncing ball: dark skin tone
+            ('⛹️\u200d♂️',                        # ⛹️‍♂️ man bouncing ball
+                ('⛹🏻\u200d♂️',                    # ⛹🏻‍♂️ man bouncing ball: light skin tone
+                 '⛹🏼\u200d♂️',                    # ⛹🏼‍♂️ man bouncing ball: medium-light skin tone
+                 '⛹🏽\u200d♂️',                    # ⛹🏽‍♂️ man bouncing ball: medium skin tone
+                 '⛹🏾\u200d♂️',                    # ⛹🏾‍♂️ man bouncing ball: medium-dark skin tone
+                 '⛹🏿\u200d♂️')),                  # ⛹🏿‍♂️ man bouncing ball: dark skin tone
+            ('⛹️\u200d♀️',                        # ⛹️‍♀️ woman bouncing ball
+                ('⛹🏻\u200d♀️',                    # ⛹🏻‍♀️ woman bouncing ball: light skin tone
+                 '⛹🏼\u200d♀️',                    # ⛹🏼‍♀️ woman bouncing ball: medium-light skin tone
+                 '⛹🏽\u200d♀️',                    # ⛹🏽‍♀️ woman bouncing ball: medium skin tone
+                 '⛹🏾\u200d♀️',                    # ⛹🏾‍♀️ woman bouncing ball: medium-dark skin tone
+                 '⛹🏿\u200d♀️')),                  # ⛹🏿‍♀️ woman bouncing ball: dark skin tone
+            ('🏋',                                 # 🏋 person lifting weights
+                ('🏋🏻',                            # 🏋🏻 person lifting weights: light skin tone
+                 '🏋🏼',                            # 🏋🏼 person lifting weights: medium-light skin tone
+                 '🏋🏽',                            # 🏋🏽 person lifting weights: medium skin tone
+                 '🏋🏾',                            # 🏋🏾 person lifting weights: medium-dark skin tone
+                 '🏋🏿')),                          # 🏋🏿 person lifting weights: dark skin tone
+            ('🏋️\u200d♂️',                        # 🏋️‍♂️ man lifting weights
+                ('🏋🏻\u200d♂️',                    # 🏋🏻‍♂️ man lifting weights: light skin tone
+                 '🏋🏼\u200d♂️',                    # 🏋🏼‍♂️ man lifting weights: medium-light skin tone
+                 '🏋🏽\u200d♂️',                    # 🏋🏽‍♂️ man lifting weights: medium skin tone
+                 '🏋🏾\u200d♂️',                    # 🏋🏾‍♂️ man lifting weights: medium-dark skin tone
+                 '🏋🏿\u200d♂️')),                  # 🏋🏿‍♂️ man lifting weights: dark skin tone
+            ('🏋️\u200d♀️',                        # 🏋️‍♀️ woman lifting weights
+                ('🏋🏻\u200d♀️',                    # 🏋🏻‍♀️ woman lifting weights: light skin tone
+                 '🏋🏼\u200d♀️',                    # 🏋🏼‍♀️ woman lifting weights: medium-light skin tone
+                 '🏋🏽\u200d♀️',                    # 🏋🏽‍♀️ woman lifting weights: medium skin tone
+                 '🏋🏾\u200d♀️',                    # 🏋🏾‍♀️ woman lifting weights: medium-dark skin tone
+                 '🏋🏿\u200d♀️')),                  # 🏋🏿‍♀️ woman lifting weights: dark skin tone
+            ('🚴',                                 # 🚴 person biking
+                ('🚴🏻',                            # 🚴🏻 person biking: light skin tone
+                 '🚴🏼',                            # 🚴🏼 person biking: medium-light skin tone
+                 '🚴🏽',                            # 🚴🏽 person biking: medium skin tone
+                 '🚴🏾',                            # 🚴🏾 person biking: medium-dark skin tone
+                 '🚴🏿')),                          # 🚴🏿 person biking: dark skin tone
+            ('🚴\u200d♂️',                         # 🚴‍♂️ man biking
+                ('🚴🏻\u200d♂️',                    # 🚴🏻‍♂️ man biking: light skin tone
+                 '🚴🏼\u200d♂️',                    # 🚴🏼‍♂️ man biking: medium-light skin tone
+                 '🚴🏽\u200d♂️',                    # 🚴🏽‍♂️ man biking: medium skin tone
+                 '🚴🏾\u200d♂️',                    # 🚴🏾‍♂️ man biking: medium-dark skin tone
+                 '🚴🏿\u200d♂️')),                  # 🚴🏿‍♂️ man biking: dark skin tone
+            ('🚴\u200d♀️',                         # 🚴‍♀️ woman biking
+                ('🚴🏻\u200d♀️',                    # 🚴🏻‍♀️ woman biking: light skin tone
+                 '🚴🏼\u200d♀️',                    # 🚴🏼‍♀️ woman biking: medium-light skin tone
+                 '🚴🏽\u200d♀️',                    # 🚴🏽‍♀️ woman biking: medium skin tone
+                 '🚴🏾\u200d♀️',                    # 🚴🏾‍♀️ woman biking: medium-dark skin tone
+                 '🚴🏿\u200d♀️')),                  # 🚴🏿‍♀️ woman biking: dark skin tone
+            ('🚵',                                 # 🚵 person mountain biking
+                ('🚵🏻',                            # 🚵🏻 person mountain biking: light skin tone
+                 '🚵🏼',                            # 🚵🏼 person mountain biking: medium-light skin tone
+                 '🚵🏽',                            # 🚵🏽 person mountain biking: medium skin tone
+                 '🚵🏾',                            # 🚵🏾 person mountain biking: medium-dark skin tone
+                 '🚵🏿')),                          # 🚵🏿 person mountain biking: dark skin tone
+            ('🚵\u200d♂️',                         # 🚵‍♂️ man mountain biking
+                ('🚵🏻\u200d♂️',                    # 🚵🏻‍♂️ man mountain biking: light skin tone
+                 '🚵🏼\u200d♂️',                    # 🚵🏼‍♂️ man mountain biking: medium-light skin tone
+                 '🚵🏽\u200d♂️',                    # 🚵🏽‍♂️ man mountain biking: medium skin tone
+                 '🚵🏾\u200d♂️',                    # 🚵🏾‍♂️ man mountain biking: medium-dark skin tone
+                 '🚵🏿\u200d♂️')),                  # 🚵🏿‍♂️ man mountain biking: dark skin tone
+            ('🚵\u200d♀️',                         # 🚵‍♀️ woman mountain biking
+                ('🚵🏻\u200d♀️',                    # 🚵🏻‍♀️ woman mountain biking: light skin tone
+                 '🚵🏼\u200d♀️',                    # 🚵🏼‍♀️ woman mountain biking: medium-light skin tone
+                 '🚵🏽\u200d♀️',                    # 🚵🏽‍♀️ woman mountain biking: medium skin tone
+                 '🚵🏾\u200d♀️',                    # 🚵🏾‍♀️ woman mountain biking: medium-dark skin tone
+                 '🚵🏿\u200d♀️')),                  # 🚵🏿‍♀️ woman mountain biking: dark skin tone
+            ('🏎', None),                          # 🏎 racing car
+            ('🏍', None),                          # 🏍 motorcycle
+            ('\U0001f938',                        # 🤸 person cartwheeling
+                ('\U0001f938🏻',                   # 🤸🏻 person cartwheeling: light skin tone
+                 '\U0001f938🏼',                   # 🤸🏼 person cartwheeling: medium-light skin tone
+                 '\U0001f938🏽',                   # 🤸🏽 person cartwheeling: medium skin tone
+                 '\U0001f938🏾',                   # 🤸🏾 person cartwheeling: medium-dark skin tone
+                 '\U0001f938🏿')),                 # 🤸🏿 person cartwheeling: dark skin tone
+            ('\U0001f938\u200d♂️',                # 🤸‍♂️ man cartwheeling
+                ('\U0001f938🏻\u200d♂️',           # 🤸🏻‍♂️ man cartwheeling: light skin tone
+                 '\U0001f938🏼\u200d♂️',           # 🤸🏼‍♂️ man cartwheeling: medium-light skin tone
+                 '\U0001f938🏽\u200d♂️',           # 🤸🏽‍♂️ man cartwheeling: medium skin tone
+                 '\U0001f938🏾\u200d♂️',           # 🤸🏾‍♂️ man cartwheeling: medium-dark skin tone
+                 '\U0001f938🏿\u200d♂️')),         # 🤸🏿‍♂️ man cartwheeling: dark skin tone
+            ('\U0001f938\u200d♀️',                # 🤸‍♀️ woman cartwheeling
+                ('\U0001f938🏻\u200d♀️',           # 🤸🏻‍♀️ woman cartwheeling: light skin tone
+                 '\U0001f938🏼\u200d♀️',           # 🤸🏼‍♀️ woman cartwheeling: medium-light skin tone
+                 '\U0001f938🏽\u200d♀️',           # 🤸🏽‍♀️ woman cartwheeling: medium skin tone
+                 '\U0001f938🏾\u200d♀️',           # 🤸🏾‍♀️ woman cartwheeling: medium-dark skin tone
+                 '\U0001f938🏿\u200d♀️')),         # 🤸🏿‍♀️ woman cartwheeling: dark skin tone
+            ('\U0001f93c', None),                 # 🤼 people wrestling
+            ('\U0001f93c\u200d♂️', None),         # 🤼‍♂️ men wrestling
+            ('\U0001f93c\u200d♀️', None),         # 🤼‍♀️ women wrestling
+            ('\U0001f93d',                        # 🤽 person playing water polo
+                ('\U0001f93d🏻',                   # 🤽🏻 person playing water polo: light skin tone
+                 '\U0001f93d🏼',                   # 🤽🏼 person playing water polo: medium-light skin tone
+                 '\U0001f93d🏽',                   # 🤽🏽 person playing water polo: medium skin tone
+                 '\U0001f93d🏾',                   # 🤽🏾 person playing water polo: medium-dark skin tone
+                 '\U0001f93d🏿')),                 # 🤽🏿 person playing water polo: dark skin tone
+            ('\U0001f93d\u200d♂️',                # 🤽‍♂️ man playing water polo
+                ('\U0001f93d🏻\u200d♂️',           # 🤽🏻‍♂️ man playing water polo: light skin tone
+                 '\U0001f93d🏼\u200d♂️',           # 🤽🏼‍♂️ man playing water polo: medium-light skin tone
+                 '\U0001f93d🏽\u200d♂️',           # 🤽🏽‍♂️ man playing water polo: medium skin tone
+                 '\U0001f93d🏾\u200d♂️',           # 🤽🏾‍♂️ man playing water polo: medium-dark skin tone
+                 '\U0001f93d🏿\u200d♂️')),         # 🤽🏿‍♂️ man playing water polo: dark skin tone
+            ('\U0001f93d\u200d♀️',                # 🤽‍♀️ woman playing water polo
+                ('\U0001f93d🏻\u200d♀️',           # 🤽🏻‍♀️ woman playing water polo: light skin tone
+                 '\U0001f93d🏼\u200d♀️',           # 🤽🏼‍♀️ woman playing water polo: medium-light skin tone
+                 '\U0001f93d🏽\u200d♀️',           # 🤽🏽‍♀️ woman playing water polo: medium skin tone
+                 '\U0001f93d🏾\u200d♀️',           # 🤽🏾‍♀️ woman playing water polo: medium-dark skin tone
+                 '\U0001f93d🏿\u200d♀️')),         # 🤽🏿‍♀️ woman playing water polo: dark skin tone
+            ('\U0001f93e',                        # 🤾 person playing handball
+                ('\U0001f93e🏻',                   # 🤾🏻 person playing handball: light skin tone
+                 '\U0001f93e🏼',                   # 🤾🏼 person playing handball: medium-light skin tone
+                 '\U0001f93e🏽',                   # 🤾🏽 person playing handball: medium skin tone
+                 '\U0001f93e🏾',                   # 🤾🏾 person playing handball: medium-dark skin tone
+                 '\U0001f93e🏿')),                 # 🤾🏿 person playing handball: dark skin tone
+            ('\U0001f93e\u200d♂️',                # 🤾‍♂️ man playing handball
+                ('\U0001f93e🏻\u200d♂️',           # 🤾🏻‍♂️ man playing handball: light skin tone
+                 '\U0001f93e🏼\u200d♂️',           # 🤾🏼‍♂️ man playing handball: medium-light skin tone
+                 '\U0001f93e🏽\u200d♂️',           # 🤾🏽‍♂️ man playing handball: medium skin tone
+                 '\U0001f93e🏾\u200d♂️',           # 🤾🏾‍♂️ man playing handball: medium-dark skin tone
+                 '\U0001f93e🏿\u200d♂️')),         # 🤾🏿‍♂️ man playing handball: dark skin tone
+            ('\U0001f93e\u200d♀️',                # 🤾‍♀️ woman playing handball
+                ('\U0001f93e🏻\u200d♀️',           # 🤾🏻‍♀️ woman playing handball: light skin tone
+                 '\U0001f93e🏼\u200d♀️',           # 🤾🏼‍♀️ woman playing handball: medium-light skin tone
+                 '\U0001f93e🏽\u200d♀️',           # 🤾🏽‍♀️ woman playing handball: medium skin tone
+                 '\U0001f93e🏾\u200d♀️',           # 🤾🏾‍♀️ woman playing handball: medium-dark skin tone
+                 '\U0001f93e🏿\u200d♀️')),         # 🤾🏿‍♀️ woman playing handball: dark skin tone
+            ('\U0001f939',                        # 🤹 person juggling
+                ('\U0001f939🏻',                   # 🤹🏻 person juggling: light skin tone
+                 '\U0001f939🏼',                   # 🤹🏼 person juggling: medium-light skin tone
+                 '\U0001f939🏽',                   # 🤹🏽 person juggling: medium skin tone
+                 '\U0001f939🏾',                   # 🤹🏾 person juggling: medium-dark skin tone
+                 '\U0001f939🏿')),                 # 🤹🏿 person juggling: dark skin tone
+            ('\U0001f939\u200d♂️',                # 🤹‍♂️ man juggling
+                ('\U0001f939🏻\u200d♂️',           # 🤹🏻‍♂️ man juggling: light skin tone
+                 '\U0001f939🏼\u200d♂️',           # 🤹🏼‍♂️ man juggling: medium-light skin tone
+                 '\U0001f939🏽\u200d♂️',           # 🤹🏽‍♂️ man juggling: medium skin tone
+                 '\U0001f939🏾\u200d♂️',           # 🤹🏾‍♂️ man juggling: medium-dark skin tone
+                 '\U0001f939🏿\u200d♂️')),         # 🤹🏿‍♂️ man juggling: dark skin tone
+            ('\U0001f939\u200d♀️',                # 🤹‍♀️ woman juggling
+                ('\U0001f939🏻\u200d♀️',           # 🤹🏻‍♀️ woman juggling: light skin tone
+                 '\U0001f939🏼\u200d♀️',           # 🤹🏼‍♀️ woman juggling: medium-light skin tone
+                 '\U0001f939🏽\u200d♀️',           # 🤹🏽‍♀️ woman juggling: medium skin tone
+                 '\U0001f939🏾\u200d♀️',           # 🤹🏾‍♀️ woman juggling: medium-dark skin tone
+                 '\U0001f939🏿\u200d♀️')),         # 🤹🏿‍♀️ woman juggling: dark skin tone
+            ('👫', None),                          # 👫 man and woman holding hands
+            ('👬', None),                          # 👬 two men holding hands
+            ('👭', None),                          # 👭 two women holding hands
+            ('💏', None),                          # 💏 kiss
+            ('👩\u200d❤️\u200d💋\u200d👨', None),    # 👩‍❤️‍💋‍👨 kiss: woman, man
+            ('👨\u200d❤️\u200d💋\u200d👨', None),    # 👨‍❤️‍💋‍👨 kiss: man, man
+            ('👩\u200d❤️\u200d💋\u200d👩', None),    # 👩‍❤️‍💋‍👩 kiss: woman, woman
+            ('💑', None),                          # 💑 couple with heart
+            ('👩\u200d❤️\u200d👨', None),           # 👩‍❤️‍👨 couple with heart: woman, man
+            ('👨\u200d❤️\u200d👨', None),           # 👨‍❤️‍👨 couple with heart: man, man
+            ('👩\u200d❤️\u200d👩', None),           # 👩‍❤️‍👩 couple with heart: woman, woman
+            ('👪', None),                          # 👪 family
+            ('👨\u200d👩\u200d👦', None),            # 👨‍👩‍👦 family: man, woman, boy
+            ('👨\u200d👩\u200d👧', None),            # 👨‍👩‍👧 family: man, woman, girl
+            ('👨\u200d👩\u200d👧\u200d👦', None),     # 👨‍👩‍👧‍👦 family: man, woman, girl, boy
+            ('👨\u200d👩\u200d👦\u200d👦', None),     # 👨‍👩‍👦‍👦 family: man, woman, boy, boy
+            ('👨\u200d👩\u200d👧\u200d👧', None),     # 👨‍👩‍👧‍👧 family: man, woman, girl, girl
+            ('👨\u200d👨\u200d👦', None),            # 👨‍👨‍👦 family: man, man, boy
+            ('👨\u200d👨\u200d👧', None),            # 👨‍👨‍👧 family: man, man, girl
+            ('👨\u200d👨\u200d👧\u200d👦', None),     # 👨‍👨‍👧‍👦 family: man, man, girl, boy
+            ('👨\u200d👨\u200d👦\u200d👦', None),     # 👨‍👨‍👦‍👦 family: man, man, boy, boy
+            ('👨\u200d👨\u200d👧\u200d👧', None),     # 👨‍👨‍👧‍👧 family: man, man, girl, girl
+            ('👩\u200d👩\u200d👦', None),            # 👩‍👩‍👦 family: woman, woman, boy
+            ('👩\u200d👩\u200d👧', None),            # 👩‍👩‍👧 family: woman, woman, girl
+            ('👩\u200d👩\u200d👧\u200d👦', None),     # 👩‍👩‍👧‍👦 family: woman, woman, girl, boy
+            ('👩\u200d👩\u200d👦\u200d👦', None),     # 👩‍👩‍👦‍👦 family: woman, woman, boy, boy
+            ('👩\u200d👩\u200d👧\u200d👧', None),     # 👩‍👩‍👧‍👧 family: woman, woman, girl, girl
+            ('👨\u200d👦', None),                   # 👨‍👦 family: man, boy
+            ('👨\u200d👦\u200d👦', None),            # 👨‍👦‍👦 family: man, boy, boy
+            ('👨\u200d👧', None),                   # 👨‍👧 family: man, girl
+            ('👨\u200d👧\u200d👦', None),            # 👨‍👧‍👦 family: man, girl, boy
+            ('👨\u200d👧\u200d👧', None),            # 👨‍👧‍👧 family: man, girl, girl
+            ('👩\u200d👦', None),                   # 👩‍👦 family: woman, boy
+            ('👩\u200d👦\u200d👦', None),            # 👩‍👦‍👦 family: woman, boy, boy
+            ('👩\u200d👧', None),                   # 👩‍👧 family: woman, girl
+            ('👩\u200d👧\u200d👦', None),            # 👩‍👧‍👦 family: woman, girl, boy
+            ('👩\u200d👧\u200d👧', None),            # 👩‍👧‍👧 family: woman, girl, girl
+            ('💪',                                 # 💪 flexed biceps
+                ('💪🏻',                            # 💪🏻 flexed biceps: light skin tone
+                 '💪🏼',                            # 💪🏼 flexed biceps: medium-light skin tone
+                 '💪🏽',                            # 💪🏽 flexed biceps: medium skin tone
+                 '💪🏾',                            # 💪🏾 flexed biceps: medium-dark skin tone
+                 '💪🏿')),                          # 💪🏿 flexed biceps: dark skin tone
+            ('\U0001f933',                        # 🤳 selfie
+                ('\U0001f933🏻',                   # 🤳🏻 selfie: light skin tone
+                 '\U0001f933🏼',                   # 🤳🏼 selfie: medium-light skin tone
+                 '\U0001f933🏽',                   # 🤳🏽 selfie: medium skin tone
+                 '\U0001f933🏾',                   # 🤳🏾 selfie: medium-dark skin tone
+                 '\U0001f933🏿')),                 # 🤳🏿 selfie: dark skin tone
+            ('👈',                                 # 👈 backhand index pointing left
+                ('👈🏻',                            # 👈🏻 backhand index pointing left: light skin tone
+                 '👈🏼',                            # 👈🏼 backhand index pointing left: medium-light skin tone
+                 '👈🏽',                            # 👈🏽 backhand index pointing left: medium skin tone
+                 '👈🏾',                            # 👈🏾 backhand index pointing left: medium-dark skin tone
+                 '👈🏿')),                          # 👈🏿 backhand index pointing left: dark skin tone
+            ('👉',                                 # 👉 backhand index pointing right
+                ('👉🏻',                            # 👉🏻 backhand index pointing right: light skin tone
+                 '👉🏼',                            # 👉🏼 backhand index pointing right: medium-light skin tone
+                 '👉🏽',                            # 👉🏽 backhand index pointing right: medium skin tone
+                 '👉🏾',                            # 👉🏾 backhand index pointing right: medium-dark skin tone
+                 '👉🏿')),                          # 👉🏿 backhand index pointing right: dark skin tone
+            ('☝',                                 # ☝ index pointing up
+                ('☝🏻',                            # ☝🏻 index pointing up: light skin tone
+                 '☝🏼',                            # ☝🏼 index pointing up: medium-light skin tone
+                 '☝🏽',                            # ☝🏽 index pointing up: medium skin tone
+                 '☝🏾',                            # ☝🏾 index pointing up: medium-dark skin tone
+                 '☝🏿')),                          # ☝🏿 index pointing up: dark skin tone
+            ('👆',                                 # 👆 backhand index pointing up
+                ('👆🏻',                            # 👆🏻 backhand index pointing up: light skin tone
+                 '👆🏼',                            # 👆🏼 backhand index pointing up: medium-light skin tone
+                 '👆🏽',                            # 👆🏽 backhand index pointing up: medium skin tone
+                 '👆🏾',                            # 👆🏾 backhand index pointing up: medium-dark skin tone
+                 '👆🏿')),                          # 👆🏿 backhand index pointing up: dark skin tone
+            ('🖕',                                 # 🖕 middle finger
+                ('🖕🏻',                            # 🖕🏻 middle finger: light skin tone
+                 '🖕🏼',                            # 🖕🏼 middle finger: medium-light skin tone
+                 '🖕🏽',                            # 🖕🏽 middle finger: medium skin tone
+                 '🖕🏾',                            # 🖕🏾 middle finger: medium-dark skin tone
+                 '🖕🏿')),                          # 🖕🏿 middle finger: dark skin tone
+            ('👇',                                 # 👇 backhand index pointing down
+                ('👇🏻',                            # 👇🏻 backhand index pointing down: light skin tone
+                 '👇🏼',                            # 👇🏼 backhand index pointing down: medium-light skin tone
+                 '👇🏽',                            # 👇🏽 backhand index pointing down: medium skin tone
+                 '👇🏾',                            # 👇🏾 backhand index pointing down: medium-dark skin tone
+                 '👇🏿')),                          # 👇🏿 backhand index pointing down: dark skin tone
+            ('✌',                                 # ✌ victory hand
+                ('✌🏻',                            # ✌🏻 victory hand: light skin tone
+                 '✌🏼',                            # ✌🏼 victory hand: medium-light skin tone
+                 '✌🏽',                            # ✌🏽 victory hand: medium skin tone
+                 '✌🏾',                            # ✌🏾 victory hand: medium-dark skin tone
+                 '✌🏿')),                          # ✌🏿 victory hand: dark skin tone
+            ('\U0001f91e',                        # 🤞 crossed fingers
+                ('\U0001f91e🏻',                   # 🤞🏻 crossed fingers: light skin tone
+                 '\U0001f91e🏼',                   # 🤞🏼 crossed fingers: medium-light skin tone
+                 '\U0001f91e🏽',                   # 🤞🏽 crossed fingers: medium skin tone
+                 '\U0001f91e🏾',                   # 🤞🏾 crossed fingers: medium-dark skin tone
+                 '\U0001f91e🏿')),                 # 🤞🏿 crossed fingers: dark skin tone
+            ('🖖',                                 # 🖖 vulcan salute
+                ('🖖🏻',                            # 🖖🏻 vulcan salute: light skin tone
+                 '🖖🏼',                            # 🖖🏼 vulcan salute: medium-light skin tone
+                 '🖖🏽',                            # 🖖🏽 vulcan salute: medium skin tone
+                 '🖖🏾',                            # 🖖🏾 vulcan salute: medium-dark skin tone
+                 '🖖🏿')),                          # 🖖🏿 vulcan salute: dark skin tone
+            ('🤘',                                 # 🤘 sign of the horns
+                ('🤘🏻',                            # 🤘🏻 sign of the horns: light skin tone
+                 '🤘🏼',                            # 🤘🏼 sign of the horns: medium-light skin tone
+                 '🤘🏽',                            # 🤘🏽 sign of the horns: medium skin tone
+                 '🤘🏾',                            # 🤘🏾 sign of the horns: medium-dark skin tone
+                 '🤘🏿')),                          # 🤘🏿 sign of the horns: dark skin tone
+            ('\U0001f919',                        # 🤙 call me hand
+                ('\U0001f919🏻',                   # 🤙🏻 call me hand: light skin tone
+                 '\U0001f919🏼',                   # 🤙🏼 call me hand: medium-light skin tone
+                 '\U0001f919🏽',                   # 🤙🏽 call me hand: medium skin tone
+                 '\U0001f919🏾',                   # 🤙🏾 call me hand: medium-dark skin tone
+                 '\U0001f919🏿')),                 # 🤙🏿 call me hand: dark skin tone
+            ('🖐',                                 # 🖐 raised hand with fingers splayed
+                ('🖐🏻',                            # 🖐🏻 raised hand with fingers splayed: light skin tone
+                 '🖐🏼',                            # 🖐🏼 raised hand with fingers splayed: medium-light skin tone
+                 '🖐🏽',                            # 🖐🏽 raised hand with fingers splayed: medium skin tone
+                 '🖐🏾',                            # 🖐🏾 raised hand with fingers splayed: medium-dark skin tone
+                 '🖐🏿')),                          # 🖐🏿 raised hand with fingers splayed: dark skin tone
+            ('✋',                                 # ✋ raised hand
+                ('✋🏻',                            # ✋🏻 raised hand: light skin tone
+                 '✋🏼',                            # ✋🏼 raised hand: medium-light skin tone
+                 '✋🏽',                            # ✋🏽 raised hand: medium skin tone
+                 '✋🏾',                            # ✋🏾 raised hand: medium-dark skin tone
+                 '✋🏿')),                          # ✋🏿 raised hand: dark skin tone
+            ('👌',                                 # 👌 OK hand
+                ('👌🏻',                            # 👌🏻 OK hand: light skin tone
+                 '👌🏼',                            # 👌🏼 OK hand: medium-light skin tone
+                 '👌🏽',                            # 👌🏽 OK hand: medium skin tone
+                 '👌🏾',                            # 👌🏾 OK hand: medium-dark skin tone
+                 '👌🏿')),                          # 👌🏿 OK hand: dark skin tone
+            ('👍',                                 # 👍 thumbs up
+                ('👍🏻',                            # 👍🏻 thumbs up: light skin tone
+                 '👍🏼',                            # 👍🏼 thumbs up: medium-light skin tone
+                 '👍🏽',                            # 👍🏽 thumbs up: medium skin tone
+                 '👍🏾',                            # 👍🏾 thumbs up: medium-dark skin tone
+                 '👍🏿')),                          # 👍🏿 thumbs up: dark skin tone
+            ('👎',                                 # 👎 thumbs down
+                ('👎🏻',                            # 👎🏻 thumbs down: light skin tone
+                 '👎🏼',                            # 👎🏼 thumbs down: medium-light skin tone
+                 '👎🏽',                            # 👎🏽 thumbs down: medium skin tone
+                 '👎🏾',                            # 👎🏾 thumbs down: medium-dark skin tone
+                 '👎🏿')),                          # 👎🏿 thumbs down: dark skin tone
+            ('✊',                                 # ✊ raised fist
+                ('✊🏻',                            # ✊🏻 raised fist: light skin tone
+                 '✊🏼',                            # ✊🏼 raised fist: medium-light skin tone
+                 '✊🏽',                            # ✊🏽 raised fist: medium skin tone
+                 '✊🏾',                            # ✊🏾 raised fist: medium-dark skin tone
+                 '✊🏿')),                          # ✊🏿 raised fist: dark skin tone
+            ('👊',                                 # 👊 oncoming fist
+                ('👊🏻',                            # 👊🏻 oncoming fist: light skin tone
+                 '👊🏼',                            # 👊🏼 oncoming fist: medium-light skin tone
+                 '👊🏽',                            # 👊🏽 oncoming fist: medium skin tone
+                 '👊🏾',                            # 👊🏾 oncoming fist: medium-dark skin tone
+                 '👊🏿')),                          # 👊🏿 oncoming fist: dark skin tone
+            ('\U0001f91b',                        # 🤛 left-facing fist
+                ('\U0001f91b🏻',                   # 🤛🏻 left-facing fist: light skin tone
+                 '\U0001f91b🏼',                   # 🤛🏼 left-facing fist: medium-light skin tone
+                 '\U0001f91b🏽',                   # 🤛🏽 left-facing fist: medium skin tone
+                 '\U0001f91b🏾',                   # 🤛🏾 left-facing fist: medium-dark skin tone
+                 '\U0001f91b🏿')),                 # 🤛🏿 left-facing fist: dark skin tone
+            ('\U0001f91c',                        # 🤜 right-facing fist
+                ('\U0001f91c🏻',                   # 🤜🏻 right-facing fist: light skin tone
+                 '\U0001f91c🏼',                   # 🤜🏼 right-facing fist: medium-light skin tone
+                 '\U0001f91c🏽',                   # 🤜🏽 right-facing fist: medium skin tone
+                 '\U0001f91c🏾',                   # 🤜🏾 right-facing fist: medium-dark skin tone
+                 '\U0001f91c🏿')),                 # 🤜🏿 right-facing fist: dark skin tone
+            ('\U0001f91a',                        # 🤚 raised back of hand
+                ('\U0001f91a🏻',                   # 🤚🏻 raised back of hand: light skin tone
+                 '\U0001f91a🏼',                   # 🤚🏼 raised back of hand: medium-light skin tone
+                 '\U0001f91a🏽',                   # 🤚🏽 raised back of hand: medium skin tone
+                 '\U0001f91a🏾',                   # 🤚🏾 raised back of hand: medium-dark skin tone
+                 '\U0001f91a🏿')),                 # 🤚🏿 raised back of hand: dark skin tone
+            ('👋',                                 # 👋 waving hand
+                ('👋🏻',                            # 👋🏻 waving hand: light skin tone
+                 '👋🏼',                            # 👋🏼 waving hand: medium-light skin tone
+                 '👋🏽',                            # 👋🏽 waving hand: medium skin tone
+                 '👋🏾',                            # 👋🏾 waving hand: medium-dark skin tone
+                 '👋🏿')),                          # 👋🏿 waving hand: dark skin tone
+            ('👏',                                 # 👏 clapping hands
+                ('👏🏻',                            # 👏🏻 clapping hands: light skin tone
+                 '👏🏼',                            # 👏🏼 clapping hands: medium-light skin tone
+                 '👏🏽',                            # 👏🏽 clapping hands: medium skin tone
+                 '👏🏾',                            # 👏🏾 clapping hands: medium-dark skin tone
+                 '👏🏿')),                          # 👏🏿 clapping hands: dark skin tone
+            ('✍',                                 # ✍ writing hand
+                ('✍🏻',                            # ✍🏻 writing hand: light skin tone
+                 '✍🏼',                            # ✍🏼 writing hand: medium-light skin tone
+                 '✍🏽',                            # ✍🏽 writing hand: medium skin tone
+                 '✍🏾',                            # ✍🏾 writing hand: medium-dark skin tone
+                 '✍🏿')),                          # ✍🏿 writing hand: dark skin tone
+            ('👐',                                 # 👐 open hands
+                ('👐🏻',                            # 👐🏻 open hands: light skin tone
+                 '👐🏼',                            # 👐🏼 open hands: medium-light skin tone
+                 '👐🏽',                            # 👐🏽 open hands: medium skin tone
+                 '👐🏾',                            # 👐🏾 open hands: medium-dark skin tone
+                 '👐🏿')),                          # 👐🏿 open hands: dark skin tone
+            ('🙌',                                 # 🙌 raising hands
+                ('🙌🏻',                            # 🙌🏻 raising hands: light skin tone
+                 '🙌🏼',                            # 🙌🏼 raising hands: medium-light skin tone
+                 '🙌🏽',                            # 🙌🏽 raising hands: medium skin tone
+                 '🙌🏾',                            # 🙌🏾 raising hands: medium-dark skin tone
+                 '🙌🏿')),                          # 🙌🏿 raising hands: dark skin tone
+            ('🙏',                                 # 🙏 folded hands
+                ('🙏🏻',                            # 🙏🏻 folded hands: light skin tone
+                 '🙏🏼',                            # 🙏🏼 folded hands: medium-light skin tone
+                 '🙏🏽',                            # 🙏🏽 folded hands: medium skin tone
+                 '🙏🏾',                            # 🙏🏾 folded hands: medium-dark skin tone
+                 '🙏🏿')),                          # 🙏🏿 folded hands: dark skin tone
+            ('\U0001f91d', None),                 # 🤝 handshake
+            ('💅',                                 # 💅 nail polish
+                ('💅🏻',                            # 💅🏻 nail polish: light skin tone
+                 '💅🏼',                            # 💅🏼 nail polish: medium-light skin tone
+                 '💅🏽',                            # 💅🏽 nail polish: medium skin tone
+                 '💅🏾',                            # 💅🏾 nail polish: medium-dark skin tone
+                 '💅🏿')),                          # 💅🏿 nail polish: dark skin tone
+            ('👂',                                 # 👂 ear
+                ('👂🏻',                            # 👂🏻 ear: light skin tone
+                 '👂🏼',                            # 👂🏼 ear: medium-light skin tone
+                 '👂🏽',                            # 👂🏽 ear: medium skin tone
+                 '👂🏾',                            # 👂🏾 ear: medium-dark skin tone
+                 '👂🏿')),                          # 👂🏿 ear: dark skin tone
+            ('👃',                                 # 👃 nose
+                ('👃🏻',                            # 👃🏻 nose: light skin tone
+                 '👃🏼',                            # 👃🏼 nose: medium-light skin tone
+                 '👃🏽',                            # 👃🏽 nose: medium skin tone
+                 '👃🏾',                            # 👃🏾 nose: medium-dark skin tone
+                 '👃🏿')),                          # 👃🏿 nose: dark skin tone
+            ('👣', None),                          # 👣 footprints
+            ('👀', None),                          # 👀 eyes
+            ('👁', None),                          # 👁 eye
+            ('👁️\u200d🗨️', None),                 # 👁️‍🗨️ eye in speech bubble
+            ('👅', None),                          # 👅 tongue
+            ('👄', None),                          # 👄 mouth
+        ]],
+    ['❤',                                         # category: Emotion
+        [
+            ('💋', None),                          # 💋 kiss mark
+            ('💘', None),                          # 💘 heart with arrow
+            ('❤', None),                          # ❤ red heart
+            ('💓', None),                          # 💓 beating heart
+            ('💔', None),                          # 💔 broken heart
+            ('💕', None),                          # 💕 two hearts
+            ('💖', None),                          # 💖 sparkling heart
+            ('💗', None),                          # 💗 growing heart
+            ('💙', None),                          # 💙 blue heart
+            ('💚', None),                          # 💚 green heart
+            ('💛', None),                          # 💛 yellow heart
+            ('💜', None),                          # 💜 purple heart
+            ('\U0001f5a4', None),                 # 🖤 black heart
+            ('💝', None),                          # 💝 heart with ribbon
+            ('💞', None),                          # 💞 revolving hearts
+            ('💟', None),                          # 💟 heart decoration
+            ('❣', None),                          # ❣ heavy heart exclamation
+            ('💌', None),                          # 💌 love letter
+            ('💤', None),                          # 💤 zzz
+            ('💢', None),                          # 💢 anger symbol
+            ('💣', None),                          # 💣 bomb
+            ('💥', None),                          # 💥 collision
+            ('💦', None),                          # 💦 sweat droplets
+            ('💨', None),                          # 💨 dashing away
+            ('💫', None),                          # 💫 dizzy
+            ('💬', None),                          # 💬 speech balloon
+            ('🗨', None),                          # 🗨 left speech bubble
+            ('🗯', None),                          # 🗯 right anger bubble
+            ('💭', None),                          # 💭 thought balloon
+            ('🕳', None),                          # 🕳 hole
+            ('👓', None),                          # 👓 glasses
+            ('🕶', None),                          # 🕶 sunglasses
+            ('👔', None),                          # 👔 necktie
+            ('👕', None),                          # 👕 t-shirt
+            ('👖', None),                          # 👖 jeans
+            ('👗', None),                          # 👗 dress
+            ('👘', None),                          # 👘 kimono
+            ('👙', None),                          # 👙 bikini
+            ('👚', None),                          # 👚 woman’s clothes
+            ('👛', None),                          # 👛 purse
+            ('👜', None),                          # 👜 handbag
+            ('👝', None),                          # 👝 clutch bag
+            ('🛍', None),                          # 🛍 shopping bags
+            ('🎒', None),                          # 🎒 school backpack
+            ('👞', None),                          # 👞 man’s shoe
+            ('👟', None),                          # 👟 running shoe
+            ('👠', None),                          # 👠 high-heeled shoe
+            ('👡', None),                          # 👡 woman’s sandal
+            ('👢', None),                          # 👢 woman’s boot
+            ('👑', None),                          # 👑 crown
+            ('👒', None),                          # 👒 woman’s hat
+            ('🎩', None),                          # 🎩 top hat
+            ('🎓', None),                          # 🎓 graduation cap
+            ('⛑', None),                          # ⛑ rescue worker’s helmet
+            ('📿', None),                          # 📿 prayer beads
+            ('💄', None),                          # 💄 lipstick
+            ('💍', None),                          # 💍 ring
+            ('💎', None),                          # 💎 gem stone
+        ]],
+    ['🐱',                                         # category: Animals & Nature
+        [
+            ('🐵', None),                          # 🐵 monkey face
+            ('🐒', None),                          # 🐒 monkey
+            ('\U0001f98d', None),                 # 🦍 gorilla
+            ('🐶', None),                          # 🐶 dog face
+            ('🐕', None),                          # 🐕 dog
+            ('🐩', None),                          # 🐩 poodle
+            ('🐺', None),                          # 🐺 wolf face
+            ('\U0001f98a', None),                 # 🦊 fox face
+            ('🐱', None),                          # 🐱 cat face
+            ('🐈', None),                          # 🐈 cat
+            ('🦁', None),                          # 🦁 lion face
+            ('🐯', None),                          # 🐯 tiger face
+            ('🐅', None),                          # 🐅 tiger
+            ('🐆', None),                          # 🐆 leopard
+            ('🐴', None),                          # 🐴 horse face
+            ('🐎', None),                          # 🐎 horse
+            ('\U0001f98c', None),                 # 🦌 deer
+            ('🦄', None),                          # 🦄 unicorn face
+            ('🐮', None),                          # 🐮 cow face
+            ('🐂', None),                          # 🐂 ox
+            ('🐃', None),                          # 🐃 water buffalo
+            ('🐄', None),                          # 🐄 cow
+            ('🐷', None),                          # 🐷 pig face
+            ('🐖', None),                          # 🐖 pig
+            ('🐗', None),                          # 🐗 boar
+            ('🐽', None),                          # 🐽 pig nose
+            ('🐏', None),                          # 🐏 ram
+            ('🐑', None),                          # 🐑 sheep
+            ('🐐', None),                          # 🐐 goat
+            ('🐪', None),                          # 🐪 camel
+            ('🐫', None),                          # 🐫 two-hump camel
+            ('🐘', None),                          # 🐘 elephant
+            ('\U0001f98f', None),                 # 🦏 rhinoceros
+            ('🐭', None),                          # 🐭 mouse face
+            ('🐁', None),                          # 🐁 mouse
+            ('🐀', None),                          # 🐀 rat
+            ('🐹', None),                          # 🐹 hamster face
+            ('🐰', None),                          # 🐰 rabbit face
+            ('🐇', None),                          # 🐇 rabbit
+            ('🐿', None),                          # 🐿 chipmunk
+            ('\U0001f987', None),                 # 🦇 bat
+            ('🐻', None),                          # 🐻 bear face
+            ('🐨', None),                          # 🐨 koala
+            ('🐼', None),                          # 🐼 panda face
+            ('🐾', None),                          # 🐾 paw prints
+            ('🦃', None),                          # 🦃 turkey
+            ('🐔', None),                          # 🐔 chicken
+            ('🐓', None),                          # 🐓 rooster
+            ('🐣', None),                          # 🐣 hatching chick
+            ('🐤', None),                          # 🐤 baby chick
+            ('🐥', None),                          # 🐥 front-facing baby chick
+            ('🐦', None),                          # 🐦 bird
+            ('🐧', None),                          # 🐧 penguin
+            ('🕊', None),                          # 🕊 dove
+            ('\U0001f985', None),                 # 🦅 eagle
+            ('\U0001f986', None),                 # 🦆 duck
+            ('\U0001f989', None),                 # 🦉 owl
+            ('🐸', None),                          # 🐸 frog face
+            ('🐊', None),                          # 🐊 crocodile
+            ('🐢', None),                          # 🐢 turtle
+            ('\U0001f98e', None),                 # 🦎 lizard
+            ('🐍', None),                          # 🐍 snake
+            ('🐲', None),                          # 🐲 dragon face
+            ('🐉', None),                          # 🐉 dragon
+            ('🐳', None),                          # 🐳 spouting whale
+            ('🐋', None),                          # 🐋 whale
+            ('🐬', None),                          # 🐬 dolphin
+            ('🐟', None),                          # 🐟 fish
+            ('🐠', None),                          # 🐠 tropical fish
+            ('🐡', None),                          # 🐡 blowfish
+            ('\U0001f988', None),                 # 🦈 shark
+            ('🐙', None),                          # 🐙 octopus
+            ('🐚', None),                          # 🐚 spiral shell
+            ('🦀', None),                          # 🦀 crab
+            ('\U0001f990', None),                 # 🦐 shrimp
+            ('\U0001f991', None),                 # 🦑 squid
+            ('\U0001f98b', None),                 # 🦋 butterfly
+            ('🐌', None),                          # 🐌 snail
+            ('🐛', None),                          # 🐛 bug
+            ('🐜', None),                          # 🐜 ant
+            ('🐝', None),                          # 🐝 honeybee
+            ('🐞', None),                          # 🐞 lady beetle
+            ('🕷', None),                          # 🕷 spider
+            ('🕸', None),                          # 🕸 spider web
+            ('🦂', None),                          # 🦂 scorpion
+            ('💐', None),                          # 💐 bouquet
+            ('🌸', None),                          # 🌸 cherry blossom
+            ('💮', None),                          # 💮 white flower
+            ('🏵', None),                          # 🏵 rosette
+            ('🌹', None),                          # 🌹 rose
+            ('\U0001f940', None),                 # 🥀 wilted flower
+            ('🌺', None),                          # 🌺 hibiscus
+            ('🌻', None),                          # 🌻 sunflower
+            ('🌼', None),                          # 🌼 blossom
+            ('🌷', None),                          # 🌷 tulip
+            ('🌱', None),                          # 🌱 seedling
+            ('🌲', None),                          # 🌲 evergreen tree
+            ('🌳', None),                          # 🌳 deciduous tree
+            ('🌴', None),                          # 🌴 palm tree
+            ('🌵', None),                          # 🌵 cactus
+            ('🌾', None),                          # 🌾 sheaf of rice
+            ('🌿', None),                          # 🌿 herb
+            ('☘', None),                          # ☘ shamrock
+            ('🍀', None),                          # 🍀 four leaf clover
+            ('🍁', None),                          # 🍁 maple leaf
+            ('🍂', None),                          # 🍂 fallen leaf
+            ('🍃', None),                          # 🍃 leaf fluttering in wind
+        ]],
+    ['🍒',                                         # category: Food & Drink
+        [
+            ('🍇', None),                          # 🍇 grapes
+            ('🍈', None),                          # 🍈 melon
+            ('🍉', None),                          # 🍉 watermelon
+            ('🍊', None),                          # 🍊 tangerine
+            ('🍋', None),                          # 🍋 lemon
+            ('🍌', None),                          # 🍌 banana
+            ('🍍', None),                          # 🍍 pineapple
+            ('🍎', None),                          # 🍎 red apple
+            ('🍏', None),                          # 🍏 green apple
+            ('🍐', None),                          # 🍐 pear
+            ('🍑', None),                          # 🍑 peach
+            ('🍒', None),                          # 🍒 cherries
+            ('🍓', None),                          # 🍓 strawberry
+            ('\U0001f95d', None),                 # 🥝 kiwi fruit
+            ('🍅', None),                          # 🍅 tomato
+            ('\U0001f951', None),                 # 🥑 avocado
+            ('🍆', None),                          # 🍆 eggplant
+            ('\U0001f954', None),                 # 🥔 potato
+            ('\U0001f955', None),                 # 🥕 carrot
+            ('🌽', None),                          # 🌽 ear of corn
+            ('🌶', None),                          # 🌶 hot pepper
+            ('\U0001f952', None),                 # 🥒 cucumber
+            ('🍄', None),                          # 🍄 mushroom
+            ('\U0001f95c', None),                 # 🥜 peanuts
+            ('🌰', None),                          # 🌰 chestnut
+            ('🍞', None),                          # 🍞 bread
+            ('\U0001f950', None),                 # 🥐 croissant
+            ('\U0001f956', None),                 # 🥖 baguette bread
+            ('\U0001f95e', None),                 # 🥞 pancakes
+            ('🧀', None),                          # 🧀 cheese wedge
+            ('🍖', None),                          # 🍖 meat on bone
+            ('🍗', None),                          # 🍗 poultry leg
+            ('\U0001f953', None),                 # 🥓 bacon
+            ('🍔', None),                          # 🍔 hamburger
+            ('🍟', None),                          # 🍟 french fries
+            ('🍕', None),                          # 🍕 pizza
+            ('🌭', None),                          # 🌭 hot dog
+            ('🌮', None),                          # 🌮 taco
+            ('🌯', None),                          # 🌯 burrito
+            ('\U0001f959', None),                 # 🥙 stuffed flatbread
+            ('\U0001f95a', None),                 # 🥚 egg
+            ('🍳', None),                          # 🍳 cooking
+            ('\U0001f958', None),                 # 🥘 shallow pan of food
+            ('🍲', None),                          # 🍲 pot of food
+            ('\U0001f957', None),                 # 🥗 green salad
+            ('🍿', None),                          # 🍿 popcorn
+            ('🍱', None),                          # 🍱 bento box
+            ('🍘', None),                          # 🍘 rice cracker
+            ('🍙', None),                          # 🍙 rice ball
+            ('🍚', None),                          # 🍚 cooked rice
+            ('🍛', None),                          # 🍛 curry rice
+            ('🍜', None),                          # 🍜 steaming bowl
+            ('🍝', None),                          # 🍝 spaghetti
+            ('🍠', None),                          # 🍠 roasted sweet potato
+            ('🍢', None),                          # 🍢 oden
+            ('🍣', None),                          # 🍣 sushi
+            ('🍤', None),                          # 🍤 fried shrimp
+            ('🍥', None),                          # 🍥 fish cake with swirl
+            ('🍡', None),                          # 🍡 dango
+            ('🍦', None),                          # 🍦 soft ice cream
+            ('🍧', None),                          # 🍧 shaved ice
+            ('🍨', None),                          # 🍨 ice cream
+            ('🍩', None),                          # 🍩 doughnut
+            ('🍪', None),                          # 🍪 cookie
+            ('🎂', None),                          # 🎂 birthday cake
+            ('🍰', None),                          # 🍰 shortcake
+            ('🍫', None),                          # 🍫 chocolate bar
+            ('🍬', None),                          # 🍬 candy
+            ('🍭', None),                          # 🍭 lollipop
+            ('🍮', None),                          # 🍮 custard
+            ('🍯', None),                          # 🍯 honey pot
+            ('🍼', None),                          # 🍼 baby bottle
+            ('\U0001f95b', None),                 # 🥛 glass of milk
+            ('☕', None),                          # ☕ hot beverage
+            ('🍵', None),                          # 🍵 teacup without handle
+            ('🍶', None),                          # 🍶 sake
+            ('🍾', None),                          # 🍾 bottle with popping cork
+            ('🍷', None),                          # 🍷 wine glass
+            ('🍸', None),                          # 🍸 cocktail glass
+            ('🍹', None),                          # 🍹 tropical drink
+            ('🍺', None),                          # 🍺 beer mug
+            ('🍻', None),                          # 🍻 clinking beer mugs
+            ('\U0001f942', None),                 # 🥂 clinking glasses
+            ('\U0001f943', None),                 # 🥃 tumbler glass
+            ('🍽', None),                          # 🍽 fork and knife with plate
+            ('🍴', None),                          # 🍴 fork and knife
+            ('\U0001f944', None),                 # 🥄 spoon
+            ('🔪', None),                          # 🔪 kitchen knife
+            ('🏺', None),                          # 🏺 amphora
+        ]],
+    ['🏛',                                         # category: Travel & Places
+        [
+            ('🌍', None),                          # 🌍 globe showing Europe-Africa
+            ('🌎', None),                          # 🌎 globe showing Americas
+            ('🌏', None),                          # 🌏 globe showing Asia-Australia
+            ('🌐', None),                          # 🌐 globe with meridians
+            ('🗺', None),                          # 🗺 world map
+            ('🗾', None),                          # 🗾 map of Japan
+            ('🏔', None),                          # 🏔 snow-capped mountain
+            ('⛰', None),                          # ⛰ mountain
+            ('🌋', None),                          # 🌋 volcano
+            ('🗻', None),                          # 🗻 mount fuji
+            ('🏕', None),                          # 🏕 camping
+            ('🏖', None),                          # 🏖 beach with umbrella
+            ('🏜', None),                          # 🏜 desert
+            ('🏝', None),                          # 🏝 desert island
+            ('🏞', None),                          # 🏞 national park
+            ('🏟', None),                          # 🏟 stadium
+            ('🏛', None),                          # 🏛 classical building
+            ('🏗', None),                          # 🏗 building construction
+            ('🏘', None),                          # 🏘 house
+            ('🏙', None),                          # 🏙 cityscape
+            ('🏚', None),                          # 🏚 derelict house
+            ('🏠', None),                          # 🏠 house
+            ('🏡', None),                          # 🏡 house with garden
+            ('🏢', None),                          # 🏢 office building
+            ('🏣', None),                          # 🏣 Japanese post office
+            ('🏤', None),                          # 🏤 post office
+            ('🏥', None),                          # 🏥 hospital
+            ('🏦', None),                          # 🏦 bank
+            ('🏨', None),                          # 🏨 hotel
+            ('🏩', None),                          # 🏩 love hotel
+            ('🏪', None),                          # 🏪 convenience store
+            ('🏫', None),                          # 🏫 school
+            ('🏬', None),                          # 🏬 department store
+            ('🏭', None),                          # 🏭 factory
+            ('🏯', None),                          # 🏯 Japanese castle
+            ('🏰', None),                          # 🏰 castle
+            ('💒', None),                          # 💒 wedding
+            ('🗼', None),                          # 🗼 Tokyo tower
+            ('🗽', None),                          # 🗽 Statue of Liberty
+            ('⛪', None),                          # ⛪ church
+            ('🕌', None),                          # 🕌 mosque
+            ('🕍', None),                          # 🕍 synagogue
+            ('⛩', None),                          # ⛩ shinto shrine
+            ('🕋', None),                          # 🕋 kaaba
+            ('⛲', None),                          # ⛲ fountain
+            ('⛺', None),                          # ⛺ tent
+            ('🌁', None),                          # 🌁 foggy
+            ('🌃', None),                          # 🌃 night with stars
+            ('🌄', None),                          # 🌄 sunrise over mountains
+            ('🌅', None),                          # 🌅 sunrise
+            ('🌆', None),                          # 🌆 cityscape at dusk
+            ('🌇', None),                          # 🌇 sunset
+            ('🌉', None),                          # 🌉 bridge at night
+            ('♨', None),                          # ♨ hot springs
+            ('🌌', None),                          # 🌌 milky way
+            ('🎠', None),                          # 🎠 carousel horse
+            ('🎡', None),                          # 🎡 ferris wheel
+            ('🎢', None),                          # 🎢 roller coaster
+            ('💈', None),                          # 💈 barber pole
+            ('🎪', None),                          # 🎪 circus tent
+            ('🎭', None),                          # 🎭 performing arts
+            ('🖼', None),                          # 🖼 framed picture
+            ('🎨', None),                          # 🎨 artist palette
+            ('🎰', None),                          # 🎰 slot machine
+            ('🚂', None),                          # 🚂 locomotive
+            ('🚃', None),                          # 🚃 railway car
+            ('🚄', None),                          # 🚄 high-speed train
+            ('🚅', None),                          # 🚅 high-speed train with bullet nose
+            ('🚆', None),                          # 🚆 train
+            ('🚇', None),                          # 🚇 metro
+            ('🚈', None),                          # 🚈 light rail
+            ('🚉', None),                          # 🚉 station
+            ('🚊', None),                          # 🚊 tram
+            ('🚝', None),                          # 🚝 monorail
+            ('🚞', None),                          # 🚞 mountain railway
+            ('🚋', None),                          # 🚋 tram car
+            ('🚌', None),                          # 🚌 bus
+            ('🚍', None),                          # 🚍 oncoming bus
+            ('🚎', None),                          # 🚎 trolleybus
+            ('🚐', None),                          # 🚐 minibus
+            ('🚑', None),                          # 🚑 ambulance
+            ('🚒', None),                          # 🚒 fire engine
+            ('🚓', None),                          # 🚓 police car
+            ('🚔', None),                          # 🚔 oncoming police car
+            ('🚕', None),                          # 🚕 taxi
+            ('🚖', None),                          # 🚖 oncoming taxi
+            ('🚗', None),                          # 🚗 automobile
+            ('🚘', None),                          # 🚘 oncoming automobile
+            ('🚙', None),                          # 🚙 sport utility vehicle
+            ('🚚', None),                          # 🚚 delivery truck
+            ('🚛', None),                          # 🚛 articulated lorry
+            ('🚜', None),                          # 🚜 tractor
+            ('🚲', None),                          # 🚲 bicycle
+            ('\U0001f6f4', None),                 # 🛴 kick scooter
+            ('\U0001f6f5', None),                 # 🛵 motor scooter
+            ('🚏', None),                          # 🚏 bus stop
+            ('🛣', None),                          # 🛣 motorway
+            ('🛤', None),                          # 🛤 railway track
+            ('⛽', None),                          # ⛽ fuel pump
+            ('🚨', None),                          # 🚨 police car light
+            ('🚥', None),                          # 🚥 horizontal traffic light
+            ('🚦', None),                          # 🚦 vertical traffic light
+            ('🚧', None),                          # 🚧 construction
+            ('\U0001f6d1', None),                 # 🛑 stop sign
+            ('⚓', None),                          # ⚓ anchor
+            ('⛵', None),                          # ⛵ sailboat
+            ('\U0001f6f6', None),                 # 🛶 canoe
+            ('🚤', None),                          # 🚤 speedboat
+            ('🛳', None),                          # 🛳 passenger ship
+            ('⛴', None),                          # ⛴ ferry
+            ('🛥', None),                          # 🛥 motor boat
+            ('🚢', None),                          # 🚢 ship
+            ('✈', None),                          # ✈ airplane
+            ('🛩', None),                          # 🛩 small airplane
+            ('🛫', None),                          # 🛫 airplane departure
+            ('🛬', None),                          # 🛬 airplane arrival
+            ('💺', None),                          # 💺 seat
+            ('🚁', None),                          # 🚁 helicopter
+            ('🚟', None),                          # 🚟 suspension railway
+            ('🚠', None),                          # 🚠 mountain cableway
+            ('🚡', None),                          # 🚡 aerial tramway
+            ('🚀', None),                          # 🚀 rocket
+            ('🛰', None),                          # 🛰 satellite
+            ('🛎', None),                          # 🛎 bellhop bell
+            ('🚪', None),                          # 🚪 door
+            ('🛌',                                 # 🛌 person in bed
+                ('🛌🏻',                            # 🛌🏻 person in bed: light skin tone
+                 '🛌🏼',                            # 🛌🏼 person in bed: medium-light skin tone
+                 '🛌🏽',                            # 🛌🏽 person in bed: medium skin tone
+                 '🛌🏾',                            # 🛌🏾 person in bed: medium-dark skin tone
+                 '🛌🏿')),                          # 🛌🏿 person in bed: dark skin tone
+            ('🛏', None),                          # 🛏 bed
+            ('🛋', None),                          # 🛋 couch and lamp
+            ('🚽', None),                          # 🚽 toilet
+            ('🚿', None),                          # 🚿 shower
+            ('🛀',                                 # 🛀 person taking bath
+                ('🛀🏻',                            # 🛀🏻 person taking bath: light skin tone
+                 '🛀🏼',                            # 🛀🏼 person taking bath: medium-light skin tone
+                 '🛀🏽',                            # 🛀🏽 person taking bath: medium skin tone
+                 '🛀🏾',                            # 🛀🏾 person taking bath: medium-dark skin tone
+                 '🛀🏿')),                          # 🛀🏿 person taking bath: dark skin tone
+            ('🛁', None),                          # 🛁 bathtub
+            ('⌛', None),                          # ⌛ hourglass
+            ('⏳', None),                          # ⏳ hourglass with flowing sand
+            ('⌚', None),                          # ⌚ watch
+            ('⏰', None),                          # ⏰ alarm clock
+            ('⏱', None),                          # ⏱ stopwatch
+            ('⏲', None),                          # ⏲ timer clock
+            ('🕰', None),                          # 🕰 mantelpiece clock
+            ('🕛', None),                          # 🕛 twelve o’clock
+            ('🕧', None),                          # 🕧 twelve-thirty
+            ('🕐', None),                          # 🕐 one o’clock
+            ('🕜', None),                          # 🕜 one-thirty
+            ('🕑', None),                          # 🕑 two o’clock
+            ('🕝', None),                          # 🕝 two-thirty
+            ('🕒', None),                          # 🕒 three o’clock
+            ('🕞', None),                          # 🕞 three-thirty
+            ('🕓', None),                          # 🕓 four o’clock
+            ('🕟', None),                          # 🕟 four-thirty
+            ('🕔', None),                          # 🕔 five o’clock
+            ('🕠', None),                          # 🕠 five-thirty
+            ('🕕', None),                          # 🕕 six o’clock
+            ('🕡', None),                          # 🕡 six-thirty
+            ('🕖', None),                          # 🕖 seven o’clock
+            ('🕢', None),                          # 🕢 seven-thirty
+            ('🕗', None),                          # 🕗 eight o’clock
+            ('🕣', None),                          # 🕣 eight-thirty
+            ('🕘', None),                          # 🕘 nine o’clock
+            ('🕤', None),                          # 🕤 nine-thirty
+            ('🕙', None),                          # 🕙 ten o’clock
+            ('🕥', None),                          # 🕥 ten-thirty
+            ('🕚', None),                          # 🕚 eleven o’clock
+            ('🕦', None),                          # 🕦 eleven-thirty
+            ('🌑', None),                          # 🌑 new moon
+            ('🌒', None),                          # 🌒 waxing crescent moon
+            ('🌓', None),                          # 🌓 first quarter moon
+            ('🌔', None),                          # 🌔 waxing gibbous moon
+            ('🌕', None),                          # 🌕 full moon
+            ('🌖', None),                          # 🌖 waning gibbous moon
+            ('🌗', None),                          # 🌗 last quarter moon
+            ('🌘', None),                          # 🌘 waning crescent moon
+            ('🌙', None),                          # 🌙 crescent moon
+            ('🌚', None),                          # 🌚 new moon face
+            ('🌛', None),                          # 🌛 first quarter moon with face
+            ('🌜', None),                          # 🌜 last quarter moon with face
+            ('🌡', None),                          # 🌡 thermometer
+            ('☀', None),                          # ☀ sun
+            ('🌝', None),                          # 🌝 full moon with face
+            ('🌞', None),                          # 🌞 sun with face
+            ('⭐', None),                          # ⭐ white medium star
+            ('🌟', None),                          # 🌟 glowing star
+            ('🌠', None),                          # 🌠 shooting star
+            ('☁', None),                          # ☁ cloud
+            ('⛅', None),                          # ⛅ sun behind cloud
+            ('⛈', None),                          # ⛈ cloud with lightning and rain
+            ('🌤', None),                          # 🌤 sun behind small cloud
+            ('🌥', None),                          # 🌥 sun behind large cloud
+            ('🌦', None),                          # 🌦 sun behind rain cloud
+            ('🌧', None),                          # 🌧 cloud with rain
+            ('🌨', None),                          # 🌨 cloud with snow
+            ('🌩', None),                          # 🌩 cloud with lightning
+            ('🌪', None),                          # 🌪 tornado
+            ('🌫', None),                          # 🌫 fog
+            ('🌬', None),                          # 🌬 wind face
+            ('🌀', None),                          # 🌀 cyclone
+            ('🌈', None),                          # 🌈 rainbow
+            ('🌂', None),                          # 🌂 closed umbrella
+            ('☂', None),                          # ☂ umbrella
+            ('☔', None),                          # ☔ umbrella with rain drops
+            ('⛱', None),                          # ⛱ umbrella on ground
+            ('⚡', None),                          # ⚡ high voltage
+            ('❄', None),                          # ❄ snowflake
+            ('☃', None),                          # ☃ snowman
+            ('⛄', None),                          # ⛄ snowman without snow
+            ('☄', None),                          # ☄ comet
+            ('🔥', None),                          # 🔥 fire
+            ('💧', None),                          # 💧 droplet
+            ('🌊', None),                          # 🌊 water wave
+        ]],
+    ['⚽',                                         # category: Activities
+        [
+            ('🎃', None),                          # 🎃 jack-o-lantern
+            ('🎄', None),                          # 🎄 Christmas tree
+            ('🎆', None),                          # 🎆 fireworks
+            ('🎇', None),                          # 🎇 sparkler
+            ('✨', None),                          # ✨ sparkles
+            ('🎈', None),                          # 🎈 balloon
+            ('🎉', None),                          # 🎉 party popper
+            ('🎊', None),                          # 🎊 confetti ball
+            ('🎋', None),                          # 🎋 tanabata tree
+            ('🎍', None),                          # 🎍 pine decoration
+            ('🎎', None),                          # 🎎 Japanese dolls
+            ('🎏', None),                          # 🎏 carp streamer
+            ('🎐', None),                          # 🎐 wind chime
+            ('🎑', None),                          # 🎑 moon viewing ceremony
+            ('🎀', None),                          # 🎀 ribbon
+            ('🎁', None),                          # 🎁 wrapped gift
+            ('🎗', None),                          # 🎗 reminder ribbon
+            ('🎟', None),                          # 🎟 admission tickets
+            ('🎫', None),                          # 🎫 ticket
+            ('🎖', None),                          # 🎖 military medal
+            ('🏆', None),                          # 🏆 trophy
+            ('🏅', None),                          # 🏅 sports medal
+            ('\U0001f947', None),                 # 🥇 1st place medal
+            ('\U0001f948', None),                 # 🥈 2nd place medal
+            ('\U0001f949', None),                 # 🥉 3rd place medal
+            ('⚽', None),                          # ⚽ soccer ball
+            ('⚾', None),                          # ⚾ baseball
+            ('🏀', None),                          # 🏀 basketball
+            ('🏐', None),                          # 🏐 volleyball
+            ('🏈', None),                          # 🏈 american football
+            ('🏉', None),                          # 🏉 rugby football
+            ('🎾', None),                          # 🎾 tennis
+            ('🎱', None),                          # 🎱 pool 8 ball
+            ('🎳', None),                          # 🎳 bowling
+            ('🏏', None),                          # 🏏 cricket
+            ('🏑', None),                          # 🏑 field hockey
+            ('🏒', None),                          # 🏒 ice hockey
+            ('🏓', None),                          # 🏓 ping pong
+            ('🏸', None),                          # 🏸 badminton
+            ('\U0001f94a', None),                 # 🥊 boxing glove
+            ('\U0001f94b', None),                 # 🥋 martial arts uniform
+            ('\U0001f945', None),                 # 🥅 goal net
+            ('🎯', None),                          # 🎯 direct hit
+            ('⛳', None),                          # ⛳ flag in hole
+            ('⛸', None),                          # ⛸ ice skate
+            ('🎣', None),                          # 🎣 fishing pole
+            ('🎽', None),                          # 🎽 running shirt
+            ('🎿', None),                          # 🎿 skis
+            ('🎮', None),                          # 🎮 video game
+            ('🕹', None),                          # 🕹 joystick
+            ('🎲', None),                          # 🎲 game die
+            ('♠', None),                          # ♠ spade suit
+            ('♥', None),                          # ♥ heart suit
+            ('♦', None),                          # ♦ diamond suit
+            ('♣', None),                          # ♣ club suit
+            ('🃏', None),                          # 🃏 joker
+            ('🀄', None),                          # 🀄 mahjong red dragon
+            ('🎴', None),                          # 🎴 flower playing cards
+        ]],
+    ['🔔',                                         # category: Objects
+        [
+            ('🔇', None),                          # 🔇 muted speaker
+            ('🔈', None),                          # 🔈 speaker low volume
+            ('🔉', None),                          # 🔉 speaker medium volume
+            ('🔊', None),                          # 🔊 speaker high volume
+            ('📢', None),                          # 📢 loudspeaker
+            ('📣', None),                          # 📣 megaphone
+            ('📯', None),                          # 📯 postal horn
+            ('🔔', None),                          # 🔔 bell
+            ('🔕', None),                          # 🔕 bell with slash
+            ('🎼', None),                          # 🎼 musical score
+            ('🎵', None),                          # 🎵 musical note
+            ('🎶', None),                          # 🎶 musical notes
+            ('🎙', None),                          # 🎙 studio microphone
+            ('🎚', None),                          # 🎚 level slider
+            ('🎛', None),                          # 🎛 control knobs
+            ('🎤', None),                          # 🎤 microphone
+            ('🎧', None),                          # 🎧 headphone
+            ('📻', None),                          # 📻 radio
+            ('🎷', None),                          # 🎷 saxophone
+            ('🎸', None),                          # 🎸 guitar
+            ('🎹', None),                          # 🎹 musical keyboard
+            ('🎺', None),                          # 🎺 trumpet
+            ('🎻', None),                          # 🎻 violin
+            ('\U0001f941', None),                 # 🥁 drum
+            ('📱', None),                          # 📱 mobile phone
+            ('📲', None),                          # 📲 mobile phone with arrow
+            ('☎', None),                          # ☎ telephone
+            ('📞', None),                          # 📞 telephone receiver
+            ('📟', None),                          # 📟 pager
+            ('📠', None),                          # 📠 fax machine
+            ('🔋', None),                          # 🔋 battery
+            ('🔌', None),                          # 🔌 electric plug
+            ('💻', None),                          # 💻 laptop computer
+            ('🖥', None),                          # 🖥 desktop computer
+            ('🖨', None),                          # 🖨 printer
+            ('⌨', None),                          # ⌨ keyboard
+            ('🖱', None),                          # 🖱 computer mouse
+            ('🖲', None),                          # 🖲 trackball
+            ('💽', None),                          # 💽 computer disk
+            ('💾', None),                          # 💾 floppy disk
+            ('💿', None),                          # 💿 optical disk
+            ('📀', None),                          # 📀 dvd
+            ('🎥', None),                          # 🎥 movie camera
+            ('🎞', None),                          # 🎞 film frames
+            ('📽', None),                          # 📽 film projector
+            ('🎬', None),                          # 🎬 clapper board
+            ('📺', None),                          # 📺 television
+            ('📷', None),                          # 📷 camera
+            ('📸', None),                          # 📸 camera with flash
+            ('📹', None),                          # 📹 video camera
+            ('📼', None),                          # 📼 videocassette
+            ('🔍', None),                          # 🔍 left-pointing magnifying glass
+            ('🔎', None),                          # 🔎 right-pointing magnifying glass
+            ('🔬', None),                          # 🔬 microscope
+            ('🔭', None),                          # 🔭 telescope
+            ('📡', None),                          # 📡 satellite antenna
+            ('🕯', None),                          # 🕯 candle
+            ('💡', None),                          # 💡 light bulb
+            ('🔦', None),                          # 🔦 flashlight
+            ('🏮', None),                          # 🏮 red paper lantern
+            ('📔', None),                          # 📔 notebook with decorative cover
+            ('📕', None),                          # 📕 closed book
+            ('📖', None),                          # 📖 open book
+            ('📗', None),                          # 📗 green book
+            ('📘', None),                          # 📘 blue book
+            ('📙', None),                          # 📙 orange book
+            ('📚', None),                          # 📚 books
+            ('📓', None),                          # 📓 notebook
+            ('📒', None),                          # 📒 ledger
+            ('📃', None),                          # 📃 page with curl
+            ('📜', None),                          # 📜 scroll
+            ('📄', None),                          # 📄 page facing up
+            ('📰', None),                          # 📰 newspaper
+            ('🗞', None),                          # 🗞 rolled-up newspaper
+            ('📑', None),                          # 📑 bookmark tabs
+            ('🔖', None),                          # 🔖 bookmark
+            ('🏷', None),                          # 🏷 label
+            ('💰', None),                          # 💰 money bag
+            ('💴', None),                          # 💴 yen banknote
+            ('💵', None),                          # 💵 dollar banknote
+            ('💶', None),                          # 💶 euro banknote
+            ('💷', None),                          # 💷 pound banknote
+            ('💸', None),                          # 💸 money with wings
+            ('💳', None),                          # 💳 credit card
+            ('💹', None),                          # 💹 chart increasing with yen
+            ('💱', None),                          # 💱 currency exchange
+            ('💲', None),                          # 💲 heavy dollar sign
+            ('✉', None),                          # ✉ envelope
+            ('📧', None),                          # 📧 e-mail
+            ('📨', None),                          # 📨 incoming envelope
+            ('📩', None),                          # 📩 envelope with arrow
+            ('📤', None),                          # 📤 outbox tray
+            ('📥', None),                          # 📥 inbox tray
+            ('📦', None),                          # 📦 package
+            ('📫', None),                          # 📫 closed mailbox with raised flag
+            ('📪', None),                          # 📪 closed mailbox with lowered flag
+            ('📬', None),                          # 📬 open mailbox with raised flag
+            ('📭', None),                          # 📭 open mailbox with lowered flag
+            ('📮', None),                          # 📮 postbox
+            ('🗳', None),                          # 🗳 ballot box with ballot
+            ('✏', None),                          # ✏ pencil
+            ('✒', None),                          # ✒ black nib
+            ('🖋', None),                          # 🖋 fountain pen
+            ('🖊', None),                          # 🖊 pen
+            ('🖌', None),                          # 🖌 paintbrush
+            ('🖍', None),                          # 🖍 crayon
+            ('📝', None),                          # 📝 memo
+            ('💼', None),                          # 💼 briefcase
+            ('📁', None),                          # 📁 file folder
+            ('📂', None),                          # 📂 open file folder
+            ('🗂', None),                          # 🗂 card index dividers
+            ('📅', None),                          # 📅 calendar
+            ('📆', None),                          # 📆 tear-off calendar
+            ('🗒', None),                          # 🗒 spiral notepad
+            ('🗓', None),                          # 🗓 spiral calendar
+            ('📇', None),                          # 📇 card index
+            ('📈', None),                          # 📈 chart increasing
+            ('📉', None),                          # 📉 chart decreasing
+            ('📊', None),                          # 📊 bar chart
+            ('📋', None),                          # 📋 clipboard
+            ('📌', None),                          # 📌 pushpin
+            ('📍', None),                          # 📍 round pushpin
+            ('📎', None),                          # 📎 paperclip
+            ('🖇', None),                          # 🖇 linked paperclips
+            ('📏', None),                          # 📏 straight ruler
+            ('📐', None),                          # 📐 triangular ruler
+            ('✂', None),                          # ✂ scissors
+            ('🗃', None),                          # 🗃 card file box
+            ('🗄', None),                          # 🗄 file cabinet
+            ('🗑', None),                          # 🗑 wastebasket
+            ('🔒', None),                          # 🔒 locked
+            ('🔓', None),                          # 🔓 unlocked
+            ('🔏', None),                          # 🔏 locked with pen
+            ('🔐', None),                          # 🔐 locked with key
+            ('🔑', None),                          # 🔑 key
+            ('🗝', None),                          # 🗝 old key
+            ('🔨', None),                          # 🔨 hammer
+            ('⛏', None),                          # ⛏ pick
+            ('⚒', None),                          # ⚒ hammer and pick
+            ('🛠', None),                          # 🛠 hammer and wrench
+            ('🗡', None),                          # 🗡 dagger
+            ('⚔', None),                          # ⚔ crossed swords
+            ('🔫', None),                          # 🔫 pistol
+            ('🏹', None),                          # 🏹 bow and arrow
+            ('🛡', None),                          # 🛡 shield
+            ('🔧', None),                          # 🔧 wrench
+            ('🔩', None),                          # 🔩 nut and bolt
+            ('⚙', None),                          # ⚙ gear
+            ('🗜', None),                          # 🗜 clamp
+            ('⚗', None),                          # ⚗ alembic
+            ('⚖', None),                          # ⚖ balance scale
+            ('🔗', None),                          # 🔗 link
+            ('⛓', None),                          # ⛓ chains
+            ('💉', None),                          # 💉 syringe
+            ('💊', None),                          # 💊 pill
+            ('🚬', None),                          # 🚬 cigarette
+            ('⚰', None),                          # ⚰ coffin
+            ('⚱', None),                          # ⚱ funeral urn
+            ('🗿', None),                          # 🗿 moai
+            ('🛢', None),                          # 🛢 oil drum
+            ('🔮', None),                          # 🔮 crystal ball
+            ('\U0001f6d2', None),                 # 🛒 shopping cart
+        ]],
+    ['🔶',                                         # category: Symbols
+        [
+            ('🏧', None),                          # 🏧 ATM sign
+            ('🚮', None),                          # 🚮 litter in bin sign
+            ('🚰', None),                          # 🚰 potable water
+            ('♿', None),                          # ♿ wheelchair symbol
+            ('🚹', None),                          # 🚹 men’s room
+            ('🚺', None),                          # 🚺 women’s room
+            ('🚻', None),                          # 🚻 restroom
+            ('🚼', None),                          # 🚼 baby symbol
+            ('🚾', None),                          # 🚾 water closet
+            ('🛂', None),                          # 🛂 passport control
+            ('🛃', None),                          # 🛃 customs
+            ('🛄', None),                          # 🛄 baggage claim
+            ('🛅', None),                          # 🛅 left luggage
+            ('⚠', None),                          # ⚠ warning
+            ('🚸', None),                          # 🚸 children crossing
+            ('⛔', None),                          # ⛔ no entry
+            ('🚫', None),                          # 🚫 prohibited
+            ('🚳', None),                          # 🚳 no bicycles
+            ('🚭', None),                          # 🚭 no smoking
+            ('🚯', None),                          # 🚯 no littering
+            ('🚱', None),                          # 🚱 non-potable water
+            ('🚷', None),                          # 🚷 no pedestrians
+            ('📵', None),                          # 📵 no mobile phones
+            ('🔞', None),                          # 🔞 no one under eighteen
+            ('☢', None),                          # ☢ radioactive
+            ('☣', None),                          # ☣ biohazard
+            ('⬆', None),                          # ⬆ up arrow
+            ('↗', None),                          # ↗ up-right arrow
+            ('➡', None),                          # ➡ right arrow
+            ('↘', None),                          # ↘ down-right arrow
+            ('⬇', None),                          # ⬇ down arrow
+            ('↙', None),                          # ↙ down-left arrow
+            ('⬅', None),                          # ⬅ left arrow
+            ('↖', None),                          # ↖ up-left arrow
+            ('↕', None),                          # ↕ up-down arrow
+            ('↔', None),                          # ↔ left-right arrow
+            ('↩', None),                          # ↩ right arrow curving left
+            ('↪', None),                          # ↪ left arrow curving right
+            ('⤴', None),                          # ⤴ right arrow curving up
+            ('⤵', None),                          # ⤵ right arrow curving down
+            ('🔃', None),                          # 🔃 clockwise vertical arrows
+            ('🔄', None),                          # 🔄 anticlockwise arrows button
+            ('🔙', None),                          # 🔙 BACK arrow
+            ('🔚', None),                          # 🔚 END arrow
+            ('🔛', None),                          # 🔛 ON! arrow
+            ('🔜', None),                          # 🔜 SOON arrow
+            ('🔝', None),                          # 🔝 TOP arrow
+            ('🛐', None),                          # 🛐 place of worship
+            ('⚛', None),                          # ⚛ atom symbol
+            ('🕉', None),                          # 🕉 om
+            ('✡', None),                          # ✡ star of David
+            ('☸', None),                          # ☸ wheel of dharma
+            ('☯', None),                          # ☯ yin yang
+            ('✝', None),                          # ✝ latin cross
+            ('☦', None),                          # ☦ orthodox cross
+            ('☪', None),                          # ☪ star and crescent
+            ('☮', None),                          # ☮ peace symbol
+            ('🕎', None),                          # 🕎 menorah
+            ('🔯', None),                          # 🔯 dotted six-pointed star
+            ('♈', None),                          # ♈ Aries
+            ('♉', None),                          # ♉ Taurus
+            ('♊', None),                          # ♊ Gemini
+            ('♋', None),                          # ♋ Cancer
+            ('♌', None),                          # ♌ Leo
+            ('♍', None),                          # ♍ Virgo
+            ('♎', None),                          # ♎ Libra
+            ('♏', None),                          # ♏ Scorpius
+            ('♐', None),                          # ♐ Sagittarius
+            ('♑', None),                          # ♑ Capricorn
+            ('♒', None),                          # ♒ Aquarius
+            ('♓', None),                          # ♓ Pisces
+            ('⛎', None),                          # ⛎ Ophiuchus
+            ('🔀', None),                          # 🔀 shuffle tracks button
+            ('🔁', None),                          # 🔁 repeat button
+            ('🔂', None),                          # 🔂 repeat single button
+            ('▶', None),                          # ▶ play button
+            ('⏩', None),                          # ⏩ fast-forward button
+            ('⏭', None),                          # ⏭ next track button
+            ('⏯', None),                          # ⏯ play or pause button
+            ('◀', None),                          # ◀ reverse button
+            ('⏪', None),                          # ⏪ fast reverse button
+            ('⏮', None),                          # ⏮ last track button
+            ('🔼', None),                          # 🔼 up button
+            ('⏫', None),                          # ⏫ fast up button
+            ('🔽', None),                          # 🔽 down button
+            ('⏬', None),                          # ⏬ fast down button
+            ('⏸', None),                          # ⏸ pause button
+            ('⏹', None),                          # ⏹ stop button
+            ('⏺', None),                          # ⏺ record button
+            ('⏏', None),                          # ⏏ eject button
+            ('🎦', None),                          # 🎦 cinema
+            ('🔅', None),                          # 🔅 dim button
+            ('🔆', None),                          # 🔆 bright button
+            ('📶', None),                          # 📶 antenna bars
+            ('📳', None),                          # 📳 vibration mode
+            ('📴', None),                          # 📴 mobile phone off
+            ('♻', None),                          # ♻ recycling symbol
+            ('📛', None),                          # 📛 name badge
+            ('⚜', None),                          # ⚜ fleur-de-lis
+            ('🔰', None),                          # 🔰 Japanese symbol for beginner
+            ('🔱', None),                          # 🔱 trident emblem
+            ('⭕', None),                          # ⭕ heavy large circle
+            ('✅', None),                          # ✅ white heavy check mark
+            ('☑', None),                          # ☑ ballot box with check
+            ('✔', None),                          # ✔ heavy check mark
+            ('✖', None),                          # ✖ heavy multiplication x
+            ('❌', None),                          # ❌ cross mark
+            ('❎', None),                          # ❎ cross mark button
+            ('➕', None),                          # ➕ heavy plus sign
+            ('♀', None),                          # ♀ female sign
+            ('♂', None),                          # ♂ male sign
+            ('⚕', None),                          # ⚕ medical symbol
+            ('➖', None),                          # ➖ heavy minus sign
+            ('➗', None),                          # ➗ heavy division sign
+            ('➰', None),                          # ➰ curly loop
+            ('➿', None),                          # ➿ double curly loop
+            ('〽', None),                          # 〽 part alternation mark
+            ('✳', None),                          # ✳ eight-spoked asterisk
+            ('✴', None),                          # ✴ eight-pointed star
+            ('❇', None),                          # ❇ sparkle
+            ('‼', None),                          # ‼ double exclamation mark
+            ('⁉', None),                          # ⁉ exclamation question mark
+            ('❓', None),                          # ❓ question mark
+            ('❔', None),                          # ❔ white question mark
+            ('❕', None),                          # ❕ white exclamation mark
+            ('❗', None),                          # ❗ exclamation mark
+            ('〰', None),                          # 〰 wavy dash
+            ('©', None),                          # © copyright
+            ('®', None),                          # ® registered
+            ('™', None),                          # ™ trade mark
+            ('#️⃣', None),                        # #️⃣ keycap: #
+            ('*️⃣', None),                        # *️⃣ keycap: *
+            ('0️⃣', None),                        # 0️⃣ keycap: 0
+            ('1️⃣', None),                        # 1️⃣ keycap: 1
+            ('2️⃣', None),                        # 2️⃣ keycap: 2
+            ('3️⃣', None),                        # 3️⃣ keycap: 3
+            ('4️⃣', None),                        # 4️⃣ keycap: 4
+            ('5️⃣', None),                        # 5️⃣ keycap: 5
+            ('6️⃣', None),                        # 6️⃣ keycap: 6
+            ('7️⃣', None),                        # 7️⃣ keycap: 7
+            ('8️⃣', None),                        # 8️⃣ keycap: 8
+            ('9️⃣', None),                        # 9️⃣ keycap: 9
+            ('🔟', None),                          # 🔟 keycap 10
+            ('💯', None),                          # 💯 hundred points
+            ('🔠', None),                          # 🔠 input latin uppercase
+            ('🔡', None),                          # 🔡 input latin lowercase
+            ('🔢', None),                          # 🔢 input numbers
+            ('🔣', None),                          # 🔣 input symbols
+            ('🔤', None),                          # 🔤 input latin letters
+            ('🅰', None),                          # 🅰 A button (blood type)
+            ('🆎', None),                          # 🆎 AB button (blood type)
+            ('🅱', None),                          # 🅱 B button (blood type)
+            ('🆑', None),                          # 🆑 CL button
+            ('🆒', None),                          # 🆒 COOL button
+            ('🆓', None),                          # 🆓 FREE button
+            ('ℹ', None),                          # ℹ information
+            ('🆔', None),                          # 🆔 ID button
+            ('Ⓜ', None),                          # Ⓜ circled M
+            ('🆕', None),                          # 🆕 NEW button
+            ('🆖', None),                          # 🆖 NG button
+            ('🅾', None),                          # 🅾 O button (blood type)
+            ('🆗', None),                          # 🆗 OK button
+            ('🅿', None),                          # 🅿 P button
+            ('🆘', None),                          # 🆘 SOS button
+            ('🆙', None),                          # 🆙 UP! button
+            ('🆚', None),                          # 🆚 VS button
+            ('🈁', None),                          # 🈁 Japanese “here” button
+            ('🈂', None),                          # 🈂 Japanese “service charge” button
+            ('🈷', None),                          # 🈷 Japanese “monthly amount” button
+            ('🈶', None),                          # 🈶 Japanese “not free of charge” button
+            ('🈯', None),                          # 🈯 Japanese “reserved” button
+            ('🉐', None),                          # 🉐 Japanese “bargain” button
+            ('🈹', None),                          # 🈹 Japanese “discount” button
+            ('🈚', None),                          # 🈚 Japanese “free of charge” button
+            ('🈲', None),                          # 🈲 Japanese “prohibited” button
+            ('🉑', None),                          # 🉑 Japanese “acceptable” button
+            ('🈸', None),                          # 🈸 Japanese “application” button
+            ('🈴', None),                          # 🈴 Japanese “passing grade” button
+            ('🈳', None),                          # 🈳 Japanese “vacancy” button
+            ('㊗', None),                          # ㊗ Japanese “congratulations” button
+            ('㊙', None),                          # ㊙ Japanese “secret” button
+            ('🈺', None),                          # 🈺 Japanese “open for business” button
+            ('🈵', None),                          # 🈵 Japanese “no vacancy” button
+            ('▪', None),                          # ▪ black small square
+            ('▫', None),                          # ▫ white small square
+            ('◻', None),                          # ◻ white medium square
+            ('◼', None),                          # ◼ black medium square
+            ('◽', None),                          # ◽ white medium-small square
+            ('◾', None),                          # ◾ black medium-small square
+            ('⬛', None),                          # ⬛ black large square
+            ('⬜', None),                          # ⬜ white large square
+            ('🔶', None),                          # 🔶 large orange diamond
+            ('🔷', None),                          # 🔷 large blue diamond
+            ('🔸', None),                          # 🔸 small orange diamond
+            ('🔹', None),                          # 🔹 small blue diamond
+            ('🔺', None),                          # 🔺 red triangle pointed up
+            ('🔻', None),                          # 🔻 red triangle pointed down
+            ('💠', None),                          # 💠 diamond with a dot
+            ('🔘', None),                          # 🔘 radio button
+            ('🔲', None),                          # 🔲 black square button
+            ('🔳', None),                          # 🔳 white square button
+            ('⚪', None),                          # ⚪ white circle
+            ('⚫', None),                          # ⚫ black circle
+            ('🔴', None),                          # 🔴 red circle
+            ('🔵', None),                          # 🔵 blue circle
+        ]],
+    ['🏁',                                         # category: Flags
+        [
+            ('🏁', None),                          # 🏁 chequered flag
+            ('🚩', None),                          # 🚩 triangular flag
+            ('🎌', None),                          # 🎌 crossed flags
+            ('🏴', None),                          # 🏴 black flag
+            ('🏳', None),                          # 🏳 white flag
+            ('🏳️\u200d🌈', None),                  # 🏳️‍🌈 rainbow flag
+            ('🇦🇨', None),                         # 🇦🇨 Ascension Island
+            ('🇦🇩', None),                         # 🇦🇩 Andorra
+            ('🇦🇪', None),                         # 🇦🇪 United Arab Emirates
+            ('🇦🇫', None),                         # 🇦🇫 Afghanistan
+            ('🇦🇬', None),                         # 🇦🇬 Antigua & Barbuda
+            ('🇦🇮', None),                         # 🇦🇮 Anguilla
+            ('🇦🇱', None),                         # 🇦🇱 Albania
+            ('🇦🇲', None),                         # 🇦🇲 Armenia
+            ('🇦🇴', None),                         # 🇦🇴 Angola
+            ('🇦🇶', None),                         # 🇦🇶 Antarctica
+            ('🇦🇷', None),                         # 🇦🇷 Argentina
+            ('🇦🇸', None),                         # 🇦🇸 American Samoa
+            ('🇦🇹', None),                         # 🇦🇹 Austria
+            ('🇦🇺', None),                         # 🇦🇺 Australia
+            ('🇦🇼', None),                         # 🇦🇼 Aruba
+            ('🇦🇽', None),                         # 🇦🇽 Åland Islands
+            ('🇦🇿', None),                         # 🇦🇿 Azerbaijan
+            ('🇧🇦', None),                         # 🇧🇦 Bosnia & Herzegovina
+            ('🇧🇧', None),                         # 🇧🇧 Barbados
+            ('🇧🇩', None),                         # 🇧🇩 Bangladesh
+            ('🇧🇪', None),                         # 🇧🇪 Belgium
+            ('🇧🇫', None),                         # 🇧🇫 Burkina Faso
+            ('🇧🇬', None),                         # 🇧🇬 Bulgaria
+            ('🇧🇭', None),                         # 🇧🇭 Bahrain
+            ('🇧🇮', None),                         # 🇧🇮 Burundi
+            ('🇧🇯', None),                         # 🇧🇯 Benin
+            ('🇧🇱', None),                         # 🇧🇱 St. Barthélemy
+            ('🇧🇲', None),                         # 🇧🇲 Bermuda
+            ('🇧🇳', None),                         # 🇧🇳 Brunei
+            ('🇧🇴', None),                         # 🇧🇴 Bolivia
+            ('🇧🇶', None),                         # 🇧🇶 Caribbean Netherlands
+            ('🇧🇷', None),                         # 🇧🇷 Brazil
+            ('🇧🇸', None),                         # 🇧🇸 Bahamas
+            ('🇧🇹', None),                         # 🇧🇹 Bhutan
+            ('🇧🇻', None),                         # 🇧🇻 Bouvet Island
+            ('🇧🇼', None),                         # 🇧🇼 Botswana
+            ('🇧🇾', None),                         # 🇧🇾 Belarus
+            ('🇧🇿', None),                         # 🇧🇿 Belize
+            ('🇨🇦', None),                         # 🇨🇦 Canada
+            ('🇨🇨', None),                         # 🇨🇨 Cocos (Keeling) Islands
+            ('🇨🇩', None),                         # 🇨🇩 Congo - Kinshasa
+            ('🇨🇫', None),                         # 🇨🇫 Central African Republic
+            ('🇨🇬', None),                         # 🇨🇬 Congo - Brazzaville
+            ('🇨🇭', None),                         # 🇨🇭 Switzerland
+            ('🇨🇮', None),                         # 🇨🇮 Côte d’Ivoire
+            ('🇨🇰', None),                         # 🇨🇰 Cook Islands
+            ('🇨🇱', None),                         # 🇨🇱 Chile
+            ('🇨🇲', None),                         # 🇨🇲 Cameroon
+            ('🇨🇳', None),                         # 🇨🇳 China
+            ('🇨🇴', None),                         # 🇨🇴 Colombia
+            ('🇨🇵', None),                         # 🇨🇵 Clipperton Island
+            ('🇨🇷', None),                         # 🇨🇷 Costa Rica
+            ('🇨🇺', None),                         # 🇨🇺 Cuba
+            ('🇨🇻', None),                         # 🇨🇻 Cape Verde
+            ('🇨🇼', None),                         # 🇨🇼 Curaçao
+            ('🇨🇽', None),                         # 🇨🇽 Christmas Island
+            ('🇨🇾', None),                         # 🇨🇾 Cyprus
+            ('🇨🇿', None),                         # 🇨🇿 Czech Republic
+            ('🇩🇪', None),                         # 🇩🇪 Germany
+            ('🇩🇬', None),                         # 🇩🇬 Diego Garcia
+            ('🇩🇯', None),                         # 🇩🇯 Djibouti
+            ('🇩🇰', None),                         # 🇩🇰 Denmark
+            ('🇩🇲', None),                         # 🇩🇲 Dominica
+            ('🇩🇴', None),                         # 🇩🇴 Dominican Republic
+            ('🇩🇿', None),                         # 🇩🇿 Algeria
+            ('🇪🇦', None),                         # 🇪🇦 Ceuta & Melilla
+            ('🇪🇨', None),                         # 🇪🇨 Ecuador
+            ('🇪🇪', None),                         # 🇪🇪 Estonia
+            ('🇪🇬', None),                         # 🇪🇬 Egypt
+            ('🇪🇭', None),                         # 🇪🇭 Western Sahara
+            ('🇪🇷', None),                         # 🇪🇷 Eritrea
+            ('🇪🇸', None),                         # 🇪🇸 Spain
+            ('🇪🇹', None),                         # 🇪🇹 Ethiopia
+            ('🇪🇺', None),                         # 🇪🇺 European Union
+            ('🇫🇮', None),                         # 🇫🇮 Finland
+            ('🇫🇯', None),                         # 🇫🇯 Fiji
+            ('🇫🇰', None),                         # 🇫🇰 Falkland Islands
+            ('🇫🇲', None),                         # 🇫🇲 Micronesia
+            ('🇫🇴', None),                         # 🇫🇴 Faroe Islands
+            ('🇫🇷', None),                         # 🇫🇷 France
+            ('🇬🇦', None),                         # 🇬🇦 Gabon
+            ('🇬🇧', None),                         # 🇬🇧 United Kingdom
+            ('🇬🇩', None),                         # 🇬🇩 Grenada
+            ('🇬🇪', None),                         # 🇬🇪 Georgia
+            ('🇬🇫', None),                         # 🇬🇫 French Guiana
+            ('🇬🇬', None),                         # 🇬🇬 Guernsey
+            ('🇬🇭', None),                         # 🇬🇭 Ghana
+            ('🇬🇮', None),                         # 🇬🇮 Gibraltar
+            ('🇬🇱', None),                         # 🇬🇱 Greenland
+            ('🇬🇲', None),                         # 🇬🇲 Gambia
+            ('🇬🇳', None),                         # 🇬🇳 Guinea
+            ('🇬🇵', None),                         # 🇬🇵 Guadeloupe
+            ('🇬🇶', None),                         # 🇬🇶 Equatorial Guinea
+            ('🇬🇷', None),                         # 🇬🇷 Greece
+            ('🇬🇸', None),                         # 🇬🇸 South Georgia & South Sandwich Islands
+            ('🇬🇹', None),                         # 🇬🇹 Guatemala
+            ('🇬🇺', None),                         # 🇬🇺 Guam
+            ('🇬🇼', None),                         # 🇬🇼 Guinea-Bissau
+            ('🇬🇾', None),                         # 🇬🇾 Guyana
+            ('🇭🇰', None),                         # 🇭🇰 Hong Kong SAR China
+            ('🇭🇲', None),                         # 🇭🇲 Heard & McDonald Islands
+            ('🇭🇳', None),                         # 🇭🇳 Honduras
+            ('🇭🇷', None),                         # 🇭🇷 Croatia
+            ('🇭🇹', None),                         # 🇭🇹 Haiti
+            ('🇭🇺', None),                         # 🇭🇺 Hungary
+            ('🇮🇨', None),                         # 🇮🇨 Canary Islands
+            ('🇮🇩', None),                         # 🇮🇩 Indonesia
+            ('🇮🇪', None),                         # 🇮🇪 Ireland
+            ('🇮🇱', None),                         # 🇮🇱 Israel
+            ('🇮🇲', None),                         # 🇮🇲 Isle of Man
+            ('🇮🇳', None),                         # 🇮🇳 India
+            ('🇮🇴', None),                         # 🇮🇴 British Indian Ocean Territory
+            ('🇮🇶', None),                         # 🇮🇶 Iraq
+            ('🇮🇷', None),                         # 🇮🇷 Iran
+            ('🇮🇸', None),                         # 🇮🇸 Iceland
+            ('🇮🇹', None),                         # 🇮🇹 Italy
+            ('🇯🇪', None),                         # 🇯🇪 Jersey
+            ('🇯🇲', None),                         # 🇯🇲 Jamaica
+            ('🇯🇴', None),                         # 🇯🇴 Jordan
+            ('🇯🇵', None),                         # 🇯🇵 Japan
+            ('🇰🇪', None),                         # 🇰🇪 Kenya
+            ('🇰🇬', None),                         # 🇰🇬 Kyrgyzstan
+            ('🇰🇭', None),                         # 🇰🇭 Cambodia
+            ('🇰🇮', None),                         # 🇰🇮 Kiribati
+            ('🇰🇲', None),                         # 🇰🇲 Comoros
+            ('🇰🇳', None),                         # 🇰🇳 St. Kitts & Nevis
+            ('🇰🇵', None),                         # 🇰🇵 North Korea
+            ('🇰🇷', None),                         # 🇰🇷 South Korea
+            ('🇰🇼', None),                         # 🇰🇼 Kuwait
+            ('🇰🇾', None),                         # 🇰🇾 Cayman Islands
+            ('🇰🇿', None),                         # 🇰🇿 Kazakhstan
+            ('🇱🇦', None),                         # 🇱🇦 Laos
+            ('🇱🇧', None),                         # 🇱🇧 Lebanon
+            ('🇱🇨', None),                         # 🇱🇨 St. Lucia
+            ('🇱🇮', None),                         # 🇱🇮 Liechtenstein
+            ('🇱🇰', None),                         # 🇱🇰 Sri Lanka
+            ('🇱🇷', None),                         # 🇱🇷 Liberia
+            ('🇱🇸', None),                         # 🇱🇸 Lesotho
+            ('🇱🇹', None),                         # 🇱🇹 Lithuania
+            ('🇱🇺', None),                         # 🇱🇺 Luxembourg
+            ('🇱🇻', None),                         # 🇱🇻 Latvia
+            ('🇱🇾', None),                         # 🇱🇾 Libya
+            ('🇲🇦', None),                         # 🇲🇦 Morocco
+            ('🇲🇨', None),                         # 🇲🇨 Monaco
+            ('🇲🇩', None),                         # 🇲🇩 Moldova
+            ('🇲🇪', None),                         # 🇲🇪 Montenegro
+            ('🇲🇫', None),                         # 🇲🇫 St. Martin
+            ('🇲🇬', None),                         # 🇲🇬 Madagascar
+            ('🇲🇭', None),                         # 🇲🇭 Marshall Islands
+            ('🇲🇰', None),                         # 🇲🇰 Macedonia
+            ('🇲🇱', None),                         # 🇲🇱 Mali
+            ('🇲🇲', None),                         # 🇲🇲 Myanmar (Burma)
+            ('🇲🇳', None),                         # 🇲🇳 Mongolia
+            ('🇲🇴', None),                         # 🇲🇴 Macau SAR China
+            ('🇲🇵', None),                         # 🇲🇵 Northern Mariana Islands
+            ('🇲🇶', None),                         # 🇲🇶 Martinique
+            ('🇲🇷', None),                         # 🇲🇷 Mauritania
+            ('🇲🇸', None),                         # 🇲🇸 Montserrat
+            ('🇲🇹', None),                         # 🇲🇹 Malta
+            ('🇲🇺', None),                         # 🇲🇺 Mauritius
+            ('🇲🇻', None),                         # 🇲🇻 Maldives
+            ('🇲🇼', None),                         # 🇲🇼 Malawi
+            ('🇲🇽', None),                         # 🇲🇽 Mexico
+            ('🇲🇾', None),                         # 🇲🇾 Malaysia
+            ('🇲🇿', None),                         # 🇲🇿 Mozambique
+            ('🇳🇦', None),                         # 🇳🇦 Namibia
+            ('🇳🇨', None),                         # 🇳🇨 New Caledonia
+            ('🇳🇪', None),                         # 🇳🇪 Niger
+            ('🇳🇫', None),                         # 🇳🇫 Norfolk Island
+            ('🇳🇬', None),                         # 🇳🇬 Nigeria
+            ('🇳🇮', None),                         # 🇳🇮 Nicaragua
+            ('🇳🇱', None),                         # 🇳🇱 Netherlands
+            ('🇳🇴', None),                         # 🇳🇴 Norway
+            ('🇳🇵', None),                         # 🇳🇵 Nepal
+            ('🇳🇷', None),                         # 🇳🇷 Nauru
+            ('🇳🇺', None),                         # 🇳🇺 Niue
+            ('🇳🇿', None),                         # 🇳🇿 New Zealand
+            ('🇴🇲', None),                         # 🇴🇲 Oman
+            ('🇵🇦', None),                         # 🇵🇦 Panama
+            ('🇵🇪', None),                         # 🇵🇪 Peru
+            ('🇵🇫', None),                         # 🇵🇫 French Polynesia
+            ('🇵🇬', None),                         # 🇵🇬 Papua New Guinea
+            ('🇵🇭', None),                         # 🇵🇭 Philippines
+            ('🇵🇰', None),                         # 🇵🇰 Pakistan
+            ('🇵🇱', None),                         # 🇵🇱 Poland
+            ('🇵🇲', None),                         # 🇵🇲 St. Pierre & Miquelon
+            ('🇵🇳', None),                         # 🇵🇳 Pitcairn Islands
+            ('🇵🇷', None),                         # 🇵🇷 Puerto Rico
+            ('🇵🇸', None),                         # 🇵🇸 Palestinian Territories
+            ('🇵🇹', None),                         # 🇵🇹 Portugal
+            ('🇵🇼', None),                         # 🇵🇼 Palau
+            ('🇵🇾', None),                         # 🇵🇾 Paraguay
+            ('🇶🇦', None),                         # 🇶🇦 Qatar
+            ('🇷🇪', None),                         # 🇷🇪 Réunion
+            ('🇷🇴', None),                         # 🇷🇴 Romania
+            ('🇷🇸', None),                         # 🇷🇸 Serbia
+            ('🇷🇺', None),                         # 🇷🇺 Russia
+            ('🇷🇼', None),                         # 🇷🇼 Rwanda
+            ('🇸🇦', None),                         # 🇸🇦 Saudi Arabia
+            ('🇸🇧', None),                         # 🇸🇧 Solomon Islands
+            ('🇸🇨', None),                         # 🇸🇨 Seychelles
+            ('🇸🇩', None),                         # 🇸🇩 Sudan
+            ('🇸🇪', None),                         # 🇸🇪 Sweden
+            ('🇸🇬', None),                         # 🇸🇬 Singapore
+            ('🇸🇭', None),                         # 🇸🇭 St. Helena
+            ('🇸🇮', None),                         # 🇸🇮 Slovenia
+            ('🇸🇯', None),                         # 🇸🇯 Svalbard & Jan Mayen
+            ('🇸🇰', None),                         # 🇸🇰 Slovakia
+            ('🇸🇱', None),                         # 🇸🇱 Sierra Leone
+            ('🇸🇲', None),                         # 🇸🇲 San Marino
+            ('🇸🇳', None),                         # 🇸🇳 Senegal
+            ('🇸🇴', None),                         # 🇸🇴 Somalia
+            ('🇸🇷', None),                         # 🇸🇷 Suriname
+            ('🇸🇸', None),                         # 🇸🇸 South Sudan
+            ('🇸🇹', None),                         # 🇸🇹 São Tomé & Príncipe
+            ('🇸🇻', None),                         # 🇸🇻 El Salvador
+            ('🇸🇽', None),                         # 🇸🇽 Sint Maarten
+            ('🇸🇾', None),                         # 🇸🇾 Syria
+            ('🇸🇿', None),                         # 🇸🇿 Swaziland
+            ('🇹🇦', None),                         # 🇹🇦 Tristan da Cunha
+            ('🇹🇨', None),                         # 🇹🇨 Turks & Caicos Islands
+            ('🇹🇩', None),                         # 🇹🇩 Chad
+            ('🇹🇫', None),                         # 🇹🇫 French Southern Territories
+            ('🇹🇬', None),                         # 🇹🇬 Togo
+            ('🇹🇭', None),                         # 🇹🇭 Thailand
+            ('🇹🇯', None),                         # 🇹🇯 Tajikistan
+            ('🇹🇰', None),                         # 🇹🇰 Tokelau
+            ('🇹🇱', None),                         # 🇹🇱 Timor-Leste
+            ('🇹🇲', None),                         # 🇹🇲 Turkmenistan
+            ('🇹🇳', None),                         # 🇹🇳 Tunisia
+            ('🇹🇴', None),                         # 🇹🇴 Tonga
+            ('🇹🇷', None),                         # 🇹🇷 Turkey
+            ('🇹🇹', None),                         # 🇹🇹 Trinidad & Tobago
+            ('🇹🇻', None),                         # 🇹🇻 Tuvalu
+            ('🇹🇼', None),                         # 🇹🇼 Taiwan
+            ('🇹🇿', None),                         # 🇹🇿 Tanzania
+            ('🇺🇦', None),                         # 🇺🇦 Ukraine
+            ('🇺🇬', None),                         # 🇺🇬 Uganda
+            ('🇺🇲', None),                         # 🇺🇲 U.S. Outlying Islands
+            ('🇺🇳', None),                         # 🇺🇳 United Nations
+            ('🇺🇸', None),                         # 🇺🇸 United States
+            ('🇺🇾', None),                         # 🇺🇾 Uruguay
+            ('🇺🇿', None),                         # 🇺🇿 Uzbekistan
+            ('🇻🇦', None),                         # 🇻🇦 Vatican City
+            ('🇻🇨', None),                         # 🇻🇨 St. Vincent & Grenadines
+            ('🇻🇪', None),                         # 🇻🇪 Venezuela
+            ('🇻🇬', None),                         # 🇻🇬 British Virgin Islands
+            ('🇻🇮', None),                         # 🇻🇮 U.S. Virgin Islands
+            ('🇻🇳', None),                         # 🇻🇳 Vietnam
+            ('🇻🇺', None),                         # 🇻🇺 Vanuatu
+            ('🇼🇫', None),                         # 🇼🇫 Wallis & Futuna
+            ('🇼🇸', None),                         # 🇼🇸 Samoa
+            ('🇽🇰', None),                         # 🇽🇰 Kosovo
+            ('🇾🇪', None),                         # 🇾🇪 Yemen
+            ('🇾🇹', None),                         # 🇾🇹 Mayotte
+            ('🇿🇦', None),                         # 🇿🇦 South Africa
+            ('🇿🇲', None),                         # 🇿🇲 Zambia
+            ('🇿🇼', None),                         # 🇿🇼 Zimbabwe
+        ]],
+]

=== modified file 'Onboard/utils.py'
--- Onboard/utils.py	2016-08-15 15:08:44 +0000
+++ Onboard/utils.py	2017-02-10 10:33:11 +0000
@@ -29,7 +29,7 @@
 import colorsys
 import gettext
 import subprocess
-from math import pi, sin, cos, sqrt, log
+from math import pi, sin, cos, sqrt, log, ceil
 from contextlib import contextmanager
 
 import logging
@@ -604,30 +604,30 @@
         >>> Rect(0, 1, 1, 1).intersects(Rect(0, 0, 1, 1))
         False
         """
-        #return not self.intersection(rect).is_empty()
-        return not (self.x >= rect.x + rect.w or \
-                    self.x + self.w <= rect.x or \
-                    self.y >= rect.y + rect.h or \
+        # return not self.intersection(rect).is_empty()
+        return not (self.x >= rect.x + rect.w or
+                    self.x + self.w <= rect.x or
+                    self.y >= rect.y + rect.h or
                     self.y + self.h <= rect.y)
 
     def intersection(self, rect):
-       x0 = max(self.x, rect.x)
-       y0 = max(self.y, rect.y)
-       x1 = min(self.x + self.w,  rect.x + rect.w)
-       y1 = min(self.y + self.h,  rect.y + rect.h)
-       if x0 > x1 or y0 > y1:
-           return Rect()
-       else:
-           return Rect(x0, y0, x1 - x0, y1 - y0)
+        x0 = max(self.x, rect.x)
+        y0 = max(self.y, rect.y)
+        x1 = min(self.x + self.w, rect.x + rect.w)
+        y1 = min(self.y + self.h, rect.y + rect.h)
+        if x0 > x1 or y0 > y1:
+            return Rect()
+        else:
+            return Rect(x0, y0, x1 - x0, y1 - y0)
 
     def union(self, rect):
-       x0 = min(self.x, rect.x)
-       y0 = min(self.y, rect.y)
-       x1 = max(self.x + self.w,  rect.x + rect.w)
-       y1 = max(self.y + self.h,  rect.y + rect.h)
-       return Rect(x0, y0, x1 - x0, y1 - y0)
+        x0 = min(self.x, rect.x)
+        y0 = min(self.y, rect.y)
+        x1 = max(self.x + self.w, rect.x + rect.w)
+        y1 = max(self.y + self.h, rect.y + rect.h)
+        return Rect(x0, y0, x1 - x0, y1 - y0)
 
-    def inscribe_with_aspect(self, rect, x_align = 0.5, y_align = 0.5):
+    def inscribe_with_aspect(self, rect, x_align=0.5, y_align=0.5):
         """ Returns a new Rect with the aspect ratio of self
             that fits inside the given rectangle.
         """
@@ -722,6 +722,64 @@
 
         return rects
 
+    def flow_layout(self, item_rect, num_items,
+                    x_spacing=None, y_spacing=None,
+                    row_first=True):
+        """
+        Layout num_items sub-rectangles of size item_rect in row_first or
+        columns-first order.
+        """
+
+        if y_spacing is None:
+            y_spacing = x_spacing
+        if x_spacing is None:
+            x_spacing = 0
+
+        rects = []
+        bounds = self.copy()
+
+        if row_first:
+            # self determines width and minimum height.
+            # Items may overflow the minimum height.
+            ncols = int((self.w + x_spacing) / (item_rect.h + x_spacing))
+            nrows = int(ceil(num_items / ncols))
+
+            for row in range(nrows):
+                for col in range(ncols):
+                    x = self.x + item_rect.w * col + \
+                        x_spacing * max((col - 1), 0)
+                    y = self.y + item_rect.h * row + \
+                        y_spacing * max((row - 1), 0)
+                    r = Rect(x, y, item_rect.w, item_rect.h)
+                    rects.append(r)
+
+                    bounds.h = max(r.bottom(), self.bottom()) - self.top()
+
+                    if len(rects) >= num_items:
+                        break
+
+        else:
+            # self determines height and minimum width.
+            # Items may overflow the minimum width.
+            nrows = int((self.h + y_spacing) / (item_rect.h + y_spacing))
+            ncols = int(ceil(num_items / nrows))
+
+            for col in range(ncols):
+                for row in range(nrows):
+                    x = self.x + item_rect.w * col + \
+                        x_spacing * max((col - 1), 0)
+                    y = self.y + item_rect.h * row + \
+                        y_spacing * max((row - 1), 0)
+                    r = Rect(x, y, item_rect.w, item_rect.h)
+                    rects.append(r)
+
+                    bounds.w = max(r.right(), self.right()) - self.left()
+
+                    if len(rects) >= num_items:
+                        break
+
+        return rects, bounds
+
 
 def brighten(amount, r, g, b, a=0.0):
     """ Make the given color brighter by amount a [-1.0...1.0] """
@@ -766,7 +824,7 @@
 
     context.close_path ()
 
-def roundrect_curve(context, rect, r_pct = 100):
+def roundrect_curve(context, rect, r_pct=100):
     """
     Uses B-splines for less even looks than with arcs, but
     still allows for approximate circles at r_pct = 100.
@@ -805,6 +863,66 @@
 
     context.close_path ()
 
+
+def roundrect_curve_custom(context, rect, r_pct=100,
+                           corner_mask=0b1111):
+    """
+    Uses B-splines for less even looks than with arcs, but
+    still allows for approximate circles at r_pct = 100.
+    """
+    x0 = rect.x
+    y0 = rect.y
+    w  = rect.w
+    h  = rect.h
+    x1 = x0 + w
+    y1 = y0 + h
+
+    # full range at 50%
+    r = min(w, h) * min(r_pct / 100.0, 0.5)
+
+    # position of control points for circular curves
+    k = (r - 1) * r_pct / 200.0
+
+    line_to = context.line_to
+    curve_to = context.curve_to
+
+    # top left
+    if corner_mask & 0b0001:
+        context.move_to(x0 + r, y0)
+    else:
+        context.move_to(x0, y0)
+
+    # top right
+    if corner_mask & 0b0010:
+        line_to(x1 - r, y0)
+        curve_to(x1 - k, y0, x1, y0 + k, x1, y0 + r)
+    else:
+        line_to(x1, y0)
+
+    # bottom right
+    if corner_mask & 0b0100:
+        line_to(x1, y1 - r)
+        curve_to(x1, y1 - k, x1 - k, y1, x1 - r, y1)
+    else:
+        line_to(x1, y1)
+
+    # bottom left
+    if corner_mask & 0b1000:
+        line_to(x0 + r, y1)
+        curve_to(x0 + k, y1, x0, y1 - k, x0, y1 - r)
+    else:
+        line_to(x0, y1)
+
+    # top left
+    if corner_mask & 0b0001:
+        line_to(x0, y0 + r)
+        curve_to(x0, y0 + k, x0 + k, y0, x0 + r, y0)
+    else:
+        line_to(x0, y0)
+
+    context.close_path()
+
+
 def rounded_polygon(cr, coords, r_pct, chamfer_size):
     path = polygon_to_rounded_path(coords, r_pct, chamfer_size)
     rounded_polygon_path_to_cairo_path(cr, path)
@@ -1006,6 +1124,9 @@
     # child items
     items = ()
 
+    def get_id(self):
+        return self.id
+
     def set_items(self, items):
         self.items = items
         for item in items:
@@ -1031,6 +1152,13 @@
     def get_parent(self):
         return self.parent
 
+    def find_id(self, id):
+        """ find the first item with matching id """
+        for item in self.iter_items():
+            if item.id == id:
+                return item
+        return None
+
     def find_ids(self, ids):
         """ find all items with matching id """
         for item in self.iter_items():

=== modified file 'layouts/Compact-Utils.svg'
--- layouts/Compact-Utils.svg	2016-01-25 20:13:40 +0000
+++ layouts/Compact-Utils.svg	2017-02-10 10:33:11 +0000
@@ -13,10 +13,10 @@
    height="115"
    width="330"
    version="1"
-   inkscape:version="0.48.4 r9939"
+   inkscape:version="0.92.0 r15299"
    sodipodi:docname="Compact-Utils.svg">
   <defs
-     id="defs3548" />
+     id="defs3462" />
   <sodipodi:namedview
      pagecolor="#ffffff"
      bordercolor="#666666"
@@ -26,172 +26,242 @@
      guidetolerance="10"
      inkscape:pageopacity="0"
      inkscape:pageshadow="2"
-     inkscape:window-width="1212"
-     inkscape:window-height="589"
-     id="namedview3546"
+     inkscape:window-width="1196"
+     inkscape:window-height="546"
+     id="namedview3460"
      showgrid="false"
-     inkscape:zoom="2.2056798"
-     inkscape:cx="206.52741"
-     inkscape:cy="49.24716"
+     inkscape:zoom="2.6468157"
+     inkscape:cx="100.68439"
+     inkscape:cy="51.819549"
      inkscape:window-x="65"
      inkscape:window-y="24"
      inkscape:window-maximized="0"
      inkscape:current-layer="svg2" />
- <metadata id="metadata25">
-  <rdf:RDF>
-   <cc:Work rdf:about="">
-    <dc:format>image/svg+xml</dc:format>
-    <dc:type rdf:resource="http://purl.org/dc/dcmitype/StillImage"/>
-    <dc:title>Compact-Utils.svg</dc:title>
-    <cc:license rdf:resource="https://www.gnu.org/licenses/gpl.txt"/>
-    <dc:creator>
-     <cc:Agent>
-      <dc:title>Onboard Devel Team at https://launchpad.net/~onboard</dc:title>
-     </cc:Agent>
-    </dc:creator>
-    <dc:rights>
-     <cc:Agent>
-      <dc:title>GPL-3+ license</dc:title>
-     </cc:Agent>
-    </dc:rights>
-    <dc:publisher>
-     <cc:Agent>
-      <dc:title>Onboard Devel Team</dc:title>
-     </cc:Agent>
-    </dc:publisher>
-    <dc:source>https://launchpad.net/onboard</dc:source>
-    <dc:relation>This file is part of Onboard</dc:relation>
-    <dc:contributor>
-     <cc:Agent>
-      <dc:title>Copyright © 2013, 2016, Francesco Fumanti &lt;francesco.fumanti@gmx.net&gt;
+  <metadata
+     id="metadata43">
+    <rdf:RDF>
+      <cc:Work
+         rdf:about="">
+        <dc:format>image/svg+xml</dc:format>
+        <dc:type
+           rdf:resource="http://purl.org/dc/dcmitype/StillImage" />
+        <dc:title />
+        <cc:license
+           rdf:resource="https://www.gnu.org/licenses/gpl.txt" />
+        <dc:creator>
+          <cc:Agent>
+            <dc:title>Onboard Devel Team at https://launchpad.net/~onboard</dc:title>
+          </cc:Agent>
+        </dc:creator>
+        <dc:rights>
+          <cc:Agent>
+            <dc:title>GPL-3+ license</dc:title>
+          </cc:Agent>
+        </dc:rights>
+        <dc:publisher>
+          <cc:Agent>
+            <dc:title>Onboard Devel Team</dc:title>
+          </cc:Agent>
+        </dc:publisher>
+        <dc:source>https://launchpad.net/onboard</dc:source>
+        <dc:relation>This file is part of Onboard</dc:relation>
+        <dc:contributor>
+          <cc:Agent>
+            <dc:title>Copyright © 2013, 2016, Francesco Fumanti &lt;francesco.fumanti@gmx.net&gt;
 Copyright © 2011-2014, marmuta &lt;marmvta@gmail.com&gt;</dc:title>
-     </cc:Agent>
-    </dc:contributor>
-   </cc:Work>
-  </rdf:RDF>
- </metadata>
+          </cc:Agent>
+        </dc:contributor>
+      </cc:Work>
+    </rdf:RDF>
+  </metadata>
+  <rect
+     id="character-palette"
+     style="fill:#ffa636;fill-opacity:1;stroke:#000000;stroke-width:0.99999994"
+     height="94"
+     width="264.5"
+     y="20.5"
+     x="0.50000018"
+     inkscape:label="" />
+  <rect
+     id="palette-key-template"
+     style="fill:#ffe93c;fill-opacity:1;stroke:#000000;stroke-width:1"
+     height="18"
+     width="19"
+     y="39.5"
+     x="0.50000018"
+     inkscape:label="" />
+  <rect
+     id="palette-header-template"
+     style="fill:#ffe93c;fill-opacity:1;stroke:#000000;stroke-width:1"
+     height="18"
+     width="19"
+     y="20.5"
+     x="0.50000018"
+     inkscape:label="" />
+  <rect
+     x="266"
+     y="96.5"
+     width="18"
+     height="18"
+     style="fill:#e5d8b2;stroke:#000000;stroke-width:1"
+     id="sublayer2"
+     inkscape:label="" />
+  <rect
+     x="266"
+     y="77.5"
+     width="18"
+     height="18"
+     style="fill:#e5d8b2;stroke:#000000;stroke-width:1"
+     id="sublayer1"
+     inkscape:label="" />
+  <rect
+     x="266"
+     y="20.5"
+     width="18"
+     height="18"
+     style="fill:#e5d8b2;stroke:#000000;stroke-width:1"
+     id="BKSP"
+     inkscape:label="" />
+  <rect
+     x="266"
+     y="58.5"
+     width="18"
+     height="18"
+     style="fill:#e5d8b2;stroke:#000000;stroke-width:1"
+     id="sublayer0"
+     inkscape:label="" />
+  <rect
+     x="266"
+     y="39.5"
+     width="18"
+     height="18"
+     style="fill:#e5d8b2;stroke:#000000;stroke-width:1"
+     id="DELE"
+     inkscape:label="" />
   <rect
      id="m0"
      style="fill:#e5d8b2;stroke:#000000;stroke-width:0.86966002"
      height="20.153999"
      width="50.27"
-     y="20.434681"
-     x="62.434902" />
+     y="20.434853"
+     x="40.934895" />
   <rect
      id="m1"
      style="fill:#e5d8b2;stroke:#000000;stroke-width:0.86580998"
      height="20.150999"
      width="50.27"
-     y="20.436159"
-     x="118.50591" />
+     y="20.436333"
+     x="97.005913" />
   <rect
      id="m2"
      style="fill:#e5d8b2;stroke:#000000;stroke-width:0.86966002"
      height="20.153999"
      width="50.27"
-     y="20.434681"
-     x="174.5959" />
+     y="20.434853"
+     x="153.09593" />
   <rect
      id="m3"
      style="fill:#e5d8b2;stroke:#000000;stroke-width:0.86580998"
      height="20.150999"
      width="50.27"
-     y="20.436159"
-     x="230.6559" />
+     y="20.436333"
+     x="209.15593" />
   <rect
      id="m4"
      style="fill:#e5d8b2;stroke:#000000;stroke-width:0.86580998"
      height="20.150999"
      width="50.27"
-     y="45.095001"
-     x="62.432903" />
+     y="45.095169"
+     x="40.932896" />
   <rect
      id="m5"
      style="fill:#e5d8b2;stroke:#000000;stroke-width:0.86966002"
      height="20.153999"
      width="50.27"
-     y="45.093002"
-     x="118.51591" />
+     y="45.09317"
+     x="97.0159" />
   <rect
      id="m6"
      style="fill:#e5d8b2;stroke:#000000;stroke-width:0.86580998"
      height="20.150999"
      width="50.27"
-     y="45.095001"
-     x="174.5959" />
+     y="45.095169"
+     x="153.09593" />
   <rect
      id="m7"
      style="fill:#e5d8b2;stroke:#000000;stroke-width:0.86966002"
      height="20.153999"
      width="50.27"
-     y="45.093002"
-     x="230.66591" />
+     y="45.09317"
+     x="209.16594" />
   <rect
      id="m8"
      style="fill:#e5d8b2;stroke:#000000;stroke-width:0.86966002"
      height="20.153999"
      width="50.27"
-     y="69.751999"
-     x="62.434902" />
+     y="69.752174"
+     x="40.934895" />
   <rect
      id="m9"
      style="fill:#e5d8b2;stroke:#000000;stroke-width:0.86966002"
      height="20.153999"
      width="50.27"
-     y="69.751999"
-     x="118.51591" />
+     y="69.752174"
+     x="97.0159" />
   <rect
      id="m10"
      style="fill:#e5d8b2;stroke:#000000;stroke-width:0.86966002"
      height="20.153999"
      width="50.27"
-     y="69.751999"
-     x="174.5959" />
+     y="69.752174"
+     x="153.09593" />
   <rect
      id="m11"
      style="fill:#e5d8b2;stroke:#000000;stroke-width:0.86966002"
      height="20.153999"
      width="50.27"
-     y="69.751999"
-     x="230.66591" />
+     y="69.752174"
+     x="209.16594" />
   <rect
      id="m12"
      style="fill:#e5d8b2;stroke:#000000;stroke-width:0.86966002"
      height="20.153999"
      width="50.27"
-     y="94.411003"
-     x="62.711903" />
+     y="94.411171"
+     x="41.211895" />
   <rect
      id="m13"
      style="fill:#e5d8b2;stroke:#000000;stroke-width:0.86966002"
      height="20.153999"
      width="50.27"
-     y="94.411003"
-     x="118.51591" />
+     y="94.411171"
+     x="97.0159" />
   <rect
      id="m14"
      style="fill:#e5d8b2;stroke:#000000;stroke-width:0.86966002"
      height="20.153999"
      width="50.27"
-     y="94.411003"
-     x="174.5959" />
+     y="94.411171"
+     x="153.09593" />
   <rect
      id="m15"
      style="fill:#e5d8b2;stroke:#000000;stroke-width:0.86966002"
      height="20.153999"
      width="50.27"
-     y="94.411003"
-     x="230.66591" />
+     y="94.411171"
+     x="209.16594" />
   <rect
      id="settings"
-     height="20.153999"
-     width="40"
-     y="69.751999"
-     x="8" />
+     height="21.023998"
+     width="30"
+     y="69.317001"
+     x="5.4999995"
+     style="stroke-width:1" />
   <rect
      id="quit"
-     height="20.153999"
-     width="40"
-     y="94.411003"
-     x="8" />
+     height="21.023998"
+     width="30"
+     y="93.975998"
+     x="5.4999995"
+     style="stroke-width:1" />
 </svg>

=== modified file 'layouts/Compact.onboard'
--- layouts/Compact.onboard	2015-02-10 01:27:17 +0000
+++ layouts/Compact.onboard	2017-02-10 10:33:11 +0000
@@ -93,9 +93,9 @@
                         <key group='shifts'    id='LFSH'/>
                         <key group='shifts'    id='RTSH'/>
                         <key group='bottomrow' id='LCTL'/>
+                        <key group="bottomrow" id="LWIN"/>
                         <key group='bottomrow' id='LALT'/>
                         <key group='bottomrow' id='RALT'/>
-                        <key group="bottomrow" id="LWIN"/>
 
                         <key group="bottomrow" id="SPCE"/>
                         <key group="bottomrow" id="DELE.next-to-backspace"/>
@@ -153,26 +153,52 @@
                         <key group="editing" id="Pause" scan_priority="1"/>
                         <key group="editing" id="Scroll" scan_priority="1"/>
                     </panel>
-                    <panel layer="utils" filename="Compact-Utils.svg" border="2">
-                        <key group='snippets' id='m0'/>
-                        <key group='snippets' id='m1'/>
-                        <key group='snippets' id='m2'/>
-                        <key group='snippets' id='m3'/>
-                        <key group='snippets' id='m4'/>
-                        <key group='snippets' id='m5'/>
-                        <key group='snippets' id='m6'/>
-                        <key group='snippets' id='m7'/>
-                        <key group='snippets' id='m8'/>
-                        <key group='snippets' id='m9'/>
-                        <key group='snippets' id='m10'/>
-                        <key group='snippets' id='m11'/>
-                        <key group='snippets' id='m12'/>
-                        <key group='snippets' id='m13'/>
-                        <key group='snippets' id='m14'/>
-                        <key group='snippets' id='m15'/>
-                        <key group='bottomrow' id='quit' scan_priority="1"/>
-                        <key group='bottomrow' id='settings' scan_priority="1"/>
+
+                    <panel layer="utils" filename="Compact-Utils.svg">
+                        <panel layer="utils.emoji">
+                            <panel id='emoji-palette-panel' class="EmojiPalettePanel">
+                                <key id='character-palette' group='character-palette' label='' visible='false' sensitive='false' scannable='false'/>
+                                <key id='palette-header-template' group='character-palette' label='' visible='false' scannable='false'/>
+                                <key id='palette-key-template' group='character-palette' label='' visible='false' scannable='false'/>
+                            </panel>
+                        </panel>
+
+                        <panel layer="utils.symbols">
+                            <panel id='symbol-palette-panel' class="SymbolPalettePanel">
+                                <key id='character-palette' group='character-palette' label='' visible='false' sensitive='false' scannable='false'/>
+                                <key id='palette-header-template' group='character-palette' label='' visible='false' scannable='false'/>
+                                <key id='palette-key-template' group='character-palette' label='' visible='false' scannable='false'/>
+                            </panel>
+                        </panel>
+
+                        <panel layer="utils.snippets">
+                            <key group='snippets' id='m0'/>
+                            <key group='snippets' id='m1'/>
+                            <key group='snippets' id='m2'/>
+                            <key group='snippets' id='m3'/>
+                            <key group='snippets' id='m4'/>
+                            <key group='snippets' id='m5'/>
+                            <key group='snippets' id='m6'/>
+                            <key group='snippets' id='m7'/>
+                            <key group='snippets' id='m8'/>
+                            <key group='snippets' id='m9'/>
+                            <key group='snippets' id='m10'/>
+                            <key group='snippets' id='m11'/>
+                            <key group='snippets' id='m12'/>
+                            <key group='snippets' id='m13'/>
+                            <key group='snippets' id='m14'/>
+                            <key group='snippets' id='m15'/>
+                            <key group='bottomrow' id='quit' scan_priority="1"/>
+                            <key group='bottomrow' id='settings' scan_priority="1"/>
+                        </panel>
+
+                        <key id="BKSP" group="bottomrow" unlatch_layer="false"/>
+                        <key id="DELE" group="bottomrow" unlatch_layer="false"/>
+                        <key id='sublayer0' group='sublayer0' target_layer='utils.emoji' image='emoji.svg' button='true'/>
+                        <key id='sublayer1' group='bottomrow' target_layer='utils.symbols' label='Σ' button='true'/>
+                        <key id='sublayer2' group='bottomrow' target_layer='utils.snippets' image='snippets.svg' button='true'/>
                     </panel>
+
                 </panel>
 
             </box>
@@ -187,7 +213,7 @@
             </panel>
 
             <!--- side bar -->
-            <panel id="paneswitch" filename="Compact-Alpha.svg">        
+            <panel id="paneswitch" filename="Compact-Alpha.svg">
                 <box compact="true" orientation='vertical'>
                     <panel group='nowordlist'>
                         <key group='bottomrow' id='hide'/>
@@ -204,10 +230,10 @@
                     <panel group='wordlist'>
                         <box orientation='vertical'>
                             <key group='sidebar' id='move' svg_id='move.wordlist' expand='false' label_margin='2.5'/>
-                            <key group='sidebar' id='showclick' svg_id='showclick.wordlist' label_margin='2' expand='false'/>
-                            <key group='bottomrow' id='layer0' show_active="false" svg_id='layer0.wordlist' scan_priority='3'/>
+                            <key group='sidebar' id='showclick' svg_id='showclick.wordlist' scan_priority="3"/>
+                            <key group='bottomrow' id='layer0' svg_id='layer0.wordlist' show_active="false" scan_priority='3'/>
                             <key group='bottomrow' id='layer1' svg_id='layer1.wordlist' scan_priority='3'/>
-                            <key group="bottomrow" id="layer2" svg_id='layer2.wordlist' scan_priority="3"/>
+                            <key group="bottomrow" id="layer2" svg_id='layer2.wordlist' image="emoji_snippets.svg" scan_priority="3"/>
                         </box>
                     </panel>
                 </box>

=== added file 'layouts/images/emoji.svg'
--- layouts/images/emoji.svg	1970-01-01 00:00:00 +0000
+++ layouts/images/emoji.svg	2017-02-10 10:33:11 +0000
@@ -0,0 +1,102 @@
+<?xml version="1.0" encoding="UTF-8" standalone="no"?>
+<!-- Created with Inkscape (http://www.inkscape.org/) -->
+
+<svg
+   xmlns:dc="http://purl.org/dc/elements/1.1/"
+   xmlns:cc="http://creativecommons.org/ns#"
+   xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
+   xmlns:svg="http://www.w3.org/2000/svg"
+   xmlns="http://www.w3.org/2000/svg"
+   xmlns:sodipodi="http://sodipodi.sourceforge.net/DTD/sodipodi-0.dtd"
+   xmlns:inkscape="http://www.inkscape.org/namespaces/inkscape"
+   id="svg2985"
+   height="16"
+   width="16"
+   version="1.1"
+   inkscape:version="0.91 r13725"
+   sodipodi:docname="auxchars.svg">
+  <defs
+     id="defs3685" />
+  <sodipodi:namedview
+     pagecolor="#ffffff"
+     bordercolor="#666666"
+     borderopacity="1"
+     objecttolerance="10"
+     gridtolerance="10"
+     guidetolerance="10"
+     inkscape:pageopacity="0"
+     inkscape:pageshadow="2"
+     inkscape:window-width="640"
+     inkscape:window-height="480"
+     id="namedview3683"
+     showgrid="true"
+     inkscape:zoom="39.5"
+     inkscape:cx="3.4810127"
+     inkscape:cy="8"
+     inkscape:current-layer="svg2985">
+    <inkscape:grid
+       type="xygrid"
+       id="grid3773" />
+  </sodipodi:namedview>
+  <metadata
+     id="metadata2990">
+    <rdf:RDF>
+      <cc:Work
+         rdf:about="">
+        <dc:format>image/svg+xml</dc:format>
+        <dc:type
+           rdf:resource="http://purl.org/dc/dcmitype/StillImage" />
+        <dc:title></dc:title>
+        <cc:license
+           rdf:resource="https://www.gnu.org/licenses/gpl.txt" />
+        <dc:creator>
+          <cc:Agent>
+            <dc:title>Onboard Devel Team at https://launchpad.net/~onboard</dc:title>
+          </cc:Agent>
+        </dc:creator>
+        <dc:rights>
+          <cc:Agent>
+            <dc:title>GPL-3+ license</dc:title>
+          </cc:Agent>
+        </dc:rights>
+        <dc:publisher>
+          <cc:Agent>
+            <dc:title>Onboard Devel Team</dc:title>
+          </cc:Agent>
+        </dc:publisher>
+        <dc:source>https://launchpad.net/onboard</dc:source>
+        <dc:relation>This file is part of Onboard</dc:relation>
+        <dc:contributor>
+          <cc:Agent>
+            <dc:title>Copyright © 2016, Francesco Fumanti &lt;francesco.fumanti@gmx.net&gt;
+Copyright © 2012, 2014, marmuta &lt;marmvta@gmail.com&gt;</dc:title>
+          </cc:Agent>
+        </dc:contributor>
+      </cc:Work>
+    </rdf:RDF>
+  </metadata>
+  <circle
+     style="color:#000000;display:inline;overflow:visible;visibility:visible;opacity:1;fill:none;fill-opacity:0.91764706;fill-rule:nonzero;stroke:#000000;stroke-width:0.75;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-dasharray:none;stroke-dashoffset:0;stroke-opacity:1;marker:none;enable-background:accumulate"
+     id="path3783"
+     cx="8.125001"
+     cy="7.7231011"
+     r="4.6376581" />
+  <circle
+     style="color:#000000;display:inline;overflow:visible;visibility:visible;opacity:1;fill:none;fill-opacity:0.91764706;fill-rule:nonzero;stroke:#000000;stroke-width:0.75;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-dasharray:none;stroke-dashoffset:0;stroke-opacity:1;marker:none;enable-background:accumulate"
+     id="path3787"
+     cx="6.3488564"
+     cy="6.4492831"
+     r="0.60014987" />
+  <circle
+     r="0.60014987"
+     cy="6.4492831"
+     cx="9.9521551"
+     id="circle3789"
+     style="color:#000000;display:inline;overflow:visible;visibility:visible;opacity:1;fill:none;fill-opacity:0.91764706;fill-rule:nonzero;stroke:#000000;stroke-width:0.75;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-dasharray:none;stroke-dashoffset:0;stroke-opacity:1;marker:none;enable-background:accumulate" />
+  <path
+     style="color:#000000;display:inline;overflow:visible;visibility:visible;opacity:1;fill:none;fill-opacity:0.91764706;fill-rule:nonzero;stroke:#000000;stroke-width:0.5;stroke-linecap:round;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-dashoffset:0;stroke-opacity:1;marker:none;enable-background:accumulate"
+     d="M 5.2457809,8.8738888 C 6.8361781,10.824032 9.1497559,10.853589 11.064234,8.8498828 9.0563492,9.9331537 6.9006172,9.9331537 5.2457809,8.8738888 Z"
+     id="path3791"
+     inkscape:connector-curvature="0"
+     sodipodi:nodetypes="ccc" />
+</svg>

=== added file 'layouts/images/emoji_snippets.svg'
--- layouts/images/emoji_snippets.svg	1970-01-01 00:00:00 +0000
+++ layouts/images/emoji_snippets.svg	2017-02-10 10:33:11 +0000
@@ -0,0 +1,121 @@
+<?xml version="1.0" encoding="UTF-8" standalone="no"?>
+<!-- Created with Inkscape (http://www.inkscape.org/) -->
+
+<svg
+   xmlns:dc="http://purl.org/dc/elements/1.1/"
+   xmlns:cc="http://creativecommons.org/ns#"
+   xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
+   xmlns:svg="http://www.w3.org/2000/svg"
+   xmlns="http://www.w3.org/2000/svg"
+   xmlns:sodipodi="http://sodipodi.sourceforge.net/DTD/sodipodi-0.dtd"
+   xmlns:inkscape="http://www.inkscape.org/namespaces/inkscape"
+   id="svg2985"
+   height="16"
+   width="28"
+   version="1.1"
+   inkscape:version="0.92.0 r15299"
+   sodipodi:docname="emoji_snippets.svg">
+  <defs
+     id="defs3685" />
+  <sodipodi:namedview
+     pagecolor="#ffffff"
+     bordercolor="#666666"
+     borderopacity="1"
+     objecttolerance="10"
+     gridtolerance="10"
+     guidetolerance="10"
+     inkscape:pageopacity="0"
+     inkscape:pageshadow="2"
+     inkscape:window-width="640"
+     inkscape:window-height="480"
+     id="namedview3683"
+     showgrid="true"
+     inkscape:zoom="27.430556"
+     inkscape:cx="13.527657"
+     inkscape:cy="8.8101266"
+     inkscape:current-layer="svg2985">
+    <inkscape:grid
+       type="xygrid"
+       id="grid3773" />
+  </sodipodi:namedview>
+  <metadata
+     id="metadata2990">
+    <rdf:RDF>
+      <cc:Work
+         rdf:about="">
+        <dc:format>image/svg+xml</dc:format>
+        <dc:type
+           rdf:resource="http://purl.org/dc/dcmitype/StillImage" />
+        <dc:title></dc:title>
+        <cc:license
+           rdf:resource="https://www.gnu.org/licenses/gpl.txt" />
+        <dc:creator>
+          <cc:Agent>
+            <dc:title>Onboard Devel Team at https://launchpad.net/~onboard</dc:title>
+          </cc:Agent>
+        </dc:creator>
+        <dc:rights>
+          <cc:Agent>
+            <dc:title>GPL-3+ license</dc:title>
+          </cc:Agent>
+        </dc:rights>
+        <dc:publisher>
+          <cc:Agent>
+            <dc:title>Onboard Devel Team</dc:title>
+          </cc:Agent>
+        </dc:publisher>
+        <dc:source>https://launchpad.net/onboard</dc:source>
+        <dc:relation>This file is part of Onboard</dc:relation>
+        <dc:contributor>
+          <cc:Agent>
+            <dc:title>Copyright © 2016, Francesco Fumanti &lt;francesco.fumanti@gmx.net&gt;
+Copyright © 2012, 2014, marmuta &lt;marmvta@gmail.com&gt;</dc:title>
+          </cc:Agent>
+        </dc:contributor>
+      </cc:Work>
+    </rdf:RDF>
+  </metadata>
+  <circle
+     style="color:#000000;display:inline;overflow:visible;visibility:visible;opacity:1;fill:none;fill-opacity:0.91764706;fill-rule:nonzero;stroke:#000000;stroke-width:0.75;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-dasharray:none;stroke-dashoffset:0;stroke-opacity:1;marker:none;enable-background:accumulate"
+     id="path3783"
+     cx="8.4649496"
+     cy="7.6984925"
+     r="4.3943152" />
+  <circle
+     style="color:#000000;display:inline;overflow:visible;visibility:visible;opacity:1;fill:none;fill-opacity:0.91764706;fill-rule:nonzero;stroke:#000000;stroke-width:0.75;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-dasharray:none;stroke-dashoffset:0;stroke-opacity:1;marker:none;enable-background:accumulate"
+     id="path3787"
+     cx="6.782001"
+     cy="6.4915133"
+     r="0.56865937" />
+  <circle
+     r="0.56865937"
+     cy="6.4915133"
+     cx="10.196231"
+     id="circle3789"
+     style="color:#000000;display:inline;overflow:visible;visibility:visible;opacity:1;fill:none;fill-opacity:0.91764706;fill-rule:nonzero;stroke:#000000;stroke-width:0.75;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-dasharray:none;stroke-dashoffset:0;stroke-opacity:1;marker:none;enable-background:accumulate" />
+  <path
+     style="color:#000000;display:inline;overflow:visible;visibility:visible;opacity:1;fill:none;fill-opacity:0.91764706;fill-rule:nonzero;stroke:#000000;stroke-width:0.5;stroke-linecap:round;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-dashoffset:0;stroke-opacity:1;marker:none;enable-background:accumulate"
+     d="M 5.7368049,8.7888972 C 7.2437523,10.636714 9.4359343,10.66472 11.249958,8.7661508 9.3474287,9.7925814 7.3048102,9.7925814 5.7368049,8.7888972 Z"
+     id="path3791"
+     inkscape:connector-curvature="0"
+     sodipodi:nodetypes="ccc" />
+  <g
+     transform="translate(12.666667,0.50632911)"
+     id="layer1">
+    <path
+       inkscape:connector-curvature="0"
+       id="path4429"
+       style="fill:none;stroke:#000000;stroke-width:0.60000002"
+       d="m 4,6 h 6" />
+    <path
+       inkscape:connector-curvature="0"
+       id="path4431"
+       style="fill:none;stroke:#000000;stroke-width:0.60000002"
+       d="m 4,8 h 8" />
+    <path
+       inkscape:connector-curvature="0"
+       id="path4433"
+       style="fill:none;stroke:#000000;stroke-width:0.60000002"
+       d="m 4,10 h 7" />
+  </g>
+</svg>

=== modified file 'setup.py'
--- setup.py	2017-01-23 06:18:07 +0000
+++ setup.py	2017-02-10 10:33:11 +0000
@@ -381,6 +381,42 @@
         super(build_ext_custom, self).build_extension(ext)
 
 
+class UninstallCommand(Command):
+    user_options = []  # required by Command
+
+    dirs_to_remove = [
+        "share/onboard",
+    ]
+
+    def initialize_options(self):
+        pass
+
+    def finalize_options(self):
+        pass
+
+    def run(self):
+
+        record_file = "/tmp/onboard-uninstall.txt"
+        subprocess.call(["./setup.py", "install", "--record", record_file])
+
+        with open(record_file, 'r') as f:
+            lines = f.readlines()
+        for line in lines:
+            fn = line.strip()
+            print("removing file '" + fn + "'")
+            os.remove(fn)
+
+        for dir in self.dirs_to_remove:
+            print("removing directory '" + dir + "'")
+            try:
+                os.removedirs(dir)
+            except OSError as ex:
+                print("Failed to remove '" + dir + "':" +
+                      str(ex), file=sys.stderr)
+
+        sys.exit(0)
+
+
 ##### setup #####
 
 DistUtilsExtra.auto.setup(
@@ -436,7 +472,9 @@
 
     cmdclass = {'test': TestCommand,
                 'build_i18n': build_i18n_custom,
-                'build_ext': build_ext_custom},
+                'build_ext': build_ext_custom,
+                'uninstall': UninstallCommand,
+                }
 )
 
 

=== modified file 'themes/Charcoal.colors'
--- themes/Charcoal.colors	2015-02-10 01:27:17 +0000
+++ themes/Charcoal.colors	2017-02-10 10:33:11 +0000
@@ -53,8 +53,8 @@
             F1, F2, F3, F4, F5, F6, F7, F8, F9, F10, F11, F12,
             Prnt, Pause, ESC, Scroll,
             secondaryclick, middleclick, doubleclick, dragclick, hoverclick,
-            hide, showclick, move, layer,
-            quit, inputline,
+            hide, showclick, move, layer, sublayer,
+            quit, inputline 
 
             <!-- word suggestions -->
             <key_group>

=== added file 'tools/gen_unicode_data'
--- tools/gen_unicode_data	1970-01-01 00:00:00 +0000
+++ tools/gen_unicode_data	2017-02-10 10:33:11 +0000
@@ -0,0 +1,502 @@
+#!/usr/bin/python3
+# -*- coding: utf-8 -*-
+
+# Copyright © 2017 marmuta <marmvta@gmail.com>
+#
+# This file is part of Onboard.
+#
+# Onboard is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 3 of the License, or
+# (at your option) any later version.
+#
+# Onboard is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program. If not, see <http://www.gnu.org/licenses/>.
+
+
+import os
+import sys
+import locale
+from bisect import bisect_left
+from contextlib import contextmanager
+from xml.dom import minidom
+from urllib.request import urlopen
+import re
+from collections import OrderedDict
+
+
+class UnicodeData:
+
+    UNICODE_DATA_PATH = "unicode_data"
+
+    class CodePoint:
+        """
+        Class representing a single unicode code point.
+        """
+        code = 0
+
+        # General_category of UnicodeData.txt, English/untranslated
+        category = ""
+
+        # iso_10646_comment_field of UnicodeData.txt, English/untranslated
+        comment = ""
+
+        def __init__(self):
+            self.annotations = {}
+
+        def __str__(self):
+            return ('UnicodeData({},{},{})'
+                    .format(repr(self.code),
+                            repr(self.category),
+                            repr(self.comment),
+                            ))
+
+    class Annotation:
+        """ Annotation of a single entry in CLDR/common/annotations/*.xml """
+        # The code sequence can consist of one or more code-points.
+        # If there are multiple glyphs, they are apparently always joined
+        # by \u200d, the "zero width joiner" character.
+        code_sequence = ""
+
+        annotations = ()        # tuple of keywords, translated
+        annotation_tts = ""     # text-to-speech description, translated
+
+        def __str__(self):
+            return ('Annotation({},{},{})'
+                    .format(repr(self.code_sequence),
+                            repr(self.annotations),
+                            repr(self.annotation_tts)))
+
+    def __init__(self):
+        self._lang_ids = []
+
+        # All code points, sorted by code point
+        self._code_points = []        # list of code point objects
+        self._code_points_index = []  # list of integers for faster bisect
+
+        self._annotations = {}
+
+        # Emoji
+        # emoji-default: those expected to have an emoji presentation by
+        #                default, but can also have a text presentation
+        # text-default:  those expected to have a text presentation by default,
+        #                but could also have an emoji presentation
+        # text-only:     those that should only have a text presentation
+        self._emoji_default = set()
+        self._text_default = set()
+
+        # emoji that can be modified with a preceding modifier character
+        self._emoji_with_modifier = set()
+
+        # self.gen_unicode_data()
+
+    def set_language_ids(self, lang_ids):
+        """
+        Set multiple lang_ids to be active at the same time.
+        """
+        self._lang_ids = lang_ids
+        self._load_annotations(self._lang_ids)
+
+    def get_code_point(self, code):
+        """
+        Doctests:
+        >>> ud = UnicodeData()
+        >>> lang_id = "en_US"
+        >>> ud.set_language_ids([lang_id])
+        >>> ud.get_annotation_for_sequence("🤦", lang_id) #doctest: +ELLIPSIS
+        <...
+        """
+        a = self._code_points_index
+        i = bisect_left(a, code)
+        if i != len(a) and a[i] == code:
+            return self._code_points[i]
+        return None
+
+    def get_emojis_with_emoji_default(self):
+        return (self.get_code_point(code) for code in self._emoji_default)
+
+    def get_annotation_for_sequence(self, code_sequence, lang_id):
+        """
+        Doctests:
+        >>> ud = UnicodeData()
+        >>> str(ud.get_code_point(ord('👩')))
+        "UnicodeData(128105,'WOMAN','')"
+        """
+        annotations = self._annotations.get(lang_id)
+        if annotations:
+            return annotations.get(code_sequence)
+        return None
+
+    def _load_annotations(self, lang_ids):
+        self._annotations.clear()
+
+        for lang_id in lang_ids:
+            for base_name in self._get_cldr_locale_base_names(lang_id):
+                path = self._cldr_path('common/annotations',
+                                       base_name + ".xml")
+
+                if os.path.exists(path):
+                    # {code_sequence : Annotation()}
+                    annotations = self._annotations.setdefault(lang_id,
+                                                               OrderedDict())
+
+                    self._load_annotation_file(path, annotations)
+
+    def _load_annotation_file(self, path, annotations_out):
+        with self._parse_xml(path) as dom:
+            for node in dom.getElementsByTagName("annotation"):
+                text = "".join([n.data for n in node.childNodes
+                                if n.nodeType == n.TEXT_NODE])
+                cp = self._get_attribute(node, 'cp', "-1")
+
+                a = annotations_out.setdefault(cp, self.Annotation())
+                a.code_sequence = cp
+                if self._get_attribute(node, 'type', "") == 'tts':
+                    a.annotation_tts = text
+                else:
+                    a.annotations = tuple(s.strip()
+                                          for s in text.split("|"))
+
+    def _get_cldr_locale_base_names(self, lang_id):
+        """
+        Doctests:
+        >>> ud = UnicodeData()
+        >>> ud._get_cldr_locale_base_names('en_DE')
+        ['en', 'en_001', 'en_150', 'en_DE']
+        >>> ud._get_cldr_locale_base_names('de_DE')
+        ['de', 'de_DE']
+        """
+        parent_locales = {}
+        path = self._cldr_path('common/supplemental', 'supplementalData.xml')
+        with self._parse_xml(path) as dom:
+            for node in dom.getElementsByTagName("parentLocale"):
+                parent = node.attributes["parent"].value
+                locales = node.attributes["locales"].value
+                locale_ids = locales.split()
+                for lid in locale_ids:
+                    parent_locales[lid] = parent
+
+        # Find all annotation files we have to load for this
+        # particular lang_id. There can be multiple parent locales,
+        # e.g. en_DE -> en_150 -> en_001, then en.
+        candidates = []   # annotations files to load, in root to child order
+        candidates.append(lang_id)
+        lid = lang_id
+        while True:
+            lid = parent_locales.get(lid)
+            if not lid:
+                break
+            candidates.insert(0, lid)
+
+        lang_code, country_code = self.split_lang_id(lang_id)
+        if lang_code not in candidates:
+            candidates.insert(0, lang_code)
+
+        return candidates
+
+    @contextmanager
+    def _parse_xml(self, path):
+        with open(path, "r", encoding="UTF-8") as f:
+            with minidom.parse(f).documentElement as dom:
+                yield dom
+
+    @staticmethod
+    def _get_attribute(node, attribute, default):
+        attr = node.attributes.get(attribute)
+        return attr.value if attr else default
+
+    @staticmethod
+    def split_lang_id(lang_id):
+        tokens = lang_id.split("_")
+        lang_code    = tokens[0] if len(tokens) >= 1 else ""
+        country_code = tokens[1] if len(tokens) >= 2 else ""
+        return lang_code, country_code
+
+    def gen_unicode_data(self):
+        """
+        Download UNICODE tables and generate data files to include in
+        the project.
+
+        Note: this is a build-time step, and even then, this has to
+        be repeated only when updated unicode tables are released.
+        """
+        # block names, English
+        lines = self._read_cached_http(
+            'http://www.unicode.org/Public/UNIDATA/Blocks.txt',
+            'UNIDATA', 'Blocks.txt')
+        for line in lines:
+            if line:
+                line = line.split("#")[0].strip()
+
+        # code points
+        lines = self._read_cached_http(
+            'http://www.unicode.org/Public/UNIDATA/UnicodeData.txt',
+            'UNIDATA', 'UnicodeData.txt')
+        for line in lines:
+            if line:
+                line = line.split("#")[0].strip()
+                if line:
+                    fields = line.split(";")
+                    (code_value,
+                     general_category,
+                     canonical_Combining_classes,
+                     bidirectional_category,
+                     character_decomposition_mapping,
+                     decimal_digit_value,
+                     digit_value,
+                     numeric_value,
+                     mirrored_bidi,
+                     unicode_1_0_name,
+                     iso_10646_comment_field,
+                     uppercase_mapping,
+                     lowercase_mapping,
+                     titlecase_mapping,
+                     unknown
+                     ) = fields
+
+                    cp = self.CodePoint()
+                    cp.code = code_value
+                    cp.category = general_category
+                    cp.comment = iso_10646_comment_field
+                    self._code_points.append(cp)
+                    self._code_points_index.append(code_value)
+
+        # emoji-data.txt knows which characters are:
+        # - emoji (Emoji),
+        # - presentation emoji (Emoji_Presentation)
+        # - emoji modifiers (Emoji_Modifier_Base).
+        lines = self._read_cached_http(
+            'http://unicode.org/Public/emoji/3.0/emoji-data.txt',
+            'emoji', 'emoji-data.txt')
+        for line in lines:
+            if line:
+                line = line.split("#")[0].strip()
+                if line:
+                    fields = [c.strip() for c in line.split(";")]
+                    if len(fields) >= 2:
+                        code_point_range = re.split('\.\.', fields[0])
+                        code_point_range = list(int(e, 16)
+                                                for e in code_point_range)
+                        flag = fields[1].lower()
+
+                        s = None
+                        if flag == "Emoji".lower():
+                            s = self._emoji_default
+                        elif flag == "Emoji_Presentation".lower():
+                            s = self._text_default
+                        elif flag == "Emoji_Modifier_Base".lower():
+                            s = self._emoji_with_modifier
+
+                        if s is not None:
+                            if len(code_point_range) == 2:
+                                for i in range(code_point_range[0],
+                                               code_point_range[1] + 1):
+                                    s.add(i)
+                            elif len(code_point_range) == 1:
+                                s.add(code_point_range[0])
+
+    def _read_cached_http(self, url, subdir, fn):
+        lines = []
+        path = self._get_http_file(url, subdir, fn)
+        with open(path, "r", encoding="UTF-8") as f:
+            lines = f.readlines()
+        return lines
+
+    def _get_http_file(self, url, subdir, fn):
+        dir_ = self._data_path(subdir)
+        path = os.path.join(dir_, fn)
+
+        if not os.path.exists(dir_):
+            os.makedirs(dir_)
+
+        if not os.path.exists(path):
+
+            print("Downloading '{}'... ".format(url))
+            sys.stdout.flush()
+
+            response = urlopen(url)
+            data = response.read()
+            text = data.decode('utf-8')
+            with open(path, "w", encoding="UTF-8") as f:
+                f.write(text)
+
+            print("   saved as '{}'".format(path))
+
+        return path
+
+    def _cldr_path(self, subdir, fn):
+        """ path of CLDR directory """
+        cldr_subdir = os.path.join('CLDR', subdir)
+        dir_ = self._data_path(cldr_subdir)
+        return os.path.join(dir_, fn)
+
+    def _data_path(self, fn):
+        """ path of unicode_data directory """
+        return os.path.join(self.UNICODE_DATA_PATH, fn)
+
+    def gen_emoji_output(self):
+        # categories: [category label, starting codepoint, comment]
+        categories = [
+            ["😀", "😀", "Smileys", False],
+            ["👦", "👦", "People", False],
+            ["❤", "💋", "Emotion", False],
+            ["🐱", "🐵", "Animals & Nature", False],
+            ["🍒", "🍇", "Food & Drink", False],
+            ["🏛", "🌍", "Travel & Places", False],
+            ["⚽", "🎃", "Activities", False],
+            ["🔔", "🔇", "Objects", False],
+            ["🔶", "🏧", "Symbols", False],
+            ["🏁", "🏁", "Flags", False],
+        ]
+
+        print("#")
+        print("# Generated for Onboard by " + os.path.basename(__file__))
+        print("#")
+        print()
+        print("emoji_data = [")
+
+        emoji_data = self._read_emoji_data().items()
+        comment_row = 50
+
+        for codepoints, data in emoji_data:
+            alternatives, comment = data
+
+            # new category?
+            new_category_index = -1
+            for i, category in enumerate(categories):
+                if len(codepoints) == 1 and \
+                   codepoints[0] == ord(category[1]):
+                    new_category_index = i
+                    category[3] = True  # mark as found, for later checks
+                    break
+
+            # category header
+            if new_category_index > 0:
+                print(" " * 8 + "]],")
+
+            if new_category_index >= 0:
+                category = categories[new_category_index]
+                clabel = category[0]
+                ccomment = category[2]
+
+                line = " " * 4 + "[" + repr(clabel) + ", "
+                line = line.ljust(comment_row) + \
+                    "# category: " + ccomment
+                print(line)
+                print(" " * 8 + "[")
+
+                new_category_index = -1
+
+            # main emoji
+            line = " " * 12 + "(" + \
+                repr("".join([chr(cp) for cp in codepoints])) + ", " + \
+                ("None), " if not alternatives else "")
+            line = line.ljust(comment_row) + "# " + comment
+            print(line)
+
+            # skin tones for the long-press popup
+            for i, (acodepoints, acomment) in enumerate(alternatives):
+                line = " " * 16 + \
+                    ("(" if i == 0 else " ") +\
+                    repr("".join([chr(cp) for cp in acodepoints])) + \
+                    (")), " if i == len(alternatives) - 1 else ", ")
+                line = line.ljust(comment_row) + "# " + acomment
+                print(line)
+
+        print(" " * 8 + "]],")
+        print("]")
+
+        # plausibility check: have all categories been found?
+        if not all(category[3] for category in categories):
+            print()
+            print("Warning: emoji categories were not all used")
+            for category in categories:
+                print(" " * 4 + str(category))
+
+    def _read_emoji_data(self):
+        lines = self._read_cached_http(
+            'http://unicode.org/emoji/charts/emoji-ordering.txt',
+            'emoji', 'emoji-ordering.txt')
+
+        skincolors = [0x1F3FB,
+                      0x1F3FC,
+                      0x1F3FD,
+                      0x1F3FE,
+                      0x1F3FF,
+                      ]
+
+        last_noskin_cps = ()
+        emoji_data = OrderedDict()
+
+        for line in lines:
+            if line:
+                k = line.find("#")
+                data = line[:k].strip()
+                comment = line[k + 1:].strip()
+                if data:
+                    fields = [c.strip() for c in data.split(";")]
+                    codepoints = tuple(int(c.strip().replace("U+", "0x"), 16)
+                                       for c in fields[0].split())
+
+                    # Emoji modified with skin color get hidden in
+                    # long-press popups.
+                    # The emoji of the popup is the immediate predecessor of
+                    # continues skin color modifier sequences.
+                    noskincps = tuple(cp for cp in codepoints
+                                      if cp not in skincolors)
+                    if codepoints == noskincps:
+                        last_noskin_cps = codepoints
+                        emoji_data[codepoints] = [[], comment]
+
+                    elif not noskincps:  # just a skin modifier alone?
+                        print("dropping emoji: skin modifier", repr(comment),
+                              file=sys.stderr)
+
+                    else:
+                        # print(codepoints, comment, last_noskin_comment)
+                        parent = emoji_data.get(last_noskin_cps)
+                        if parent:
+                            parent[0].append([codepoints, comment])
+                        else:
+                            print("dropping emoji: no parent for",
+                                  repr(comment), file=sys.stderr)
+
+        return emoji_data
+
+
+def get_system_default_lang_id(self):
+    lang_id = locale.getdefaultlocale()[0]
+    if not lang_id:  # None e.g. with LANG=C
+        lang_id = "en_US"
+    return lang_id
+
+
+if __name__ == "__main__":
+    ud = UnicodeData()
+    ud.gen_emoji_output()
+
+    exit()
+
+    # Download stuff now (first time), so the output doesn't
+    # interfere with doctests.
+    UnicodeData()
+
+    import doctest
+    doctest.testmod()
+    if 1:
+        ud = UnicodeData()
+        ud.set_language_ids(["en_US"])
+
+        if 0:
+            cps = ud.get_emoji_sequences()
+            cps = ud.get_emojis_with_emoji_default()
+            for cp in cps:
+                print(cp.code, cp.category, cp.comment,
+                      cp.annotations, cp.annotation_tts)
+
+

